/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/

package at.jku.mevss.anttracks.callcontext;

/**
 * Exception thrown to indicate that a class definition could not be successfully parsed from a data buffer.
 *
 * @author Peter Feichtinger
 */
public class BadClassDefinitionException extends RuntimeException {

    private static final long serialVersionUID = -6630507179906532418L;

    private final byte[] mData;

    /**
     * Constructs a new {@link BadClassDefinitionException} with the specified data buffer, detail message, and cause.
     * <p>
     * Note that the detail message associated with {@code cause} is <i>not</i> automatically incorporated in this exception's detail
     * message.
     *
     * @param data    The class definition data that caused the exception to be thrown, may be {@code null}.
     * @param message The detail message (which is saved for later retrieval by {@link #getMessage()}).
     * @param cause   The cause (which is saved for later retrieval by {@link #getCause()}). (A {@code null} value is permitted, and
     *                indicates
     *                that the cause is nonexistent or unknown.)
     */
    public BadClassDefinitionException(byte[] data, String message, Throwable cause) {
        super(message, cause);
        mData = clone(data);
    }

    /**
     * Constructs a new {@link BadClassDefinitionException} with the specified data buffer and detail message. The cause is not initialized,
     * and may subsequently be initialized by a call to {@link #initCause}.
     *
     * @param data    The class definition data that caused the exception to be thrown, may be {@code null}.
     * @param message The detail message. The detail message is saved for later retrieval by the {@link #getMessage()} method.
     */
    public BadClassDefinitionException(byte[] data, String message) {
        super(message);
        mData = clone(data);
    }

    /**
     * Constructs a new {@link BadClassDefinitionException} with the specified data buffer, cause, and a detail message of
     * {@code (cause==null ? null : cause.toString())} (which typically contains the class and detail message of {@code cause}). This
     * constructor is useful for exceptions that are little more than wrappers for other throwables.
     *
     * @param data  The class definition data that caused the exception to be thrown, may be {@code null}.
     * @param cause The cause (which is saved for later retrieval by the {@link #getCause()} method). (A {@code null} value is permitted,
     *              and
     *              indicates that the cause is nonexistent or unknown.)
     */
    public BadClassDefinitionException(byte[] data, Throwable cause) {
        super("Failed to create class from byte array.", cause);
        mData = clone(data);
    }

    /**
     * Get the class file data that caused this exception to be thrown. Note that the data array is returned by reference, so it should not
     * be changed unless the original data is not needed any more.
     *
     * @return The class file data by reference, or {@code null} if none was set.
     */
    public byte[] getData() {
        return mData;
    }

    /**
     * Clone the specified byte array.
     *
     * @param data The array to copy, or {@code null}.
     * @return A copy of {@code data}, or {@code null} if {@code data} is {@code null}.
     */
    private static byte[] clone(byte[] data) {
        return (data != null ? data.clone() : null);
    }
}
