/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/

package at.jku.ssw.ftcm;

import at.jku.ssw.ftcm.preferences.Feature;
import at.jku.ssw.ftcm.preferences.ProjectSettingsManager;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

/**
 * This class handles the selection process of a feature during the assignment
 * process. The dialog shows a combo box of defined features of which the user
 * can choose one.
 *
 * @author David Pfeffer
 */
public class FeatureSelectDialog extends Dialog {

    // Access to the project settings where the features are stored
    private ProjectSettingsManager psm;

    // The selected index in the combo box
    private int selectionIndex;

    /**
     * Creates a new selection dialog.
     *
     * @param parentShell The parent shell
     * @param res         The resource where the marker will be added
     */
    public FeatureSelectDialog(Shell parentShell, IResource res) {
        super(parentShell);
        this.psm = ProjectSettingsManager.getInstance(res.getProject());
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets
     * .Shell)
     */
    @Override
    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText("Selection dialog");
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets
     * .Composite)
     */
    @Override
    protected Control createDialogArea(Composite parent) {

        // Create a container with layout to put our elements into
        final Composite container = (Composite) super.createDialogArea(parent);
        container.setLayout(new GridLayout(2, false));

        // A label for the user
        final Label label = new Label(container, SWT.NONE);
        label.setText("Choose feature");

        final Combo comboFeature = new Combo(container, SWT.READ_ONLY | SWT.DROP_DOWN);

        // Save the selection index because the combobox will be cleaned up on
        // dialog close
        comboFeature.addListener(SWT.Selection, new Listener() {

            @Override
            public void handleEvent(Event event) {
                selectionIndex = ((Combo) event.widget).getSelectionIndex();
            }
        });

        // Fill the combobox box with elements
        for (Feature feature : psm.getFeatureList()) {
            String s = String.valueOf(feature.getId() + ": " + feature.getDescription());
            comboFeature.add(s);
            comboFeature.setData(s, feature);
        }

        comboFeature.select(0);
        container.pack();

        return container;
    }

    /**
     * Returns the selected feature
     *
     * @return The selected feature
     */
    public Feature getFeature() {
        return psm.getFeatureList().get(selectionIndex);
    }
}
