/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/

package at.jku.ssw.ftcm.handler;

import at.jku.ssw.ftcm.Activator;
import at.jku.ssw.ftcm.FeatureSelectDialog;
import at.jku.ssw.marker.FeatureMarkerFactory;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.ui.handlers.HandlerUtil;

/**
 * This class handles the assignment of a feature to a statement element.
 *
 * @author David Pfeffer
 */
public class StatementFeatureAssignHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        try {
            // Get the selection and check if it is valid
            TextSelection selection = FeatureMarkerFactory.getTextSelection();
            if (selection == null || selection.getLength() == 0) {
                new MessageDialog(HandlerUtil.getActiveShell(event),
                                  "No selection taken",
                                  null,
                                  "There was no selection taken which could be assigned to a feature.",
                                  MessageDialog.ERROR,
                                  new String[]{"Ok"},
                                  0).open();
                return null;
            }

            // Grab the file and ask the user which feature to assign
            IFile file = (IFile) Activator.getEditor().getEditorInput().getAdapter(IFile.class);
            FeatureSelectDialog fsd = new FeatureSelectDialog(Activator.getShell(), file);
            if (fsd.open() == Dialog.OK && fsd.getFeature() != null) {
                IDocument doc = Activator.getEditor().getDocumentProvider().getDocument(Activator.getEditor().getEditorInput());
                FeatureMarkerFactory.createStatementMarker(file, doc, selection.getStartLine(), selection.getEndLine(), fsd.getFeature());
            }
        } catch (CoreException e) {
            e.printStackTrace();
        }

        return null;
    }
}