/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/

package at.jku.ssw.marker;

import at.jku.ssw.ftcm.Activator;
import at.jku.ssw.ftcm.preferences.Feature;
import at.jku.ssw.ftcm.preferences.ProjectSettingsManager;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.AnnotationPainter.IDrawingStrategy;
import org.eclipse.jface.text.source.AnnotationPainter.ITextStyleStrategy;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.ui.texteditor.SimpleMarkerAnnotation;

public class FeatureDrawingStrategy implements IDrawingStrategy, ITextStyleStrategy {

    @Override
    public void draw(Annotation annotation, GC gc, StyledText textWidget, int offset, int length, Color color) {

        if (length != 0) {
            if (gc != null) {
                try {
                    // Determine the bounds rectangle of the text in the
                    // annotation
                    Rectangle bounds = textWidget.getTextBounds(offset, offset + length - 1);

                    // Check if we have a feature marker
                    if (annotation instanceof SimpleMarkerAnnotation && ((SimpleMarkerAnnotation) annotation).getMarker().isSubtypeOf(FeatureMarkerFactory.FEATURE_MARKER)) {
                        SimpleMarkerAnnotation sma = (SimpleMarkerAnnotation) annotation;

                        // Go over all features in the project and look for
                        // matching entries
                        for (Feature f : ProjectSettingsManager.getInstance(sma.getMarker().getResource().getProject()).getFeatureList()) {
                            Color c = new Color(Activator.getShell().getDisplay(), f.getR(), f.getG(), f.getB());
                            if (f.getId() == sma.getMarker().getAttribute("feature", 0)) {
                                // If the marker was already processed by the
                                // Feature Builder we fill the background, if
                                // not we only draw a surrounding rectangle
                                if (sma.getMarker().getAttribute("built", false)) {
                                    gc.setBackground(c);
                                    gc.setAlpha(65);
                                    gc.fillRectangle(bounds);
                                } else {
                                    gc.setForeground(c);
                                    gc.drawRectangle(bounds);
                                }
                            }
                        }
                    } else {
                        // For not recognized annotations we provide the typical
                        // box strategy
                        gc.setForeground(color);
                        gc.drawRectangle(bounds);
                    }
                } catch (CoreException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            } else {
                textWidget.redrawRange(offset, length, true);
            }
        }

    }

    @Override
    public void applyTextStyle(StyleRange styleRange, Color annotationColor) {
        // We cannot use the text style approach. This methods provides us only
        // the annotation color and we do not have access to the whole
        // annotation object.
    }

}
