/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/

package at.jku.ssw.marker;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.Position;
import org.eclipse.ui.texteditor.IMarkerUpdater;

public class MarkerUpdater implements IMarkerUpdater {

    @Override
    public String getMarkerType() {
        return FeatureMarkerFactory.FEATURE_STATEMENT_MARKER;
    }

    @Override
    public String[] getAttribute() {
        return null;
    }

    @Override
    public boolean updateMarker(IMarker marker, IDocument doc, Position position) {
        // System.err.println("updateMarker");

        try {
            int start = position.getOffset();
            int end = position.getOffset() + position.getLength();

            if (position.getLength() == 0) {
                marker.delete();
                return true;
            }

            marker.setAttribute(IMarker.CHAR_START, start);
            marker.setAttribute(IMarker.CHAR_END, end);

            marker.setAttribute(IMarker.LINE_NUMBER, doc.getLineOfOffset(start) + 1);
            marker.setAttribute("lines", doc.getNumberOfLines(position.getOffset(), position.getLength()));

            return true;
        } catch (CoreException | BadLocationException e) {
            return false;
        }
    }

}
