/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.classification;

import java.util.Arrays;

/**
 * This class represents a classifier that is only applied on a given set of objects, identified by
 * {@link ConditionalClassifier#getClassifier()}
 *
 * @author Markus Weninger
 */
public abstract class ConditionalClassifier {

    /**
     * The classifiers that should be applied at the tree node defined by {@link ConditionalClassifier#getConditions()}
     *
     * @return The classifier
     */
    public abstract ClassifierChain getClassifiers();

    /**
     * Defines the node on which the classifier should be applied as an {@link Object[]}. {@code getConditions()[0]} represents the root
     * object, {@code getConditions()[1]} the key on level 1, and so on.
     *
     * @return The object array defining the node on which this classifier should be applied
     */
    public abstract Object[] getConditions();

    /**
     * Gets the classifier's name
     *
     * @return The classifier's name
     */
    public String getName() {
        return getClassifiers() + " applied @ " + Arrays.toString(getConditions());
    }

    @Override
    public String toString() {
        return getName();
    }
}
