/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.classification.enumerations;

import at.jku.mevss.anttracks.classification.annotations.Classifier;

/**
 * This enum is used by the {@link Classifier} annotation to define its behaviour.
 *
 * @author Markus Weninger
 */
public enum ClassifierType {

    /**
     * A classifier of this type identifies a heap object with a single identifier. Example: A heap object can only be allocation by a
     * single subsystem -> VM, Interpreter, C1 or C2.
     */
    ONE("One"),
    /**
     * A classifier of this type may identify a heap object by multiple identifies. Example: A heap object may be assigned to multiple
     * features (feature 1 & feature 4). The identifier of such a classifier is assumed to be an array.
     */
    MANY("Many"),
    /**
     * A classifier of this type identifies a heap object not by a single identifier, but by a sequence of multiple identifiers. Example: A
     * heap object may be identified by its stack trace (A calls B, B calls C -> Identifier = [A, B, C]). The identifier of such a
     * classifier is assumed to be an array.
     */
    HIERARCHY("Hierarchy"),
    /**
     * A classifier of this type may identify a heap by multiple sequences of identifiers. Example: A heap object may be identified by the
     * path to all his roots (Root1 -> A, Root2 -> B -> A, Root3 -> C -> B -> A, thus the identifier of A would be:
     * [ [Root1]; [B, Root2]; [C, B, Root3] ]). The identifier of such a classifier is assumed to be an array of arrays.
     */
    MANY_HIERARCHY("Many hierarchy");

    private String text;

    ClassifierType(String text) {
        this.text = text;
    }

    public String getText() {
        return text;
    }
}
