/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.classification.nodes;

import at.jku.mevss.anttracks.heap.objects.ObjectInfo;
import at.jku.mevss.anttracks.heap.roots.RootPtr;
import at.jku.mevss.anttracks.heap.space.SpaceInfo;
import at.jku.mevss.anttracks.heap.symbols.AllocatedType;
import at.jku.mevss.anttracks.heap.symbols.AllocationSite;
import at.jku.mevss.anttracks.parser.EventType;

import java.util.List;

public class GroupingException extends Exception {

    private static final long serialVersionUID = 8795850048980974276L;

    public final ObjectInfo objectInfo;
    public final long address;
    public final SpaceInfo space;
    public final AllocatedType type;
    public final int size;
    public final boolean isArray;
    public final int arrayLength;
    public final AllocationSite allocationSite;
    public final EventType eventType;
    public final List<RootPtr> rootPtrs;

    public GroupingException(long address,
                             ObjectInfo objectInfo,
                             SpaceInfo space,
                             AllocatedType type,
                             int size,
                             boolean isArray,
                             int arrayLength,
                             AllocationSite allocationSite,
                             EventType eventType,
                             List<RootPtr> rootPtrs,
                             Exception ex) {
        super(String.format("%d\t%s\t%s\t%d\t%s\t%d\t%s\t%s\t%s - Cause: %s",
                            address,
                            space.toString(),
                            type.toString(),
                            size,
                            isArray ? "array" : "non-array",
                            arrayLength,
                            allocationSite.callSites[0].toString(),
                            eventType.toString(),
                            rootPtrs != null ? rootPtrs.stream().map(rPtr -> rPtr.toString()).reduce((a, b) -> a + ", " + b).get() : "no root",
                            ex.getCause()), ex);
        this.objectInfo = objectInfo;
        this.address = address;
        this.space = space;
        this.type = type;
        this.size = size;
        this.isArray = isArray;
        this.arrayLength = arrayLength;
        this.allocationSite = allocationSite;
        this.eventType = eventType;
        this.rootPtrs = rootPtrs;
    }
}
