/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.compiler;

import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;

public class OnTheFlyCompilationClassLoader extends ClassLoader {

    /**
     * ClassName -> Byte code
     */
    Map<String, ByteCode> onTheFlyCompiledCode = new HashMap<>();

    public OnTheFlyCompilationClassLoader(ClassLoader parent) {
        super(parent);
    }

    public ByteCode getByteCodeStub(String className) {
        ByteCode empty;
        try {
            empty = new ByteCode(className);

            onTheFlyCompiledCode.put(className, empty);
            return empty;
        } catch (URISyntaxException e) {
            e.printStackTrace();
            return null;
        }
    }

    @Override
    protected Class<?> findClass(String name) throws ClassNotFoundException {
        ByteCode bc = onTheFlyCompiledCode.get(name);
        if (bc == null) {
            return super.findClass(name);
        }
        byte[] byteCode = bc.get();
        return defineClass(name, byteCode, 0, byteCode.length);
    }
}
