/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.compiler;

import javax.tools.*;
import java.io.IOException;
import java.io.Writer;
import java.net.URISyntaxException;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class OnTheFlyCompiler {
    static JavaCompiler javac = ToolProvider.getSystemJavaCompiler();

    public String extractPackage(String sourceCode) {
        Pattern classNamePattern = Pattern.compile("package\\s+([a-zA_Z_][\\.\\w]*);");
        Matcher classNameMatcher = classNamePattern.matcher(sourceCode);
        if (!classNameMatcher.find()) {
            return null;
        }
        return classNameMatcher.group(1);
    }

    public String extractClassName(String sourceCode) {
        Pattern classNamePattern = Pattern.compile("class\\s+(\\w*)");
        Matcher classNameMatcher = classNamePattern.matcher(sourceCode);
        if (!classNameMatcher.find()) {
            return null;
        }
        return classNameMatcher.group(1);
    }

    public Class<?> compile(String sourceCode) throws IllegalArgumentException {
        // Extract class name
        String packageName = extractPackage(sourceCode);
        if (packageName == null) {
            packageName = "";
        }
        String className = extractClassName(sourceCode);
        if (className == null) {
            throw new IllegalArgumentException("Source code does not contain a class definition");
        }
        if (packageName.length() > 0) {
            className = packageName + "." + className;
        }

        Source source;
        try {
            source = new Source(className, sourceCode);
        } catch (URISyntaxException e) {
            return null;
        }

        StringBuilder errorMessage = new StringBuilder();
        JavaFileManager fileManager = new OnTheFlyCompilationFileManager();
        DiagnosticListener<? super JavaFileObject> diagnosticListener = null;
        List<String> options = null;
        List<String> classes = null;
        // May throw RuntimeException on error
        JavaCompiler.CompilationTask task = javac.getTask(new Writer() {

            @Override
            public void write(char[] cbuf, int off, int len) throws IOException {
                for (int i = 0; i < len; i++) {
                    errorMessage.append(cbuf[i + off]);
                }
            }

            @Override
            public void flush() throws IOException {
                // TODO Auto-generated method stub
            }

            @Override
            public void close() throws IOException {
                // TODO Auto-generated method stub
            }

        }, fileManager, diagnosticListener, options, classes, source.singletonList());
        boolean result = task.call();
        if (result == false) {
            throw new IllegalArgumentException(errorMessage.toString());
        }
        try {
            return fileManager.getClassLoader(null).loadClass(className);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
            return null;
        }
    }
}
