/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/

package at.jku.mevss.anttracks.diff;

import at.jku.mevss.anttracks.heap.DetailedHeap;
import at.jku.mevss.anttracks.heap.objects.ObjectInfo;
import at.jku.mevss.anttracks.heap.space.SpaceType;
import at.jku.mevss.anttracks.heap.symbols.Symbols;
import at.jku.mevss.anttracks.parser.EventType;
import at.jku.mevss.anttracks.parser.ParsingInfo;
import at.jku.mevss.anttracks.parser.ThreadLocalHeap;
import at.jku.mevss.anttracks.parser.heap.HeapEventHandler;
import at.jku.mevss.anttracks.util.TraceException;

import java.util.List;

public class HeapDiffEventHandler extends HeapEventHandler {

    private final HeapDiff heapDiff;

    public HeapDiffEventHandler(DetailedHeap heap, HeapDiff heapDiff, Symbols symbols, ParsingInfo parsingInfo) {
        super(heap, symbols, parsingInfo);
        this.heapDiff = heapDiff;
    }

    @Override
    public List<ObjectInfo> doParseGCMoveRegion(EventType type, long fromAddr, long toAddr, SpaceType toSpaceType, int numOfObjects, ThreadLocalHeap threadLocalHeap)
            throws TraceException {
        List<ObjectInfo> movedObjects = super.doParseGCMoveRegion(type, fromAddr, toAddr, toSpaceType, numOfObjects, threadLocalHeap);
        heapDiff.updateMultipleAddresses(fromAddr, toAddr, movedObjects);
        return movedObjects;
    }

    @Override
    public long doGCMove(EventType type, ThreadLocalHeap threadLocalHeap, ObjectInfo obj, long fromAddr, long toAddr, SpaceType toSpaceType) throws TraceException {
        long assignedAddr = super.doGCMove(type, threadLocalHeap, obj, fromAddr, toAddr, toSpaceType);
        heapDiff.updateAddress(fromAddr, assignedAddr);
        return assignedAddr;
    }
}
