/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.features;

import java.util.ArrayList;
import java.util.List;

public class Feature {

    public final String name;
    public final int r, g, b;

    private final List<PackageMapping> packages;
    private final List<TypeMapping> types;
    private final List<MethodMapping> methods;
    private final List<InstructionMapping> instructions;

    public Feature(String name, int r, int g, int b) {
        this.name = name;
        this.r = r;
        this.g = g;
        this.b = b;
        packages = new ArrayList<>(1);
        types = new ArrayList<>(1);
        methods = new ArrayList<>(1);
        instructions = new ArrayList<>(1);
    }

    public List<PackageMapping> getPackages() {
        return packages;
    }

    public List<TypeMapping> getTypes() {
        return types;
    }

    public List<MethodMapping> getMethods() {
        return methods;
    }

    public List<InstructionMapping> getInstructions() {
        return instructions;
    }

    public void addPackage(String pakkage) {
        packages.add(new PackageMapping(pakkage));
    }

    public void addType(String type) {
        types.add(new TypeMapping(type));
    }

    public void addMethod(String type, String method, String signature) {
        methods.add(new MethodMapping(type, method, signature));
    }

    public void addInstructions(String type, String method, String signature, int from, int to) {
        instructions.add(new InstructionMapping(type, method, signature, from, to));
    }

}
