package at.jku.mevss.anttracks.heap;

import at.jku.mevss.anttracks.heap.objects.ObjectInfo;
import at.jku.mevss.anttracks.util.TraceException;

public interface FrontBackObjectAccess extends ObjectAccess {

    public abstract void setObjectPointersInFront(long objAddr, long[] ptrs) throws TraceException;

    public abstract void setObjectForwardingAddrInFront(long objAddr, long forwAddr) throws TraceException;

    public abstract void setObjectForwardingAddrAndPtrsInFront(long objAddr, long forwAddr, long[] ptrs) throws TraceException;

    public abstract void setFromPointerCountInFront(long objAddr, int n) throws TraceException;

    public abstract ObjectInfo getObjectInFront(long objAddr) throws TraceException;

    public abstract long[] getObjectPointersInFront(long objAddr) throws TraceException;

    public abstract long[] getObjectFromPointersInFront(long objAddr) throws TraceException;

    public abstract long getObjectForwardingAddressUnescapedInFront(long objAddr) throws TraceException;

    public abstract long getObjectForwardingAddressEscapedInFront(long objAddr) throws TraceException;

    public abstract void markInFront(long objAddr) throws TraceException;

    public abstract boolean isMarkedInFront(long objAddr) throws TraceException;

    public abstract void setFromPointersInFront(long objAddr, long[] fromPtrs) throws TraceException;

    public abstract boolean ptrNeedAdjustmentInFront(long objAddr) throws TraceException;

    public abstract void setPtrNeedAdjustmentInFront(long objAddr, boolean needAdjustment) throws TraceException;

    public abstract void setObjectPointersInBack(long objAddr, long[] ptrs) throws TraceException;

    public abstract void setObjectForwardingAddrInBack(long objAddr, long forwAddr) throws TraceException;

    public abstract void setObjectForwardingAddrAndPtrsInBack(long objAddr, long forwAddr, long[] ptrs) throws TraceException;

    public abstract void setFromPointerCountInBack(long objAddr, int n) throws TraceException;

    public abstract ObjectInfo getObjectInBack(long objAddr) throws TraceException;

    public abstract long[] getObjectPointersInBack(long objAddr) throws TraceException;

    public abstract long[] getObjectFromPointersInBack(long objAddr) throws TraceException;

    public abstract long getObjectForwardingAddressUnescapedInBack(long objAddr) throws TraceException;

    public abstract long getObjectForwardingAddressEscapedInBack(long objAddr) throws TraceException;

    public abstract void markInBack(long objAddr) throws TraceException;

    public abstract boolean isMarkedInBack(long objAddr) throws TraceException;

    public abstract void setFromPointersInBack(long objAddr, long[] fromPtrs) throws TraceException;

    public abstract boolean ptrNeedAdjustmentInBack(long objAddr) throws TraceException;

    public abstract void setPtrNeedAdjustmentInBack(long objAddr, boolean needAdjustment) throws TraceException;

    default void setObjectPointers(long objAddr, long[] ptrs, boolean inFront) throws TraceException {
        if (inFront) {
            setObjectPointersInFront(objAddr, ptrs);
        } else {
            setObjectPointersInBack(objAddr, ptrs);
        }
    }

    default void setObjectForwardingAddr(long objAddr, long forwAddr, boolean inFront) throws TraceException {
        if (inFront) {
            setObjectForwardingAddrInFront(objAddr, forwAddr);
        } else {
            setObjectForwardingAddrInBack(objAddr, forwAddr);
        }
    }

    default void setObjectForwardingAddrAndPtrs(long objAddr, long forwAddr, long[] ptrs, boolean inFront) throws TraceException {
        if (inFront) {
            setObjectForwardingAddrAndPtrsInFront(objAddr, forwAddr, ptrs);
        } else {
            setObjectForwardingAddrAndPtrsInBack(objAddr, forwAddr, ptrs);
        }
    }

    default void setFromPointerCount(long objAddr, int n, boolean inFront) throws TraceException {
        if (inFront) {
            setFromPointerCountInFront(objAddr, n);
        } else {
            setFromPointerCountInBack(objAddr, n);
        }
    }

    default ObjectInfo getObject(long objAddr, boolean inFront) throws TraceException {
        return inFront ? getObjectInFront(objAddr) : getObjectInBack(objAddr);
    }

    default long[] getObjectPointers(long objAddr, boolean inFront) throws TraceException {
        return inFront ? getObjectPointersInFront(objAddr) : getObjectPointersInBack(objAddr);
    }

    default long getObjectForwardingAddressUnescaped(long objAddr, boolean inFront) throws TraceException {
        return inFront ? getObjectForwardingAddressUnescapedInFront(objAddr) : getObjectForwardingAddressUnescapedInBack(objAddr);
    }

    default long getObjectForwardingAddressEscaped(long objAddr, boolean inFront) throws TraceException {
        return inFront ? getObjectForwardingAddressEscapedInFront(objAddr) : getObjectForwardingAddressEscapedInBack(objAddr);
    }

    default void mark(long objAddr, boolean inFront) throws TraceException {
        if (inFront) {
            markInFront(objAddr);
        } else {
            markInBack(objAddr);
        }
    }

    default boolean isMarked(long objAddr, boolean inFront) throws TraceException {
        return inFront ? isMarkedInFront(objAddr) : isMarkedInBack(objAddr);
    }

    default void setFromPointers(long objAddr, long[] fromPtrs, boolean inFront) throws TraceException {
        if (inFront) {
            setFromPointersInFront(objAddr, fromPtrs);
        } else {
            setFromPointersInBack(objAddr, fromPtrs);
        }
    }

    default boolean ptrNeedAdjustment(long objAddr, boolean inFront) throws TraceException {
        return inFront ? ptrNeedAdjustmentInFront(objAddr) : ptrNeedAdjustmentInBack(objAddr);
    }

    default void setPtrNeedAdjustment(long objAddr, boolean needAdjustment, boolean inFront) throws TraceException {
        if (inFront) {
            setPtrNeedAdjustmentInFront(objAddr, needAdjustment);
        } else {
            setPtrNeedAdjustmentInBack(objAddr, needAdjustment);
        }
    }
}
