/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.heap;

import at.jku.mevss.anttracks.heap.objects.ArrayObjectInfo;
import at.jku.mevss.anttracks.heap.objects.ObjectInfo;
import at.jku.mevss.anttracks.heap.space.SpaceInfo;
import at.jku.mevss.anttracks.heap.symbols.AllocatedType;
import at.jku.mevss.anttracks.heap.symbols.AllocationSite;
import at.jku.mevss.anttracks.parser.EventType;

public class HeapObject {
    public final long address;
    public final SpaceInfo space;
    public final AllocatedType type;
    public final int size;
    public final boolean isArray;
    public final int arrayLength;
    public final AllocationSite allocationSite;
    public final long[] pointedFrom;
    public final long[] pointsTo;
    public final EventType eventType;

    // TODO remove redundant info
    public final ObjectInfo objectInfo;

    public HeapObject(long address,
                      SpaceInfo space,
                      AllocatedType type,
                      int size,
                      boolean isArray,
                      int arrayLength,
                      AllocationSite allocationSite,
                      long[] pointedFrom,
                      long[] pointsTo,
                      EventType eventType,
                      ObjectInfo objectInfo) {
        this.address = address;
        this.space = space;
        this.type = type;
        this.size = size;
        this.isArray = isArray;
        this.arrayLength = arrayLength;
        this.allocationSite = allocationSite;
        this.pointedFrom = pointedFrom;
        this.pointsTo = pointsTo;
        this.eventType = eventType;

        this.objectInfo = objectInfo;
    }

    public HeapObject(long addr, SpaceInfo space, long[] pointedFrom, long[] pointsTo, ObjectInfo eObj) {
        this(addr,
             space,
             eObj.type,
             eObj.size,
             eObj instanceof ArrayObjectInfo,
             (eObj instanceof ArrayObjectInfo) ? ((ArrayObjectInfo) eObj).getLength() : 0,
             eObj.allocationSite,
             pointedFrom,
             pointsTo,
             eObj.eventType,
             eObj);
    }
}
