package at.jku.mevss.anttracks.heap.datastructures.cocor;

import at.jku.mevss.anttracks.heap.datastructures.DataStructureDefinitions;

import java.util.List;
import java.util.stream.Collectors;

public class Parser {
    public static final int _EOF = 0;
    public static final int _ident = 1;
    public static final int _wildcard = 2;
    public static final int _number = 3;
    public static final int _float = 4;
    public static final int _string = 5;
    public static final int maxT = 25;

    static final boolean _T = true;
    static final boolean _x = false;
    static final int minErrDist = 2;

    public Token t;    // last recognized token
    public Token la;   // lookahead token
    int errDist = minErrDist;

    public Scanner scanner;
    public Errors errors;

    private DataStructureDefinitions definitions = new DataStructureDefinitions();

    public DataStructureDefinitions getDataStructureDefinitions() { return definitions; }

    public List<DataStructurePart> getAllDataStructureParts() { return definitions.parts; }

    public List<DataStructurePart> getHeadDataStructureParts() { return definitions.parts.stream().filter(dsp -> dsp.isHead()).collect(Collectors.toList()); }

    public List<DataStructurePart> getInternalDataStructureParts() { return definitions.parts.stream().filter(dsp -> dsp.isInternal()).collect(Collectors.toList()); }

    private void addPointsToType(DataStructurePart dsPart, PointsToType ptt) {
        // This only works if the array type is not wildcarded, find some other way
    /*if(ptt.getName().endsWith("[]")) {
      if(!definitions.stream().anyMatch(dsp -> dsp.getType().equals(ptt.getName()))) {
        DataStructurePart arrayType = new DataStructurePart(ptt.getName(), false);
        arrayType.addPointsToType(new PointsToType(ptt.getName().substring(0, ptt.getName().length() - 2), true));
        definitions.addDataStructurePart(arrayType);
      }
    }*/
        dsPart.addPointsToType(ptt);
    }

    public Parser(Scanner scanner) {
        this.scanner = scanner;
        errors = new Errors();
    }

    void SynErr(int n) {
        if (errDist >= minErrDist) { errors.SynErr(la.line, la.col, n); }
        errDist = 0;
    }

    public void SemErr(String msg) {
        if (errDist >= minErrDist) { errors.SemErr(t.line, t.col, msg); }
        errDist = 0;
    }

    void Get() {
        for (; ; ) {
            t = la;
            la = scanner.Scan();
            if (la.kind <= maxT) {
                ++errDist;
                break;
            }

            la = t;
        }
    }

    void Expect(int n) {
        if (la.kind == n) { Get(); } else { SynErr(n); }
    }

    boolean StartOf(int s) {
        return set[s][la.kind];
    }

    void ExpectWeak(int n, int follow) {
        if (la.kind == n) { Get(); } else {
            SynErr(n);
            while (!StartOf(follow)) { Get(); }
        }
    }

    boolean WeakSeparator(int n, int syFol, int repFol) {
        int kind = la.kind;
        if (kind == n) {
            Get();
            return true;
        } else if (StartOf(repFol)) { return false; } else {
            SynErr(n);
            while (!(set[syFol][kind] || set[repFol][kind] || set[0][kind])) {
                Get();
                kind = la.kind;
            }
            return StartOf(syFol);
        }
    }

    void DataStructures() {
        while (StartOf(1)) {
            if (la.kind == 6) {
                NS(null);
            } else {
                DataStructurePart dsPart = DS(null);
                definitions.addDataStructurePart(dsPart);
            }
        }
    }

    void NS(String outerNamespace) {
        Expect(6);
        Expect(1);
        String namespace = (outerNamespace != null ? outerNamespace + "." : "") + t.val;
        Expect(7);
        while (StartOf(1)) {
            if (la.kind == 6) {
                NS(namespace);
            } else {
                DataStructurePart dsPart = DS(namespace);
                definitions.addDataStructurePart(dsPart);
            }
        }
        Expect(8);
    }

    DataStructurePart DS(String namespace) {
        DataStructurePart d;
        boolean includeSupertypes = false;
        boolean isHead = false;
        if (StartOf(2)) {
            switch (la.kind) {
                case 9: {
                    Get();
                    break;
                }
                case 10: {
                    Get();
                    break;
                }
                case 11: {
                    Get();
                    break;
                }
                case 12: {
                    Get();
                    break;
                }
                case 13: {
                    Get();
                    Expect(14);
                    break;
                }
                case 15: {
                    Get();
                    break;
                }
            }
            isHead = true;
        }
        String name = ReferenceType();
        d = new DataStructurePart(namespace != null ? namespace + "." + name : name, isHead);
        Expect(7);
        while (la.kind == 1 || la.kind == 2 || la.kind == 16) {
            if (la.kind == 1 || la.kind == 2) {
                name = ReferenceType();
                addPointsToType(d, new PointsToType(name, true));
                if (namespace != null && !name.contains("*") && !name.contains(".")) { addPointsToType(d, new PointsToType(namespace + "." + name, true)); }
            } else {
                Get();
                name = ReferenceType();
                Expect(17);
                addPointsToType(d, new PointsToType(name, false));
                if (namespace != null && !name.contains("*") && !name.contains(".")) { addPointsToType(d, new PointsToType(namespace + "." + name, false)); }
            }
            Expect(18);
        }
        Expect(8);
        return d;
    }

    String ReferenceType() {
        String name;
        name = "";
        String x = FullyQualified();
        name += x;
        if (la.kind == 19) {
            Get();
            x = FullyQualified();
            name += "<" + x + ">";
            Expect(20);
        }
        while (la.kind == 21) {
            Get();
            Expect(22);
            name += "[]";
        }
        return name;
    }

    String FullyQualified() {
        String name;
        name = "";
        String x = IdentWithWildCard();
        name += x;
        while (la.kind == 23) {
            Get();
            name += ".";
            x = IdentWithWildCard();
            name += x;
        }
        while (la.kind == 24) {
            Get();
            name += "$";
            x = IdentWithWildCard();
            name += x;
        }
        return name;
    }

    String IdentWithWildCard() {
        String iden;
        iden = "";
        if (la.kind == 2) {
            Get();
            iden += "*";
            if (la.kind == 1) {
                Get();
                iden += t.val;
                if (la.kind == 2) {
                    Get();
                    iden += "*";
                }
            }
        } else if (la.kind == 1) {
            Get();
            iden += t.val;
            if (la.kind == 2) {
                Get();
                iden += "*";
            }
        } else { SynErr(26); }
        return iden;
    }

    public void Parse() {
        la = new Token();
        la.val = "";
        Get();
        DataStructures();
        Expect(0);

    }

    private static final boolean[][] set = {
            {_T, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x},
            {_x, _T, _T, _x, _x, _x, _T, _x, _x, _T, _T, _T, _T, _T, _x, _T, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x},
            {_x, _x, _x, _x, _x, _x, _x, _x, _x, _T, _T, _T, _T, _T, _x, _T, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x, _x}

    };

    public static class Errors { // Changed by MW to allow access to error count from outside the parser
        public int count = 0;                                    // number of errors detected
        public java.io.PrintStream errorStream = System.out;     // error messages go to this stream
        public String errMsgFormat = "-- line {0} col {1}: {2}"; // 0=line, 1=column, 2=text

        protected void printMsg(int line, int column, String msg) {
            StringBuffer b = new StringBuffer(errMsgFormat);
            int pos = b.indexOf("{0}");
            if (pos >= 0) {
                b.delete(pos, pos + 3);
                b.insert(pos, line);
            }
            pos = b.indexOf("{1}");
            if (pos >= 0) {
                b.delete(pos, pos + 3);
                b.insert(pos, column);
            }
            pos = b.indexOf("{2}");
            if (pos >= 0) { b.replace(pos, pos + 3, msg); }
            errorStream.println(b.toString());
        }

        public void SynErr(int line, int col, int n) {
            String s;
            switch (n) {
                case 0:
                    s = "EOF expected";
                    break;
                case 1:
                    s = "ident expected";
                    break;
                case 2:
                    s = "wildcard expected";
                    break;
                case 3:
                    s = "number expected";
                    break;
                case 4:
                    s = "float expected";
                    break;
                case 5:
                    s = "string expected";
                    break;
                case 6:
                    s = "\"package\" expected";
                    break;
                case 7:
                    s = "\"{\" expected";
                    break;
                case 8:
                    s = "\"}\" expected";
                    break;
                case 9:
                    s = "\"DS\" expected";
                    break;
                case 10:
                    s = "\"ds\" expected";
                    break;
                case 11:
                    s = "\"Datastructure\" expected";
                    break;
                case 12:
                    s = "\"DataStructure\" expected";
                    break;
                case 13:
                    s = "\"datastructure\" expected";
                    break;
                case 14:
                    s = "\"head\" expected";
                    break;
                case 15:
                    s = "\"Head\" expected";
                    break;
                case 16:
                    s = "\"(\" expected";
                    break;
                case 17:
                    s = "\")\" expected";
                    break;
                case 18:
                    s = "\";\" expected";
                    break;
                case 19:
                    s = "\"<\" expected";
                    break;
                case 20:
                    s = "\">\" expected";
                    break;
                case 21:
                    s = "\"[\" expected";
                    break;
                case 22:
                    s = "\"]\" expected";
                    break;
                case 23:
                    s = "\".\" expected";
                    break;
                case 24:
                    s = "\"$\" expected";
                    break;
                case 25:
                    s = "??? expected";
                    break;
                case 26:
                    s = "invalid IdentWithWildCard";
                    break;
                default:
                    s = "error " + n;
                    break;
            }
            printMsg(line, col, s);
            count++;
        }

        public void SemErr(int line, int col, String s) {
            printMsg(line, col, s);
            count++;
        }

        public void SemErr(String s) {
            errorStream.println(s);
            count++;
        }

        public void Warning(int line, int col, String s) {
            printMsg(line, col, s);
        }

        public void Warning(String s) {
            errorStream.println(s);
        }
    } // Errors

    public static class FatalError extends RuntimeException {
        public static final long serialVersionUID = 1L;

        public FatalError(String s) { super(s); }
    }
} // end Parser
