package at.jku.mevss.anttracks.heap.iteration;

import at.jku.mevss.anttracks.heap.DetailedHeap;
import at.jku.mevss.anttracks.heap.ObjectVisitor;
import at.jku.mevss.anttracks.heap.objects.ArrayObjectInfo;
import at.jku.mevss.anttracks.heap.objects.ObjectInfo;
import at.jku.mevss.anttracks.util.TraceException;

public class FakeHeapSpliterator extends HeapSpliteratorBase {

    public FakeHeapSpliterator(DetailedHeap heap, int startSpace, int startLab, int fenceSpace, int fenceLab, boolean current, boolean sorted) {
        this(heap, new SpliteratorRange(startSpace, startLab, fenceSpace, fenceLab), current, sorted);
    }

    public FakeHeapSpliterator(DetailedHeap heap, SpliteratorRange spliteratorRange, boolean current, boolean sorted) {
        super(heap, spliteratorRange, current, sorted);
    }

    public boolean tryAdvance(ObjectVisitor action, ObjectVisitor.Settings vistitorSettings) {
        if (!advanceStep()) {
            return false;
        }

        try {
            ObjectInfo obj = getCurrentObjectInfo();
            action.visit(getCurrentAddress(),
                         obj,
                         getCurrentSpaceInfo(),
                         obj.type,
                         obj.size,
                         vistitorSettings.arrayInfoNeeded ? obj instanceof ArrayObjectInfo : false,
                         vistitorSettings.arrayInfoNeeded ? ((obj instanceof ArrayObjectInfo) ? ((ArrayObjectInfo) obj).getLength() : -1) : -1,
                         obj.allocationSite,
                         vistitorSettings.pointerInfoNeeded ? getCurrentFromPointes() : null,
                         vistitorSettings.pointerInfoNeeded ? getCurrentToPointers() : null,
                         obj.eventType,
                         vistitorSettings.rootPointerInfoNeeded ? heap.rootPtrs.get(getCurrentAddress()) : null);
        } catch (TraceException e) {
            e.printStackTrace();
            return false;
        }

        return true;
    }

    FakeHeapSpliterator trySplit() {
        SpliteratorRange lower = getLowerSpliteratorRange();
        SpliteratorRange upper = getUpperSpliteratorRange();

        if (lower != null && upper != null) {
            setRange(lower);
            FakeHeapSpliterator fhs = new FakeHeapSpliterator(heap, upper, this.current, this.sorted);
            fhs.setListener(this.listener);
            return fhs;
        }
        return null;
    }
}