/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.heap.listener;

import at.jku.mevss.anttracks.heap.*;
import at.jku.mevss.anttracks.heap.objects.ObjectInfo;
import at.jku.mevss.anttracks.heap.space.Space;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

public abstract class NewObjectLogListener implements HeapListener {

    private final Map<Space, Long> fills;

    {
        fills = new HashMap<>();
    }

    @Override
    public void phaseChanging(DetailedHeap heap,
                              GarbageCollectionType from,
                              GarbageCollectionType to,
                              GarbageCollectionCause cause,
                              boolean concurrent,
                              boolean failed,
                              long time,
                              long position,
                              long length,
                              boolean withinSelectedTimeWindow) {
        if (from == GarbageCollectionType.MUTATOR) {
            ExecutorService threadPool = Executors.newCachedThreadPool();
            for (Space space : heap.getSpacesCloned()) {
                if (space == null) {
                    continue;
                }
                final Long first_addr = fills.get(space);
                space.iterate(
                        heap,
                        (address, objectInfo, spaze, type, size, isArray, arrayLength, allocationSite, pointedFrom, pointsTo, eventType, rootPtrs) -> {
                            if (first_addr != null && address < first_addr) {
                                return;
                            }
                            record(objectInfo);
                        },
                        ObjectVisitor.Settings.NO_INFOS,
                        null,
                        true,
                        threadPool,
                        null);
            }
            try {
                threadPool.shutdown();
                threadPool.awaitTermination(0, TimeUnit.MILLISECONDS);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
        }
    }

    protected abstract void record(ObjectInfo obj);

    @Override
    public void phaseChanged(DetailedHeap heap,
                             GarbageCollectionType from,
                             GarbageCollectionType to,
                             GarbageCollectionCause cause,
                             boolean concurrent,
                             boolean failed,
                             long time,
                             long position,
                             long length,
                             boolean withinSelectedTimeWindow) {
        if (to == GarbageCollectionType.MUTATOR) {
            fills.clear();
            for (Space space : heap.getSpacesCloned()) {
                if (space == null) {
                    continue;
                }
                fills.put(space, space.getFill());
            }
        }
    }
}
