/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.heap.listener;

import at.jku.mevss.anttracks.heap.DetailedHeap;
import at.jku.mevss.anttracks.heap.objects.ObjectInfo;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicLong;

public class ObjectSizeLogListener extends NewObjectLogListener {

    private final File file;

    private final Object lock;
    private final Map<Integer, AtomicLong> sizes;

    public ObjectSizeLogListener(File file) {
        this.file = file;
        if (file.exists()) {
            file.delete();
        }
        lock = new Object();
        sizes = new ConcurrentHashMap<>();
    }

    public static void init(DetailedHeap heap) {
        String log = System.getProperty("statistics.objectsize.log");
        if (log != null) {
            heap.addListener(new ObjectSizeLogListener(new File(log)));
        }
    }

    @Override
    protected void record(ObjectInfo obj) {
        record(obj.size);
    }

    private void record(int size) {
        AtomicLong counter = sizes.get(size);
        if (counter == null) {
            synchronized (lock) {
                counter = sizes.get(size);
                if (counter == null) {
                    counter = new AtomicLong(0);
                    sizes.put(size, counter);
                }
            }
        }
        counter.incrementAndGet();
    }

    @Override
    public void close(DetailedHeap heap) {
        try (BufferedWriter out = new BufferedWriter(new FileWriter(file))) {
            for (Integer size : sizes.keySet()) {
                long count = sizes.get(size).get();
                if (count == 0) {
                    continue;
                }
                out.write(String.valueOf(size));
                out.write(" ");
                out.write(String.valueOf(count));
                out.write("\n");
            }
        } catch (IOException e) {
            e.printStackTrace(System.err);
        }
    }
}
