/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.heap.listener;

import at.jku.mevss.anttracks.heap.DetailedHeap;
import at.jku.mevss.anttracks.heap.GarbageCollectionCause;
import at.jku.mevss.anttracks.heap.GarbageCollectionType;
import at.jku.mevss.anttracks.heap.HeapListener;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

public class PauseTimeLogListener implements HeapListener {

    private final File file_minor;
    private final File file_major;

    private long time;

    public PauseTimeLogListener(String file) {
        file_major = new File(file.replace("{0}", "major"));
        file_minor = new File(file.replace("{0}", "minor"));
        if (file_major.exists()) {
            file_major.delete();
        }
        if (file_minor.exists()) {
            file_minor.delete();
        }
    }

    @Override
    public void phaseChanging(DetailedHeap heap,
                              GarbageCollectionType from,
                              GarbageCollectionType to,
                              GarbageCollectionCause cause,
                              boolean concurrent,
                              boolean failed,
                              long time,
                              long position,
                              long length,
                              boolean withinSelectedTimeWindow) {}

    @Override
    public void phaseChanged(DetailedHeap heap,
                             GarbageCollectionType from,
                             GarbageCollectionType to,
                             GarbageCollectionCause cause,
                             boolean concurrent,
                             boolean failed,
                             long time,
                             long position,
                             long length,
                             boolean withinSelectedTimeWindow) {
        if (from != GarbageCollectionType.MUTATOR && !(cause.name.equals("System.gc()")) && !(cause.name.equals("JvmtiEnv ForceGarbageCollection"))) {
            double duration = (time - this.time) / 1000.0;
            try (BufferedWriter out = new BufferedWriter(new FileWriter(from.full ? file_major : file_minor, true))) {
                out.write(String.format("%.6f", duration));
                out.write('\n');
            } catch (IOException e) {
                e.printStackTrace(System.err);
            }
        }
        this.time = time;
    }

    @Override
    public void close(DetailedHeap heap) {}

    public static void init(DetailedHeap heap) {
        String log = System.getProperty("statistics.pausetime.log");
        if (log != null) {
            heap.addListener(new PauseTimeLogListener(log));
        }
    }

}
