package at.jku.mevss.anttracks.heap.roots;

import at.jku.mevss.anttracks.heap.DetailedHeap;
import at.jku.mevss.anttracks.heap.symbols.AllocatedType;

import java.nio.ByteBuffer;

public class ClassRoot extends RootPtr {

    public final int classId;
    private AllocatedType klass;

    public ClassRoot(long addr, int classId) {
        super(addr);
        this.classId = classId;
    }

    public String toString() {
        StringBuilder b = new StringBuilder(getClass().getSimpleName());
        b.append(" - ").append(klass != null ? klass.getExternalName(true, false) : "???");

        return b.toString();
    }

    @Override
    public String toShortString() {
        return getClass().getSimpleName() + " " + (klass != null ? klass.getExternalName(true, false) : "???");
    }

    @Override
    public byte[] getMetadata() {
        return ByteBuffer.allocate(1 + Long.BYTES + Integer.BYTES).put(RootType.CLASS_ROOT.byteVal).putLong(addr).putInt(classId).array();
    }

    @Override
    public RootType getRootType() {
        return RootType.CLASS_ROOT;
    }

    @Override
    public void resolve(DetailedHeap heap) throws Exception {
        klass = heap.getSymbols().types.stream().filter(c -> c.id == this.classId).findFirst().orElse(null);
        if (klass == null) {
            //            throw new Exception("Could not resolve Class root: " + toString());
        } else {
            resolved = true;
        }
    }

    @Override
    public String[] hierarchy() {
        String[] ret = {"Root: Class", "Class: " + (klass != null ? klass.getExternalName(true, false) : "???")};
        return ret;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }

        if (!ClassRoot.class.isAssignableFrom(obj.getClass())) {
            return false;
        }

        final ClassRoot other = (ClassRoot) obj;

        return this.addr == other.addr;
    }
}
