package at.jku.mevss.anttracks.heap.roots;

import at.jku.mevss.anttracks.heap.DetailedHeap;

import java.nio.ByteBuffer;

public class DebugRoot extends RootPtr {
    public final String vmCall;

    public DebugRoot(long addr, String vmCall) {
        super(addr);
        this.vmCall = vmCall;
    }

    public String toString() {
        StringBuilder b = new StringBuilder(getClass().getSimpleName()).append("\n");
        b.append("- ").append(vmCall != null ? vmCall : "null").append("\n");

        return b.toString();
    }

    @Override
    public String toShortString() {
        return getClass().getSimpleName() + " " + (vmCall != null ? vmCall : "???");
    }

    @Override
    public byte[] getMetadata() {
        return ByteBuffer.allocate(1 + 1 * Long.BYTES + 1 * Integer.BYTES + vmCall.getBytes().length * Byte.BYTES)
                         .put(RootType.DEBUG_ROOT.byteVal)
                         .putLong(addr)
                         .putInt(vmCall.length())
                         .put(vmCall.getBytes())
                         .array();
    }

    @Override
    public RootType getRootType() {
        return RootType.DEBUG_ROOT;
    }

    @Override
    public String getTypeString() { return vmCall; }

    @Override
    public void resolve(DetailedHeap heap) throws Exception {
        resolved = true;
    }

    @Override
    public String[] hierarchy() {
        String[] ret = {"Root: Debug", vmCall != null ? vmCall : "null"};
        return ret;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }

        if (!DebugRoot.class.isAssignableFrom(obj.getClass())) {
            return false;
        }

        final DebugRoot other = (DebugRoot) obj;

        return this.addr == other.addr && this.vmCall.equals(other.vmCall);
    }

}
