package at.jku.mevss.anttracks.heap.roots;

import at.jku.mevss.anttracks.heap.DetailedHeap;

import java.nio.ByteBuffer;

public class JNIGlobalRoot extends RootPtr {

    public final boolean weak;

    public JNIGlobalRoot(long addr, boolean weak) {
        super(addr);
        this.weak = weak;
    }

    public String toString() {
        StringBuilder b = new StringBuilder("JNI global:\n");

        return b.toString();
    }

    @Override
    public String toShortString() {
        return getClass().getSimpleName();
    }

    @Override
    public byte[] getMetadata() {
        return ByteBuffer.allocate(1 + Long.BYTES + 1).put(RootType.JNI_GLOBAL_ROOT.byteVal).putLong(addr).put(weak ? (byte) 1 : (byte) 0).array();
    }

    @Override
    public RootType getRootType() {
        return RootType.JNI_GLOBAL_ROOT;
    }

    @Override
    public void resolve(DetailedHeap heap) throws Exception {
        resolved = true;
    }

    @Override
    public String[] hierarchy() {
        String[] ret = {"Root: JNI global"};
        return ret;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }

        if (!JNIGlobalRoot.class.isAssignableFrom(obj.getClass())) {
            return false;
        }

        final JNIGlobalRoot other = (JNIGlobalRoot) obj;

        return this.addr == other.addr;
    }
}
