/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.heap.space;

public class SpaceInfo {
    public enum TransitionType {
        None,
        // No current GC (mutator phase)
        ReplaceAll,
        // ParallelOldGC: Major GC, G1: Major GC
        Accumulative // ParallelOldGC: Minor GC, G1: Minor GC
    }

    public final String name;

    private long address;
    private long length;
    private SpaceMode mode;
    private SpaceType type;

    private TransitionType transition;

    public SpaceInfo(String name) {
        this.name = name;
    }

    public long getAddress() {
        return address;
    }

    public long getEnd() {
        return address + length;
    }

    public void setAddress(long addr) {
        address = addr;
    }

    public long getLength() {
        return length;
    }

    public void setLength(long length) {
        this.length = length;
    }

    public SpaceMode getMode() {
        return mode;
    }

    public void setMode(SpaceMode mode) {
        this.mode = mode;
    }

    public SpaceType getType() {
        return type;
    }

    public void setType(SpaceType type) {
        this.type = type;
    }

    public TransitionType getTransitionType() {
        return transition;
    }

    public void setTransitionType(TransitionType transition) {
        this.transition = transition;
    }

    public boolean isBeingCollected() {
        return getTransitionType() == SpaceInfo.TransitionType.ReplaceAll;
    }

    @Override
    public String toString() {
        return String.format("%s @ %,d - %,d (Length: %,d) (Mode: %s, Type: %s)", name, getAddress(), getAddress() + getLength(), getLength(), getMode(), getType());
    }
}
