package at.jku.mevss.anttracks.heap.space;

import at.jku.mevss.anttracks.heap.DetailedHeap;

import java.util.logging.Logger;

public class SpaceRedefinition {
    private final DetailedHeap heap;
    public final Space space;
    private final long addr;
    private final long length;
    private final boolean postponed;
    private boolean executed;

    private Logger LOGGER = Logger.getLogger(SpaceRedefinition.class.getSimpleName());

    public SpaceRedefinition(DetailedHeap heap, int index, long addr, long length) {
        this(heap, index, addr, length, false);
    }

    public SpaceRedefinition(DetailedHeap heap, int index, long addr, long length, boolean postponed) {
        this.heap = heap;
        this.space = heap.getSpace(index);
        this.addr = addr;
        this.length = length;
        this.postponed = postponed;

        // TODO Space ranges are not correctly implemented at the moment...
        // After a space redefinition event, the new ranges should be used when looking for a space that should be looked up in FRONT (or if a LAB should be allocated).
        // The old ranges should be used when looking up an object in BACK
        if (!postponed) {
            execute();
        }
    }

    public void execute() {
        if (this.executed) {
            LOGGER.warning("Space redefiniton of space " + space.getId() + " has already been executed before and is not executed again.");
            return;
        }
        space.resetAddressAndLength(addr, length);
        // LOGGER.info("Space redefiniton of space " + space.getId() + " executed... " + this.toString());
        // LOGGER.info(heap.toString());
        this.executed = true;
    }

    public boolean hasBeenExecuted() {
        return executed;
    }

    @Override
    public String toString() {
        return String.format("Current: %s; New range: %,d - %,d; New size: %,d", space.toShortString(), addr, addr + length, length);
    }
}
