/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.parser.heap;

import at.jku.mevss.anttracks.heap.DetailedHeap;
import at.jku.mevss.anttracks.heap.GarbageCollectionLookup;
import at.jku.mevss.anttracks.heap.HeapListener;
import at.jku.mevss.anttracks.heap.io.*;
import at.jku.mevss.anttracks.heap.symbols.Symbols;
import at.jku.mevss.anttracks.parser.TraceScannerFactory;

import java.io.FileNotFoundException;
import java.io.IOException;

/**
 * @author Christina Rammerstorfer
 */
public class ListenerHeapTraceParser extends HeapTraceParser {

    private final HeapListener listener;

    public ListenerHeapTraceParser(Symbols symbols, HeapListener listener) throws Exception {
        super(symbols, null);
        this.listener = listener;
    }

    public ListenerHeapTraceParser(Symbols symbols, MetaDataReaderConfig readerConfig, GarbageCollectionLookup to, HeapListener listener) throws Exception {
        super(symbols, readerConfig, to);
        this.listener = listener;
    }

    public ListenerHeapTraceParser(Symbols symbols, MetaDataReaderConfig readerConfig, long toTime, HeapListener listener) throws Exception {
        super(symbols, readerConfig, toTime);
        this.listener = listener;
    }

    public ListenerHeapTraceParser(Symbols symbols, MetaDataWriterConfig writerConfig, HeapListener listener) throws Exception {
        super(symbols, writerConfig);
        this.listener = listener;
    }

    @Override
    protected DetailedHeap generatePlainWorkspace(TraceScannerFactory factory) throws IOException {

        DetailedHeap heap = HeapBuilder.constructHeap(symbols);
        heap.addListener(listener);
        if (writerConfig != null) {
            heap.addListener(new MetaDataWriterListener(writerConfig, symbols));
        }
        return heap;
    }

    @Override
    protected DetailedHeap generateWorkspaceFromMetaData(HeapIndexReader heapIndexReader, HeapPosition heapPosition) throws FileNotFoundException, IOException {
        DetailedHeap heap;
        if (heapPosition.fileName != -1) {
            heap = heapIndexReader.getHeap(heapPosition.fileName, symbols);
        } else {
            heap = HeapBuilder.constructHeap(symbols);
        }
        heap.addListener(listener);
        return heap;
    }

}
