package at.jku.mevss.anttracks.util;

import at.jku.mevss.anttracks.heap.IndexBasedHeap;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import static at.jku.mevss.anttracks.heap.IndexBasedHeap.NULL_INDEX;

/**
 * Based on the Lengauer-Tarjan algorithm presented in 'Modern Compiler Implementation in Java'
 */

public class AntTracksDominators {

    private static final int ROOT_INDEX = 0; // consequently all pointers and objects in the heap are shifted by 1!
    // 'none' in algorithm is NULL_INDEX i.e. -1

    private final IndexBasedHeap heap;
//    long[] reachableAdresses;
//    private final int[][] toPointers;
//    private final int[][] fromPointers;
//    private final int[] superRootToPointers;

    private int N = 0;
    private final int[] dfnum;
    private final int[] vertex;
    private final int[] parent;

    private final int[][] bucket;
    private final int[] semi;
    private final int[] ancestor;
    private final int[] idom;
    private final int[] samedom;

    private final int[] best;

    public AntTracksDominators(IndexBasedHeap heap) {
        this.heap = heap;

        // make the whole heap reachable by root pointers...
//        for(int i = 0; i < heap.getObjectCount(); i++) {
//            if(!heap.isRootReachable(i)) {
//                heap.getRootPointerBlocking().put(i, Collections.singletonList(new OtherRoot(i, RootPtr.RootType.JVMTI_ROOT)));
//            }
//        }

        // compact the heap into one block of root-reachable objects...
//        reachableAdresses = new long[heap.getReachableFromRootObjectCount()];
//        toPointers = new int[heap.getReachableFromRootObjectCount()][];
//        fromPointers = new int[heap.getReachableFromRootObjectCount()][];
//        superRootToPointers = new int[(int)heap.getRootPointerBlocking().keySet().stream().filter(ptr -> ptr != NULL_INDEX).count()];
//        translatePointers();

        int nodeCount = heap.getObjectCount() + 1;  // all objects + root node

        N = 0;
        dfnum = new int[nodeCount];
        vertex = new int[nodeCount];
        parent = new int[nodeCount];

        bucket = new int[nodeCount][];
        semi = new int[nodeCount];
        ancestor = new int[nodeCount];
        idom = new int[nodeCount];
        samedom = new int[nodeCount];

        best = new int[nodeCount];

        Arrays.fill(dfnum, 0);

        Arrays.fill(semi, NULL_INDEX);
        Arrays.fill(ancestor, NULL_INDEX);
        Arrays.fill(samedom, NULL_INDEX);
        Arrays.fill(idom, NULL_INDEX);

        Arrays.fill(best, NULL_INDEX);

        DFS(NULL_INDEX, ROOT_INDEX);

        calculateDominators();
    }

    private void translatePointers() {
//        int counter = 0;
//        for(int i = 0; i < heap.getObjectCount(); i++) {
//            if(heap.isRootReachable(i)) {
//                reachableAdresses[counter] = heap.getAddress(i);
//                counter++;
//            }
//        }
//
//        counter = 0;
//        for(int i = 0; i < heap.getObjectCount(); i++) {
//            if(heap.isRootReachable(i)) {
//                int[] ptrs = heap.getToPointers(i);
//                if(ptrs != null) {
//                    toPointers[counter] = new int[ptrs.length];
//                    for(int j = 0; j < ptrs.length; j++) {
//                        if(ptrs[j] == NULL_INDEX || !heap.isRootReachable(ptrs[j])) {
//                            toPointers[counter][j] = NULL_INDEX;
//                        } else {
//                            toPointers[counter][j] = Arrays.binarySearch(reachableAdresses, heap.getAddress(ptrs[j]));
//                        }
//                    }
//                }
//
//
//                ptrs = heap.getFromPointers(i);
//                if(ptrs != null) {
//                    fromPointers[counter] = new int[ptrs.length];
//                    for(int j = 0; j < ptrs.length; j++) {
//                        if(ptrs[j] == NULL_INDEX || !heap.isRootReachable(ptrs[j])) {
//                            fromPointers[counter][j] = NULL_INDEX;
//                        } else {
//                            fromPointers[counter][j] = Arrays.binarySearch(reachableAdresses, heap.getAddress(ptrs[j]));
//                        }
//                    }
//                }
//
//                counter++;
//            }
//        }
//
//        List<Integer> rootIndices = heap.getRootPointerBlocking().keySet().stream().filter(idx -> idx != NULL_INDEX).collect(Collectors.toList());
//        for(int i = 0; i < rootIndices.size(); i++) {
//            superRootToPointers[i] = Arrays.binarySearch(reachableAdresses, heap.getAddress(rootIndices.get(i))) + 1;
//        }
    }

    private void calculateDominators() {
        for (int i = N - 1; i >= 1; i--) {
            int n = vertex[i];
            int p = parent[n];
            int s = p;

            int[] predecessors = predecessors(n);
            if (predecessors != null) {              // !!!
                for (int v : predecessors) {
                    if (v != NULL_INDEX) {          // !!!
                        int sPrime;

                        if (dfnum[v] <= dfnum[n]) {
                            sPrime = v;
                        } else {
                            sPrime = semi[AncestorWithLowestSemi(v)];
                        }

                        if (dfnum[sPrime] < dfnum[s]) {
                            s = sPrime;
                        }
                    }
                }
            }

            semi[n] = s;
            bucket[s] = union(bucket[s], n);
            Link(p, n);

            if (bucket[p] != null) {
                for (int v : bucket[p]) {
                    int y = AncestorWithLowestSemi(v);

                    if (semi[y] == semi[v]) {
                        idom[v] = p;
                    } else {
                        samedom[v] = y;
                    }
                }
            }

            bucket[p] = null;
        }

        for (int i = 1; i <= N - 1; i++) {
            int n = vertex[i];
            if (samedom[n] != NULL_INDEX) {
                idom[n] = idom[samedom[n]];
            }
        }
    }

    private void DFS(int p, int n) {
        if (dfnum[n] == 0) {
            dfnum[n] = N;
            vertex[N] = n;
            parent[n] = p;
            N = N + 1;

            int[] successors = successors(n);
            if (successors != null) {                // !!!
                for (int w : successors) {
                    if (w != NULL_INDEX) {          // !!!
                        DFS(n, w);
                    }
                }
            }
        }
    }

    private int[] successors(int n) {
        if (n < 0) {
            throw new IllegalArgumentException("successors called with index " + n);
        }

        if (n == 0) {
            return heap.getRootPointerBlocking().keySet().stream().filter(i -> i != NULL_INDEX).mapToInt(i -> i + 1).toArray();
//            return superRootToPointers;
        }

        int[] successors = heap.getToPointers(n - 1);
//        int[] successors = toPointers[n - 1];

        if (successors == null) {
            return null;
        }

        // perform shift for artificial root node TODO move this into the main algorithm to improve performance (iterate only once through successors!)
        for (int i = 0; i < successors.length; i++) {
            if (successors[i] != NULL_INDEX) {
                successors[i]++;
            }
        }

        return successors;
    }

    private int[] predecessors(int n) {
        if (n < 0) {
            throw new IllegalArgumentException("predecessors called with index " + n);
        }

        if (n == 0) {
            return null;
        }

        int[] predecessors = heap.getFromPointers(n - 1);
//        int[] predecessors = fromPointers[n - 1];

        if (predecessors == null) {
            return null;
        }

        // perform shift for artificial root node TODO move this into the main algorithm to improve performance (iterate only once through predecessors!)
        for (int i = 0; i < predecessors.length; i++) {
            if (predecessors[i] != NULL_INDEX) {
                predecessors[i]++;
            }
        }

        return predecessors;
    }

    private int AncestorWithLowestSemiNaive(int v) {
        int u = v;

        while (ancestor[v] != NULL_INDEX) {
            if (dfnum[semi[v]] < dfnum[semi[u]]) {
                u = v;
            }
            v = ancestor[v];
        }

        return u;
    }

    private void LinkNaive(int p, int n) {
        ancestor[n] = p;
    }

    private int AncestorWithLowestSemi(int v) {
        int a = ancestor[v];

        if (ancestor[a] != NULL_INDEX) {
            int b = AncestorWithLowestSemi(a);
            ancestor[v] = ancestor[a];

            if (dfnum[semi[b]] < dfnum[semi[best[v]]]) {
                best[v] = b;
            }
        }

        return best[v];
    }

    private void Link(int p, int n) {
        ancestor[n] = p;
        best[n] = n;
    }

    private int[] union(int[] arr, int n) {
        if (arr == null) {
            return new int[]{n};
        }

        for (int i = 0; i < arr.length; i++) {
            if (arr[i] == n) {
                return arr;
            }
        }

        int[] ret = Arrays.copyOf(arr, arr.length + 1);
        ret[ret.length - 1] = n;

        return ret;
    }

    // --------------------------------
    // ACTUAL INTERFACE
    // --------------------------------

    /**
     * Returns the immediate dominator of a given object
     *
     * @param n the index of an object in the heap
     * @return the index of the object that immediately dominates the given object. If the given object is immediately dominated by the super root object, -2 is returned.
     */
    public int getImmediateDominator(int n) {
        if (!heap.valid(n)) {
            throw new IllegalArgumentException("Given index is out of bounds!");
        }

//        if(!heap.isRootReachable(n)) {
//            return NULL_INDEX;  // has no dominator
//        }
//
//        int translatedN = Arrays.binarySearch(reachableAdresses, heap.getAddress(n));
//        int immediateDominator = idom[translatedN + 1];
//
//        if(immediateDominator == -1) {
//            return -1;
//        }
//        if(immediateDominator == 0) {
//            return -2;  // super root
//        } else {
//            return Arrays.binarySearch(((FastHeap)heap).addresses, reachableAdresses[immediateDominator - 1]);
//        }

        return idom[n + 1] == 0 ? -2 : idom[n + 1] - 1;
    }

    public boolean isImmediatelyDominatedBy(int n, int m) {
        return idom[n + 1] == m + 1;
    }

    public int[] immediatelyDominatedNodes(int n) {
        List<Integer> ret = new ArrayList<>();
        for (int i = 0; i < idom.length; i++) {
            if (idom[i] == n + 1) {
                ret.add(i - 1);
            }
        }

        return ret.stream()
                  .mapToInt(i -> i)
                  .toArray();
    }
}
