/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.util;

public class Counter {

    private long count;
    private final boolean allowNegative;

    public Counter() {
        this(false);
    }

    public Counter(long init) {
        this(init, false);
    }

    public Counter(boolean allowNegative) {
        this(0, allowNegative);
    }

    public Counter(long init, boolean allowNegative) {
        this.allowNegative = allowNegative;
        count = init;
    }

    public long get() {
        return count;
    }

    public synchronized void add(long modifier) {
        count += modifier;
    }

    public synchronized void add(Counter otherCounter) {
        add(otherCounter.get());
    }

    public synchronized void sub(long modifier) {
        count -= modifier;
        if (!allowNegative && count < 0) {
            count = 0;
        }
    }

    public void max(long c) {
        count = Math.max(count, c);
    }

    public void min(long c) {
        count = Math.min(count, c);
    }

    public synchronized void inc() {
        add(1);
    }

    public synchronized void dec() {
        sub(1);
    }

    @Override
    public String toString() {
        return String.valueOf(count);
    }

    public synchronized void reset() {
        count = 0;
    }
}
