/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.util;

import java.io.IOException;
import java.io.OutputStream;

public final class RunOnCloseOutputStream extends OutputStream {

    private final OutputStream out;
    private final IORunnable[] runnables;

    public RunOnCloseOutputStream(OutputStream out, IORunnable... runnables) {
        this.out = out;
        this.runnables = runnables;
    }

    @Override
    public void write(int value) throws IOException {
        out.write(value);
    }

    @Override
    public void write(byte[] data, int offset, int length) throws IOException {
        out.write(data, offset, length);
    }

    @Override
    public void flush() throws IOException {
        out.flush();
    }

    @Override
    public void close() throws IOException {
        out.close();
        for (IORunnable runnable : runnables) {
            runnable.run();
        }
    }

}
