/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.util;

import javafx.concurrent.Task;

import java.util.concurrent.Future;

public class ThreadUtil {

    private static final int SHORT_DEFERRED_PERIOD = 50;
    private static final int NORMAL_DEFERRED_PERIOD = 100;
    private static final int LONG_DEFERRED_PERIOD = 500;

    public enum DeferredPeriod {
        SHORT,
        NORMAL,
        LONG
    }

    public static Task<Void> runDeferred(Runnable runnable, DeferredPeriod deferredPeriod) {
        switch (deferredPeriod) {
            case LONG:
                return runDeferred(runnable, SHORT_DEFERRED_PERIOD);
            case NORMAL:
                return runDeferred(runnable, NORMAL_DEFERRED_PERIOD);
            case SHORT:
                return runDeferred(runnable, LONG_DEFERRED_PERIOD);
        }
        return null;
    }

    private static Task<Void> runDeferred(Runnable runnable, int milliseconds) {
        Task<Void> task = new Task<Void>() {
            @Override
            protected Void call() throws Exception {
                Thread.sleep(milliseconds);
                runnable.run();
                return null;
            }
        };
        startTask(task);
        return task;
    }

    public static <T> Future<T> startTask(Task<T> task) {
        return ParallelizationUtil.submitTask(task);
    }
}
