package at.jku.mevss.anttracks.heap.datastructures

import at.jku.mevss.anttracks.heap.DetailedHeap
import at.jku.mevss.anttracks.heap.IndexBasedHeap
import at.jku.mevss.anttracks.heap.io.MetaDataReaderConfig
import at.jku.mevss.anttracks.heap.symbols.Symbols
import at.jku.mevss.anttracks.parser.*
import at.jku.mevss.anttracks.parser.heap.HeapTraceParser
import at.jku.mevss.anttracks.parser.printing.AdditionalPrintingEventHandler
import at.jku.mevss.anttracks.util.MutableLong
import java.util.concurrent.BlockingQueue

class DataStructureDevelopmentTraceParser(symbols: Symbols,
                                          readerConfig: MetaDataReaderConfig,
                                          fromTime: Long,
                                          toTime: Long) : HeapTraceParser(symbols, readerConfig, fromTime, toTime) {
    var latestTime: Long = -1
        private set
    var initialIndexHeap: IndexBasedHeap? = null
        private set
    var addresses: Array<MovingObject>? = null
        private set

    init {
        addEventHandler { heap, _ ->
            AdditionalPrintingEventHandler(heap)
        }
    }

    override fun startSlaveThreads(queueSize: MutableLong?,
                                   masterQueue: BlockingQueue<ThreadLocalHeap>?,
                                   workspace: DetailedHeap?,
                                   handler: ErrorHandler?,
                                   check: Boolean,
                                   parsingInfo: ParsingInfo?): MutableList<TraceSlaveParser<DetailedHeap>> {
        gcPoint(workspace!!.lastGCTime)
        return super.startSlaveThreads(queueSize, masterQueue, workspace, handler, check, parsingInfo)
    }

    override fun createMainEventHandler(parsingInfo: ParsingInfo?): TraceParsingEventHandler =
            DataStructureDevelopmentEventHandler(workspace, symbols, parsingInfo!!, ::checkMoves, ::gcPoint, ::addresses)

    private fun checkMoves(): Boolean = latestTime >= fromTime && latestTime <= toTime

    private fun gcPoint(time: Long) {
        if (latestTime != time) {
            latestTime = time
            if (latestTime == fromTime) {
                initialIndexHeap = workspace.toIndexBasedHeap()
                // !! enforces that initialIndexHeap is not null and will throw NullPointerException otherwise
                addresses = initialIndexHeap!!.dataStructures.map({ ds -> MovingObject(initialIndexHeap!!.getAddress(ds.headIdx)) }).toTypedArray()
            }
        }
    }
}