/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.classification.test;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierFactoryList;
import at.jku.mevss.anttracks.classification.nodes.ListGroupingNode;
import at.jku.mevss.anttracks.heap.DetailedHeap;
import at.jku.mevss.anttracks.heap.FastHeap;
import at.jku.mevss.anttracks.util.TraceException;
import org.junit.Before;
import org.junit.Test;

import java.io.*;
import java.util.HashMap;

import static org.junit.Assert.assertEquals;

public class ListGroupingNode_Test {
    // TODO: This should be stubbed
    FastHeap fastHeap = new FastHeap(new DetailedHeap(null)) {
        @Override
        public int getSize(int objIndex) {
            return 1;
        }
    };

    ListGroupingNode flat_g1 = new ListGroupingNode(fastHeap);
    ListGroupingNode flat_g2 = new ListGroupingNode(fastHeap);
    ListGroupingNode flat_g3 = new ListGroupingNode(fastHeap);

    ListGroupingNode deep_g1 = new ListGroupingNode(fastHeap);
    ListGroupingNode deep_g2 = new ListGroupingNode(fastHeap);
    ListGroupingNode deep_g3 = new ListGroupingNode(fastHeap);

    at.jku.mevss.anttracks.classification.test.SampleClassifier sampleClassifier = new at.jku.mevss.anttracks.classification.test.SampleClassifier();

    ClassifierFactoryList availableClassifier;

    @Before
    public void initClass() throws IOException, TraceException, Exception {
        availableClassifier = new ClassifierFactoryList(null, null, null);
        availableClassifier.add(sampleClassifier);

        flat_g1 = new ListGroupingNode(fastHeap);
        flat_g1.addLeaf(1, 1);
        flat_g1.addLeaf(2, 1);
        flat_g1.addLeaf(3, 1);

        flat_g2 = new ListGroupingNode(fastHeap);
        flat_g2.addLeaf(1, 1);
        flat_g2.addLeaf(2, 1);
        flat_g2.addLeaf(3, 1);

        flat_g3 = new ListGroupingNode(fastHeap);
        flat_g3.addLeaf(1, 1);
        flat_g3.addLeaf(2, 1);
        flat_g3.addLeaf(3, 1);

        deep_g1 = new ListGroupingNode(fastHeap);
        deep_g1.addLeaf(1, 1);
        deep_g1.addLeaf(2, 1);
        deep_g1.addLeaf(3, 1);
        ListGroupingNode child = null;
        child = (ListGroupingNode) deep_g1.addChildIfNeeded(sampleClassifier, "c1");
        child.addLeaf(1, 1);
        child.addLeaf(2, 1);
        child.addLeaf(3, 1);
        child = null;
        child = (ListGroupingNode) deep_g1.addChildIfNeeded(sampleClassifier, "c2");
        child.addLeaf(2, 1);
        child.addLeaf(3, 1);
        child.addLeaf(4, 1);
        child = (ListGroupingNode) deep_g1.addChildIfNeeded(sampleClassifier, "c3");
        child.addLeaf(5, 1);
        child = (ListGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1");
        child.addLeaf(5, 1);
        child = (ListGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1");
        child.addLeaf(5, 1);
        child = (ListGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1-1");
        child.addLeaf(5, 1);
        child = (ListGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1-1-1");
        child.addLeaf(5, 1);
        child = (ListGroupingNode) child.getParent().addChildIfNeeded(sampleClassifier, "c3-1-1-1-2");
        child.addLeaf(5, 1);
        child = (ListGroupingNode) child.getParent().addChildIfNeeded(sampleClassifier, "c3-1-1-1-3");
        child.addLeaf(1000, 1);

        deep_g2 = new ListGroupingNode(fastHeap);
        deep_g2.addLeaf(1, 1);
        deep_g2.addLeaf(2, 1);
        deep_g2.addLeaf(3, 1);
        child = (ListGroupingNode) deep_g2.addChildIfNeeded(sampleClassifier, "c1");
        child.addLeaf(1, 1);
        child.addLeaf(2, 1);
        child.addLeaf(3, 1);
        child = null;
        child = (ListGroupingNode) deep_g2.addChildIfNeeded(sampleClassifier, "c2");
        child.addLeaf(2, 1);
        child.addLeaf(3, 1);
        child.addLeaf(4, 1);
        child = (ListGroupingNode) deep_g2.addChildIfNeeded(sampleClassifier, "c3");
        child.addLeaf(6, 1);
        child = (ListGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1");
        child.addLeaf(6, 1);
        child = (ListGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1");
        child.addLeaf(6, 1);
        child = (ListGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1-1");
        child.addLeaf(6, 1);
        child = (ListGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1-1-1");
        child.addLeaf(6, 1);
        child = (ListGroupingNode) child.getParent().addChildIfNeeded(sampleClassifier, "c3-1-1-1-2");
        child.addLeaf(6, 1);
        child = (ListGroupingNode) child.getParent().addChildIfNeeded(sampleClassifier, "c3-1-1-1-3");
        child.addLeaf(1000, 1);

        deep_g3 = new ListGroupingNode(fastHeap);
        deep_g3.addLeaf(1, 1);
        deep_g3.addLeaf(2, 1);
        deep_g3.addLeaf(3, 1);
        child = (ListGroupingNode) deep_g3.addChildIfNeeded(sampleClassifier, "c1");
        child.addLeaf(1, 1);
        child.addLeaf(2, 1);
        child.addLeaf(3, 1);
        child = null;
        child = (ListGroupingNode) deep_g3.addChildIfNeeded(sampleClassifier, "c2");
        child.addLeaf(2, 1);
        child.addLeaf(3, 1);
        child.addLeaf(4, 1);
        child = (ListGroupingNode) deep_g3.addChildIfNeeded(sampleClassifier, "c3");
        child.addLeaf(7, 1);
        child = (ListGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1");
        child.addLeaf(7, 1);
        child = (ListGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1");
        child.addLeaf(7, 1);
        child = (ListGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1-1");
        child.addLeaf(7, 1);
        child = (ListGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1-1-1");
        child.addLeaf(7, 1);
        child = (ListGroupingNode) child.getParent().addChildIfNeeded(sampleClassifier, "c3-1-1-1-2");
        child.addLeaf(7, 1);
        child = (ListGroupingNode) child.getParent().addChildIfNeeded(sampleClassifier, "c3-1-1-1-3");
        child.addLeaf(1000, 1);
    }

    @Test
    public void test_Merge_Flat() {
        ListGroupingNode g = (ListGroupingNode) flat_g1.merge(flat_g2).merge(flat_g3);
        assertEquals(3, g.getObjectCount());
    }

    @Test
    public void test_Merge_Deep() {
        ListGroupingNode g = (ListGroupingNode) deep_g1.merge(deep_g2).merge(deep_g3);
        assertEquals(8, g.getObjectCount());
    }

    // TODO need heap to sampleTopDown
    //    @Test
    //    public void test_Merge_SampleDown_Deep() {
    //        ListGroupingNode g = (ListGroupingNode) deep_g1.merge(deep_g2).merge(deep_g3).sampleTopDown();
    //        assertEquals(8, g.getObjectCount());
    //    }
    //
    //    @Test
    //    public void test_Merge_SampleDown_Flat() {
    //        ListGroupingNode g = (ListGroupingNode) flat_g1.merge(flat_g2).merge(flat_g3).sampleTopDown();
    //        assertEquals(3, g.getObjectCount());
    //    }

    @Test
    public void test_writeAndRead() throws FileNotFoundException {
        //        ListGroupingNode writeTree = (ListGroupingNode) deep_g1.merge(deep_g2).merge(deep_g3).sampleTopDown();
        HashMap<String, Classifier<?>> classifier = new HashMap<>();
        availableClassifier.forEach(fac -> classifier.put(fac.getName(), fac.create()));

        ListGroupingNode writeTree = (ListGroupingNode) deep_g1.merge(deep_g2).merge(deep_g3);
        ListGroupingNode.writeTree(new DataOutputStream(new FileOutputStream("./testoutput", false)), writeTree, null, -1, null);
        ListGroupingNode readTree = ListGroupingNode.readTree(new DataInputStream(new FileInputStream("./testoutput")), null, classifier);
        assertEquals(writeTree.toString(), readTree.toString());
    }
}
