package at.jku.mevss.anttracks.closure.test;

import at.jku.mevss.anttracks.heap.*;
import at.jku.mevss.anttracks.parser.EventType;
import at.jku.mevss.anttracks.util.ClosureUtil;
import at.jku.mevss.anttracks.util.TestUtility;
import at.jku.mevss.anttracks.util.TraceException;
import jku.anttracks.example.closuresize.Main;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;
import java.util.BitSet;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;

public class ClosureTest {

    private static PreprocessedHeap preprocessedHeap;

    @BeforeClass
    public static void initClass() throws IOException, TraceException, Exception {
        String traceFolder = TestUtility.getExampleTrace("PointerTraces/ClosureSize/trace", "./downloadedTrace");
        preprocessedHeap = TestUtility.preprocess(traceFolder, true);
    }

    @Before
    public void setup() {

    }

    @Test
    public void testShallowSize_X() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 0));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int xIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.X.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, xIndex); // Object of type X must be found

        int shallowSize = fastHeap.getSize(xIndex);
        assertEquals(24, shallowSize);
    }

    @Test
    public void testDeepSize_X() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 0));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int xIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.X.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, xIndex); // Object of type X must be found

        BitSet transitiveClosure = fastHeap.transitiveClosure(new int[]{xIndex}, null);
        assertEquals(88, ClosureUtil.getClosureByteCount(transitiveClosure, fastHeap));
    }

    @Test
    public void testShallowSize_Y() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 0));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int yIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.Y.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, yIndex); // Object of type Y must be found

        int shallowSize = fastHeap.getSize(yIndex);
        assertEquals(24, shallowSize);
    }

    @Test
    public void testDeepSize_Y() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 0));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int yIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.Y.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, yIndex); // Object of type X must be found

        BitSet transitiveClosure = fastHeap.transitiveClosure(new int[]{yIndex}, null);
        assertEquals(24, ClosureUtil.getClosureByteCount(transitiveClosure, fastHeap));
    }

    @Test
    public void testShallowSize_ABCD() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 0));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int aIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.A.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, aIndex); // Object of type A must be found

        int shallowSize = fastHeap.getSize(aIndex);
        assertEquals(24, shallowSize);

        int bIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.B.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, bIndex); // Object of type B must be found

        shallowSize = fastHeap.getSize(bIndex);
        assertEquals(16, shallowSize);

        int cIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.C.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, cIndex); // Object of type C must be found

        shallowSize = fastHeap.getSize(cIndex);
        assertEquals(16, shallowSize);

        int dIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.D.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, dIndex); // Object of type D must be found

        shallowSize = fastHeap.getSize(dIndex);
        assertEquals(16, shallowSize);
    }

    @Test
    public void testDeepSize_ABCD() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 0));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int aIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.A.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, aIndex); // Object of type A must be found

        BitSet transitiveClosure = fastHeap.transitiveClosure(new int[]{aIndex}, null);
        assertEquals(72, ClosureUtil.getClosureByteCount(transitiveClosure, fastHeap));

        int bIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.B.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, bIndex); // Object of type B must be found

        transitiveClosure = fastHeap.transitiveClosure(new int[]{bIndex}, null);
        assertEquals(72, ClosureUtil.getClosureByteCount(transitiveClosure, fastHeap));

        int cIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.C.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, cIndex); // Object of type C must be found

        transitiveClosure = fastHeap.transitiveClosure(new int[]{cIndex}, null);
        assertEquals(72, ClosureUtil.getClosureByteCount(transitiveClosure, fastHeap));

        int dIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.D.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, dIndex); // Object of type D must be found

        transitiveClosure = fastHeap.transitiveClosure(new int[]{dIndex}, null);
        assertEquals(72, ClosureUtil.getClosureByteCount(transitiveClosure, fastHeap));
    }

    @Test
    public void testRetainedSize_ABCD_EverythingOwned() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 0));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int aIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.A.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, aIndex); // Object of type A must be found

        BitSet gcClosure = fastHeap.gcClosure(new int[]{aIndex}, fastHeap.transitiveClosure(new int[]{aIndex}, null));
        assertEquals(72, ClosureUtil.getClosureByteCount(gcClosure, fastHeap));
    }

    @Test
    public void testRetainedSize_ABCD_DirectlyRootPointed() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 1));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int aIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.A.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, aIndex); // Object of type A must be found

        BitSet gcClosure = fastHeap.gcClosure(new int[]{aIndex}, fastHeap.transitiveClosure(new int[]{aIndex}, null));
        assertEquals(40, ClosureUtil.getClosureByteCount(gcClosure, fastHeap));
    }

    @Test
    public void testRetainedSize_ABCD_BPointedByNonClosureObject() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 2));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int aIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.A.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, aIndex); // Object of type A must be found

        BitSet gcClosure = fastHeap.gcClosure(new int[]{aIndex}, fastHeap.transitiveClosure(new int[]{aIndex}, null));
        assertEquals(40, ClosureUtil.getClosureByteCount(gcClosure, fastHeap));
    }

    @Test
    public void testRetainedSize_ABCD_CPointedByNonClosureObject() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 3));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int aIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.A.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, aIndex); // Object of type A must be found

        BitSet gcClosure = fastHeap.gcClosure(new int[]{aIndex}, fastHeap.transitiveClosure(new int[]{aIndex}, null));
        assertEquals(40, ClosureUtil.getClosureByteCount(gcClosure, fastHeap));
    }

    @Test
    public void testRetainedSize_ABCD_DPointedByNonClosureObject() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 4));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int aIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.A.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, aIndex); // Object of type A must be found

        BitSet gcClosure = fastHeap.gcClosure(new int[]{aIndex}, fastHeap.transitiveClosure(new int[]{aIndex}, null));
        assertEquals(56, ClosureUtil.getClosureByteCount(gcClosure, fastHeap));
    }

    @Test
    public void testRetainedSize_ABCD_CPointedByStaticField() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 5));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int aIndex = fastHeap.stream().filter(objIdx -> fastHeap.getType(objIdx).getExternalName(false, true).equals(Main.A.class.getName())).findAny().orElse(-1);
        assertNotEquals(-1, aIndex); // Object of type A must be found

        BitSet gcClosure = fastHeap.gcClosure(new int[]{aIndex}, fastHeap.transitiveClosure(new int[]{aIndex}, null));
        assertEquals(40, ClosureUtil.getClosureByteCount(gcClosure, fastHeap));
    }
}
