/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.compiler.test;

import at.jku.mevss.anttracks.compiler.OnTheFlyCompiler;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class OnTheFlyCompilerTest {

    OnTheFlyCompiler compiler;

    @Before
    public void setup() {
        compiler = new OnTheFlyCompiler();
    }

    // ---------------------------------------------------------------------------
    // Extract package

    @Test
    public void test_extractPackage_singleLine() {
        String result = compiler.extractPackage("package at.test;");
        Assert.assertEquals("Package must match", "at.test", result);
    }

    @Test
    public void test_extractPackage_singleLineMultiSpaces() {
        String result = compiler.extractPackage("package      \tat.test;");
        Assert.assertEquals("Package must match", "at.test", result);
    }

    // ---------------------------------------------------------------------------
    // Extract class name

    @Test
    public void test_extractClassName_singleLine() {
        String result = compiler.extractClassName("public void class Foo { }");
        Assert.assertEquals("Class name must match", "Foo", result);
    }

    @Test
    public void test_extractClassName_singleLineMultiSpaces() {
        String result = compiler.extractClassName("public         void          class      Foo       { }");
        Assert.assertEquals("Class name must match", "Foo", result);
    }

    @Test
    public void test_extractClassName_singleLineTab() {
        String result = compiler.extractClassName("public\tvoid\tclass\tFoo\t{ }");
        Assert.assertEquals("Class name must match", "Foo", result);
    }

    @Test
    public void test_extractClassName_multiLineWithSpace() {
        String result = compiler.extractClassName("public void class Foo {\n}");
        Assert.assertEquals("Class name must match", "Foo", result);
    }

    @Test
    public void test_extractClassName_multiLineWithoutSpace() {
        String result = compiler.extractClassName("public void class Foo{\n}");
        Assert.assertEquals("Class name must match", "Foo", result);
    }

    @Test
    public void test_extractClassName_multiLine() {
        String result = compiler.extractClassName("public void class Foo\n{\n}");
        Assert.assertEquals("Class name must match", "Foo", result);
    }

    @Test
    public void test_extractClassName_multiLineMultiTest() {
        String result = compiler.extractClassName("public     void    class \t   Foo  \t   \n\t{ \t    \n    }");
        Assert.assertEquals("Class name must match", "Foo", result);
    }

    @Test
    public void test_extractClassName_extends() {
        String result = compiler.extractClassName("public void class Foo  \t   \n\t extends Base \t\n{ \t    \n    }");
        Assert.assertEquals("Class name must match", "Foo", result);
    }

    @Test
    public void test_extractClassName_implements() {
        String result = compiler.extractClassName("public void class Foo  \t   \n\t implements Base \t\n{ \t    \n    }");
        Assert.assertEquals("Class name must match", "Foo", result);
    }
    // ---------------------------------------------------------------------------
    // Compile

    @Test
    public void test_compile_helloWorld() throws Exception {
        String code = "public class HelloWorld { public HelloWorld() { System.out.println(\"Hello World!\");}}";
        Class<?> clazz = compiler.compile(code);
        Assert.assertNotNull("Class must not be null", clazz);
    }

    @After
    public void teardown() {

    }
}
