/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.parser.test;

import at.jku.mevss.anttracks.diff.AddressCollection;
import at.jku.mevss.anttracks.diff.ComplexAddressCollection;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;

public class ComplexAddressCollection_Test {

    private ComplexAddressCollection test;

    @Before
    public void init() {
        test = new ComplexAddressCollection("Test");
    }

    @Test
    public void test_Put_Single() {
        test.put(4);
        assertEquals(1, test.getObjectCount());
        assertTrue(test.stream().anyMatch(x -> x == 4));
    }

    @Test
    public void test_PutAndContains_Million() {
        for (int i = 1; i <= 1_000_000; i++) {
            test.put(i);
        }
        assertEquals(1_000_000, test.stream().count());
        for (int i = 1; i <= 1_000_000; i++) {
            assertTrue(test.contains(i));
        }
    }

    @Test
    public void test_Contains_Empty() {
        for (int i = 1; i <= 1_000_000; i++) {
            assertFalse(test.contains(i));
        }
    }

    /*
    @Test
    public void test_Remove_Empty() {
        for (int i = 1; i <= 1_000_000; i++) {
            assertFalse(test.remove(i));
        }
    }

    @Test
    public void test_PutAndRemove_Million() {
        for (int i = 1; i <= 1_000_000; i++) {
            test.put(i);
        }
        for (int i = 1; i <= 1_000_000; i++) {
            assertTrue(test.remove(i));
        }
    }
    */

    @Test
    public void test_Clone_Equals() {
        for (int i = 1; i <= 15_000; i++) {
            test.put(i);
        }

        AddressCollection cloned = test.clone();
        test.stream().forEach(x -> assertTrue(cloned.contains(x)));
        cloned.stream().forEach(x -> assertTrue(test.contains(x)));
    }

    @Test
    public void test_UnionWith_Empty() {
        AddressCollection other = new ComplexAddressCollection("Test");

        for (int i = 1; i <= 15_000; i++) {
            test.put(i);
        }

        other.unionWith(test);

        test.stream().forEach(x -> assertTrue(other.contains(x)));
        other.stream().forEach(x -> assertTrue(test.contains(x)));
    }

    @Test
    public void test_UnionWith_PartlyFilled() {
        AddressCollection other = new ComplexAddressCollection("Test");

        for (int i = 1; i <= 15_000; i++) {
            test.put(i);
        }
        for (int i = 7_000; i <= 20_000; i++) {
            other.put(i);
        }

        other.unionWith(test);

        test.stream().forEach(x -> assertTrue(other.contains(x)));
    }

    @Test
    public void test_IntersectWith_Empty() {
        AddressCollection other = new ComplexAddressCollection("Test");

        for (int i = 1; i <= 15_000; i++) {
            test.put(i);
        }

        other.intersectWith(other);

        assertEquals(0, other.getObjectCount());
    }

    @Test
    public void test_IntersectWith_PartlyFilled() {
        ComplexAddressCollection other = new ComplexAddressCollection("Test");

        for (int i = 1; i <= 15_000; i++) {
            test.put(i);
        }
        for (int i = 14_000; i <= 35_000; i++) {
            other.put(i);
        }

        other.intersectWith(test);
        other.stream().forEach(x -> assertTrue(test.contains(x)));
    }

    @Test
    public void test_Without_Empty() {
        ComplexAddressCollection other = new ComplexAddressCollection("Test");

        for (int i = 1; i <= 15_000; i++) {
            test.put(i);
        }

        other.without(test);
        assertEquals(0, other.getObjectCount());
    }

    @Test
    public void test_Without_PartlyFilled() {
        ComplexAddressCollection other = new ComplexAddressCollection("Test");

        for (int i = 1; i <= 15_000; i++) {
            test.put(i);
        }

        other.without(test);
        assertEquals(0, other.getObjectCount());
    }
}
