/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.parser.test;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierChain;
import at.jku.mevss.anttracks.classification.ConditionalClassifierList;
import at.jku.mevss.anttracks.classification.Filter;
import at.jku.mevss.anttracks.classification.nodes.MapGroupingNode;
import at.jku.mevss.anttracks.heap.ObjectStream;
import at.jku.mevss.anttracks.heap.ObjectVisitor;
import at.jku.mevss.anttracks.heap.objects.ArrayObjectInfo;
import at.jku.mevss.anttracks.heap.space.SpaceInfo;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Stream;

public class TestHeap implements ObjectStream {

    private final SpaceInfo singleSpace = new SpaceInfo("Default space");
    private final Set<ObjectID> objects = new HashSet<>();
    private final TestHeap last;
    private final boolean current;

    public TestHeap(TestHeap last, boolean current) {
        this.last = last;
        this.current = current;
    }

    public TestHeap(TestHeap last, boolean current, ObjectID... objects) {
        this(last, current);
        Arrays.stream(objects).forEach(this::add);
    }

    public TestHeap(TestHeap last, boolean current, ObjectID[]... objects) {
        this(last, current);
        Arrays.stream(objects).forEach(objs -> Arrays.stream(objs).forEach(this::add));
    }

    public void add(ObjectID obj) {
        objects.add(obj);
    }

    @Override
    public void forEach(ObjectVisitor visitor, ObjectVisitor.Settings visitorSettings) {
        if (current) {
            objects.stream()
                   .forEach(obj -> visitor.visit(obj.addr,
                                                 obj.info,
                                                 singleSpace,
                                                 obj.info.type,
                                                 obj.info.size,
                                                 obj.info.isArray(),
                                                 obj.info.isArray() ? ((ArrayObjectInfo) obj.info).getLength() : -1,
                                                 obj.info.allocationSite,
                                                 obj.pointedFrom,
                                                 obj.pointsTo,
                                                 obj.info.eventType,
                                                 obj.rootPtrs));
        } else {
            if (last != null) {
                last.forEach(visitor, visitorSettings);
            }
        }
    }

    public void updateAddress(int from, int to) {
        objects.stream().filter(x -> x.addr == from).findFirst().get().addr = to;
    }

    @Override
    public <I extends ObjectVisitor> List<I> forEachParallel(ThreadVisitorGenerator<I> threadLocalDataGenerator, ObjectVisitor.Settings visitorSettings) {
        throw new UnsupportedOperationException();
    }

    @Override
    public <A> Stream<A> map(Classifier<A> classifier) {
        return null;
    }

    @Override
    public TestHeap clone() {
        throw new UnsupportedOperationException();
    }

    @Override
    public void addLabListener(IterationListener ll) {
        // TODO Auto-generated method stub

    }

    @Override
    public void removeLabListener(IterationListener ll) {
        // TODO Auto-generated method stub

    }

    @Override
    public ObjectStream filter(Filter... filter) {
        return null;
    }

    @Override
    public ObjectStream sorted() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public MapGroupingNode groupMap(ClassifierChain classifier, ConditionalClassifierList conditionalClassifier) {
        return null;
    }

    @Override
    public MapGroupingNode groupMapParallel(ClassifierChain classifier, ConditionalClassifierList conditionalClassifier) {
        return null;
    }
}
