package at.jku.mevss.anttracks.util.test;

import at.jku.mevss.anttracks.util.AntTracksBitSet;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

public class AntTracksBitSetTest {

    private AntTracksBitSet bitSet;

    @Before
    public void setUp() {
        bitSet = new AntTracksBitSet();
    }

    @Test
    public void testSetAndGet() {
        bitSet.set(1066);
        bitSet.set(3);
        bitSet.set(0);

        assertTrue(bitSet.get(3));
        assertTrue(bitSet.get(1066));
        assertTrue(bitSet.get(0));
        Assert.assertEquals(3, bitSet.cardinality());
    }

    @Test
    public void testClear() {
        bitSet.set(3);
        bitSet.clear();
        Assert.assertEquals(0, bitSet.cardinality());
    }

    @Test
    public void testStream() {
        bitSet.set(3);
        bitSet.set(1066);
        bitSet.set(0);

        Assert.assertArrayEquals(new int[]{0, 3, 1066}, bitSet.stream().toArray());
    }

    @Test
    public void testUnset() {
        bitSet.clear(10);
        assertFalse(bitSet.get(10));

        bitSet.set(1066);
        bitSet.clear(1066);
        assertFalse(bitSet.get(1066));
    }

    @Test
    public void testCompact() {
        bitSet.set(1000);
        bitSet.set(2000);
        bitSet.set(10000);
        bitSet.clear(10000);
        bitSet.trim();

        assertTrue(bitSet.get(1000));
        assertTrue(bitSet.get(2000));
    }

    @Test
    public void testOr() {
        bitSet.set(5);
        bitSet.set(69);
        bitSet.set(134);
        bitSet.set(243);
        bitSet.set(1234);
        // cause empty words at beginning and end of array
        bitSet.clear(5);
        bitSet.clear(1234);

        AntTracksBitSet otherBS = new AntTracksBitSet();
        otherBS.set(0);
        otherBS.set(75);
        otherBS.set(133);
        otherBS.set(1066);
        otherBS.set(10000);

        otherBS.clear(0);
        otherBS.clear(10000);

        bitSet.or(otherBS);

        Assert.assertArrayEquals(new int[]{69, 75, 133, 134, 243, 1066}, bitSet.stream().toArray());
    }

}
