/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.scala.trace.parser.heap.diff.interop

import java.util.function.Consumer

object Interop {
    def classify(interval: IntervalInfo, classifiers: List[Classifier]): DiffGrouping = {
        val grouping: DiffGrouping = new DiffGrouping(Some(classify(interval.perm, classifiers)),
            Some(classify(interval.born, classifiers)), Some(classify(interval.died, classifiers, interval.getLastHeap.extendedCache)),
            Some(classify(interval.temp, classifiers, interval.getLastHeap.extendedCache)));
        return grouping;
    }

    def classify(collection: AddressCollection, classifiers: List[Classifier]): Grouping = {
        val iterationHelper = new AddressIterationHelper(collection)
        val matchingAddressFilter: HeapObjectFilter = new HeapObjectFilter() {
            def filter(address: Long, space: SpaceInfo, clazz: AllocatedType, size: Int, isArray: Boolean, arrayLength: Int,
                       allocationSite: AllocationSite, pointedFrom: Array[Long], pointsTo: Array[Long], eventType: EventType) = collection.containsWithoutSyncFullBinary(address)
        }
        val group: Grouping = Grouping()
        val stream = collection.getLastHeap.toObjectStream()
            .filter(matchingAddressFilter)
            .sorted()
            .forEach(new ObjectVisitor() {
                def visit(space: SpaceInfo, addr: Long, pointedFrom: Array[Long], pointsTo: Array[Long], objInfo: ObjectInfo) = {
                    val eObj: ExtendedObjectInfo = objInfo.asExtendedObjectInfo(collection.getLastHeap.extendedCache);
                    group(addr, space, eObj.`type`, eObj.size, eObj.isInstanceOf[ExtendedArrayObjectInfo],
                        eObj match {
                            case arr: ExtendedArrayObjectInfo => arr.length
                            case _ => -1
                        }, eObj.allocationSite,
                        pointedFrom, pointsTo, eObj.eventType, classifiers)
                }
            });

        group;
    }

    def classify(collection: MultipleObjectCollection, classifiers: List[Classifier], cache: ExtendedObjectInfoCache): Grouping = {
        val group = Grouping()
        val mapper = new java.util.function.Function[ObjectInfo, (ExtendedObjectInfo, Long)] {
            def apply(objInfo: ObjectInfo): (ExtendedObjectInfo, Long) = {
                (objInfo.asExtendedObjectInfo(cache), collection.getMap.get(objInfo).get())
            }
        }
        val fun = new Consumer[(ExtendedObjectInfo, Long)]() {
            def accept(data: (ExtendedObjectInfo, Long)) {
                group.classify(-1, null, data._1.`type`, data._1.size,
                    data._1.isInstanceOf[ExtendedArrayObjectInfo],
                    data._1 match {
                        case arr: ExtendedArrayObjectInfo => arr.length
                        case _ => -1
                    },
                    data._1.allocationSite, null, null, data._1.eventType, classifiers,
                    Seq(), data._2)
            }
        }
        collection.stream().map[(ExtendedObjectInfo, Long)](mapper).forEach(fun)
        group
    }
}