/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.parser.heap.playground

import java.io.File
import java.util.function.{Consumer, Predicate}

object HeapPlayground extends App {
    val basePath = "/home/markus/m3_memory_monitoring/AntTracks/Examples/PointsTo/example01_pointsTo_singleTraceFile";
    val metaDir = basePath + File.separator + Consts.ANT_META_DIRECTORY;
    val traceFile = new File(basePath + File.separator + "trace");

    val symbolsParser: SymbolsParser = new SymbolsParser(SymbolsFile.findSymbolsFileToTrace(traceFile),
        ClassDefinitionsFile.findClassDefinitionsFileToTrace(traceFile), null, Symbols.CALLCONTEXT_STATIC);
    val symbols: Symbols = symbolsParser.parse();
    val metaDataParser: HeapTraceParser = new HeapTraceParser(symbols, new MetaDataWriterConfig(metaDir, 100, false));
    metaDataParser.parse();

    val traceParser: HeapTraceParser = new HeapTraceParser(
        symbols,
        new MetaDataConfig(metaDir),
        new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, symbols.causes.getByName("System.gc()"), 1));
    val heap: Heap = traceParser.parse();

    val pred1 = new Predicate[HeapObject]() {
        def test(obj: HeapObject): Boolean = {
            obj.size > 400;
        }
    }

    val pred2 = new Predicate[HeapObject]() {
        def test(obj: HeapObject): Boolean = {
            obj.`type`.name.contains("data");
        }
    }

    val action = new Consumer[HeapObject]() {
        def accept(obj: HeapObject): Unit = {
            System.out.println(s"${obj.`type`.name}\n   Allocation Site: ${obj.allocationSite.callSites(0)}\n   Size: ${obj.size} bytes");
        }
    }

    System.out.println(heap.stream().parallel().count());
    heap.stream().parallel().filter(pred1).filter(pred2).forEach(action);
}