/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.chart.base;

import at.jku.mevss.trace.gui.model.ChartSelection;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

public class SelectedItemsList {
    private List<ChartSelection> items;

    public SelectedItemsList() {
        this.items = new ArrayList<>();
    }

    public void clear() {
        this.items.clear();
    }

    public boolean containsItemInAnySeries(int x) {
        return this.items.stream().anyMatch(i -> i.item == x);
    }

    public boolean containsItemInAnySeries(ChartSelection sel) {
        return containsItemInAnySeries(sel.item);
    }

    public boolean containsItem(int series, int x) {
        return this.items.stream().anyMatch(i -> i.item == x && i.series == series);
    }

    public boolean containsItem(ChartSelection sel) {
        return containsItem(sel.item, sel.series);
    }

    public void add(ChartSelection item) {
        this.items.add(item);
    }

    public List<ChartSelection> get() {
        Collections.sort(items, Comparator.comparingDouble(cs -> cs.x));
        return this.items;
    }

    public ChartSelection get(int index) {
        Collections.sort(items, Comparator.comparingDouble(cs -> cs.x));
        return this.items.get(index);
    }

    public void remove(int item) {
        this.items = this.items.stream().filter(selection -> selection.item != item).collect(Collectors.toList());
    }

    public void remove(ChartSelection sel) {
        this.items = this.items.stream().filter(selection -> selection.item != sel.item).collect(Collectors.toList());
    }

    public int size() {
        return this.items.size();
    }

    public void sort() {
        Collections.sort(this.items);
    }
}
