/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.chart.fxchart.xy.stackedareapane.base;

import at.jku.mevss.trace.gui.chart.base.ChartSynchronizer;
import at.jku.mevss.trace.gui.chart.fxchart.xy.base.XYBaseJavaFXChartPane;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import javafx.collections.ObservableList;
import javafx.scene.chart.XYChart;

import java.util.Comparator;

public abstract class StackedAreaJavaFXChartPane<DATA> extends XYBaseJavaFXChartPane<DATA> {

    public StackedAreaJavaFXChartPane() {
        FXMLUtil.load(this, StackedAreaJavaFXChartPane.class);
    }

    @Override
    public void init(ChartSynchronizer chartSynchronizer) {
        super.init(chartSynchronizer);
    }

    @Override
    public double getMaxYValue(ObservableList<XYChart.Series<Number, Number>> seriesList) {
        double maxYValue = Double.MIN_VALUE;

        XYChart.Series<Number, Number> longestSeries = seriesList.stream().max(Comparator.comparingInt(s -> s.getData().size())).get();

        if (longestSeries == null) {
            return Double.MIN_VALUE;
        }

        for (int i = 0; i < longestSeries.getData().size(); i++) {
            final int j = i;
            // sum up the y value at index i of each series in list
            Double stackedSum = seriesList.stream().mapToDouble(s -> s.getData().size() > 0 ? s.getData().get(j).getYValue().doubleValue() : new Double(0)).sum();

            if (stackedSum > maxYValue) {
                maxYValue = stackedSum;
            }
        }

        return maxYValue;
    }

    @Override
    public double getMinYValue(ObservableList<XYChart.Series<Number, Number>> seriesList) {
        double minYValue = Double.MAX_VALUE;

        XYChart.Series<Number, Number> longestSeries = seriesList.stream().max(Comparator.comparingInt(s -> s.getData().size())).get();

        if (longestSeries == null) {
            return Double.MAX_VALUE;
        }

        for (int i = 0; i < longestSeries.getData().size(); i++) {
            final int j = i;
            // sum up the y value at index i of each series in list
            Double stackedSum = seriesList.stream().mapToDouble(s -> s.getData().size() > 0 ? s.getData().get(j).getYValue().doubleValue() : new Double(0)).sum();

            if (stackedSum < minYValue) {
                minYValue = stackedSum;
            }
        }

        return minYValue;
    }
}