package at.jku.mevss.trace.gui.chart.jfreechart.base;

import at.jku.mevss.trace.gui.chart.base.AntChartPane;
import at.jku.mevss.trace.gui.chart.base.ChartSynchronizer;
import javafx.beans.property.IntegerProperty;
import javafx.beans.property.SimpleIntegerProperty;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.fx.overlay.CrosshairOverlayFX;
import org.jfree.chart.plot.Crosshair;
import org.jfree.chart.plot.Plot;
import org.jfree.data.xy.AbstractIntervalXYDataset;

import java.awt.*;

public class AntJFreeChart extends JFreeChart {

    private static final long serialVersionUID = -1738161593485497407L;
    private static final boolean CROSSHAIR_ENABLED = false;
    private final ChartSynchronizer synchronizer;
    private final AntChartPane<AbstractIntervalXYDataset, ?> chartPane;

    private CrosshairOverlayFX crosshairOverlay;
    private int crosshairSeries = -1;
    private int crosshairItem = -1;

    private IntegerProperty crosshairSeriesProperty = new SimpleIntegerProperty(-1);

    public IntegerProperty crosshairSeriesProperty() {return crosshairSeriesProperty;}

    public int getCrosshairSeries() { return crosshairSeriesProperty.get();}

    public void setCrosshairSeries(int series) {crosshairSeriesProperty.set(series);}

    private IntegerProperty crosshairItemProperty = new SimpleIntegerProperty(-1);

    public IntegerProperty crosshairItemProperty() {return crosshairItemProperty;}

    public int getCrosshairItem() { return crosshairItemProperty.get();}

    public void setCrosshairItem(int item) {crosshairItemProperty.set(item);}

    public AntJFreeChart(AntChartPane<AbstractIntervalXYDataset, ?> chartPane,
                         String title,
                         Font titleFont,
                         Plot plot,
                         boolean createLegend,
                         ChartSynchronizer synchronizer) {
        super(title, titleFont, plot, createLegend);
        this.chartPane = chartPane;
        this.synchronizer = synchronizer;
        if (CROSSHAIR_ENABLED) {
            crosshairOverlay = new CrosshairOverlayFX();
            Crosshair domainCrosshair = new Crosshair(1000);
            Crosshair rangeCrosshair = new Crosshair(1);
            crosshairOverlay.addDomainCrosshair(domainCrosshair);
            crosshairOverlay.addRangeCrosshair(rangeCrosshair);

            crosshairSeriesProperty().addListener((observableValue, oldValue, newValue) -> {
                crosshairSeries = newValue.intValue();
                updateCrosshair();
            });

            crosshairItemProperty().addListener((observableValue, oldValue, newValue) -> {
                crosshairItem = newValue.intValue();
                updateCrosshair();
            });
        }
    }

    public CrosshairOverlayFX getCrosshairOverlay() {
        return crosshairOverlay;
    }

    private void updateCrosshair() {
        // TODO: Too much overhead to repaint on every mouse move
        if (CROSSHAIR_ENABLED) {
            if (crosshairItem >= 0 && crosshairSeries >= 0) {
                double x = getXYPlot().getDataset().getX(crosshairSeries, crosshairItem).doubleValue();
                double y = getXYPlot().getDataset().getY(crosshairSeries, crosshairItem).doubleValue();
                getCrosshairOverlay().getDomainCrosshairs().get(0).setValue(x);
                getCrosshairOverlay().getRangeCrosshairs().get(0).setValue(y);
            } else {
                getCrosshairOverlay().getDomainCrosshairs().get(0).setValue(0);
                getCrosshairOverlay().getRangeCrosshairs().get(0).setValue(0);
            }

            System.out.println(String.format("Hover: %d %d", crosshairSeries, crosshairItem));
        }
    }

    public ChartSynchronizer getSynchronizer() {
        return synchronizer;
    }

    public String getChartID() {
        return chartPane.getPaneId();
    }
}