package at.jku.mevss.trace.gui.chart.jfreechart.base;

import at.jku.mevss.trace.gui.chart.base.AntChartPane;
import at.jku.mevss.trace.gui.chart.base.ChartSynchronizer;
import at.jku.mevss.trace.gui.utils.ColorBrewer;
import org.jfree.chart.ChartTheme;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.StandardChartTheme;
import org.jfree.chart.annotations.XYTitleAnnotation;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.block.BlockBorder;
import org.jfree.chart.labels.*;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StackedXYAreaRenderer;
import org.jfree.chart.renderer.xy.StackedXYAreaRenderer2;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.chart.title.LegendTitle;
import org.jfree.chart.title.TextTitle;
import org.jfree.chart.ui.RectangleAnchor;
import org.jfree.chart.ui.RectangleEdge;
import org.jfree.chart.ui.TextAnchor;
import org.jfree.chart.urls.XYURLGenerator;
import org.jfree.chart.util.SortOrder;
import org.jfree.data.xy.AbstractIntervalXYDataset;
import org.jfree.data.xy.DefaultTableXYDataset;
import org.jfree.data.xy.TableXYDataset;

import java.awt.*;
import java.awt.font.GlyphVector;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

public class JFreeChartFactory {
    private static final ChartTheme currentTheme = new StandardChartTheme("JFree");
    private static String fontName = "Palatino";

    public static AntJFreeChart createStackedXYAreaChart(AntChartPane<AbstractIntervalXYDataset, ?> chartPane,
                                                         String title,
                                                         String subTitle,
                                                         String xAxisLabel,
                                                         String yAxisLabel,
                                                         TableXYDataset dataset,
                                                         XYItemLabelGenerator labelGenerator,
                                                         XYToolTipGenerator toolTipGenerator,
                                                         XYURLGenerator urlGenerator,
                                                         ChartSynchronizer synchronizer) {
        // Copied from ChartFactory
        NumberAxis xAxis = new NumberAxis(xAxisLabel);
        xAxis.setAutoRangeIncludesZero(false);
        xAxis.setLowerMargin(0.0);
        xAxis.setUpperMargin(0.0);
        NumberAxis yAxis = new NumberAxis(yAxisLabel);
        //if (urls) {
        //    urlGenerator = new StandardXYURLGenerator();
        //}

        StackedXYAreaRenderer labelRenderer = new StackedXYAreaRenderer(StackedXYAreaRenderer.SHAPES) {
            private static final long serialVersionUID = -3889774254281282048L;
            private final Shape NULL_RECTANGLE = new Rectangle2D.Float(0.0f, 0.0f, 0.0f, 0.0f);

            @Override
            public Shape getItemShape(int series, int item) {
                if (labelGenerator != null) {
                    String label = labelGenerator.generateLabel(getPlot().getDataset(), series, item);
                    if (label != null && !label.trim().isEmpty()) {
                    /*
                    Font tr = new Font("TimesRoman", Font.PLAIN, 18);
                    Font trb = new Font("TimesRoman", Font.BOLD, 18);
                    Font tri = new Font("TimesRoman", Font.ITALIC, 18);
                    Font trbi = new Font("TimesRoman", Font.BOLD+Font.ITALIC, 18);
                    Font h = new Font("Helvetica", Font.PLAIN, 18);
                    Font c = new Font("Courier", Font.PLAIN, 18);
                    Font d = new Font("Dialog", Font.PLAIN, 18);
                    Font z = new Font("ZapfDingbats", Font.PLAIN, 18);
                    */

                        Font font = new Font("Helvetica", Font.PLAIN, 12);

                        BufferedImage img = new BufferedImage(1000, 1000, BufferedImage.TYPE_INT_ARGB);
                        Graphics2D g2 = img.createGraphics();

                        try {
                            GlyphVector vect = font.createGlyphVector(g2.getFontRenderContext(), label);
                            Shape shape = vect.getOutline(0f, (float) -vect.getVisualBounds().getY());

                            return shape;
                        } finally {
                            g2.dispose();
                        }
                    }
                }
                return NULL_RECTANGLE;
            }
        };
        labelRenderer.setDefaultItemLabelGenerator(labelGenerator);
        labelRenderer.setDefaultItemLabelsVisible(true);
        labelRenderer.setShapePaint(Color.BLACK);
        labelRenderer.setDefaultStroke(new BasicStroke(0.01f));
        labelRenderer.setDefaultOutlineStroke(new BasicStroke(0.01f));

        StackedXYAreaRenderer2 renderer = new StackedXYAreaRenderer2(toolTipGenerator, urlGenerator);
        AntJFreeChartXYPlot plot = new AntJFreeChartXYPlot(chartPane, dataset, xAxis, yAxis, renderer, synchronizer);
        plot.setOrientation(PlotOrientation.VERTICAL);
        plot.setRangeAxis(yAxis);  // forces recalculation of the axis range

        boolean legend = false;
        AntJFreeChart chart = new AntJFreeChart(chartPane, title, JFreeChart.DEFAULT_TITLE_FONT, plot, legend, synchronizer);
        currentTheme.apply(chart);
        // -----------------------------

        if (subTitle != null) {
            chart.addSubtitle(new TextTitle(subTitle, new Font(fontName, Font.PLAIN, 14)));
        }

        XYItemRenderer mainRenderer = plot.getRenderer();
        Color[] chartSeriesColors = chartPane.getSeriesColors() != null ?
                                    chartPane.getSeriesColors() :
                                    ColorBrewer.Set1.getColorPalette(ColorBrewer.Set1.getMaximumColorCount());
        for (int cId = 0; cId < 10; cId++) {
            renderer.setSeriesPaint(cId, chartSeriesColors[cId % chartSeriesColors.length]);
            renderer.setSeriesPositiveItemLabelPosition(cId, new ItemLabelPosition(ItemLabelAnchor.CENTER, TextAnchor.BOTTOM_LEFT, TextAnchor.BOTTOM_LEFT, 45));
        }
        plot.setRenderer(0, plot.new AntShapeRendererForStackedCharts(chart, chartPane, synchronizer));
        plot.setRenderer(1, labelRenderer);
        plot.setRenderer(2, mainRenderer);
        plot.setDataset(0, plot.getDataset());
        plot.setDataset(1, plot.getDataset());
        plot.setDataset(2, plot.getDataset());
        formatChart(chart, mainRenderer);

        return chart;
    }

    public static AntJFreeChart createLineXYChart(AntChartPane<AbstractIntervalXYDataset, ?> chartPane,
                                                  String title,
                                                  String subTitle,
                                                  String xAxisLabel,
                                                  String yAxisLabel,
                                                  DefaultTableXYDataset dataset,
                                                  XYItemLabelGenerator labelGenerator,
                                                  ChartSynchronizer synchronizer) {
        NumberAxis xAxis = new NumberAxis(xAxisLabel);
        xAxis.setAutoRangeIncludesZero(false);
        NumberAxis yAxis = new NumberAxis(yAxisLabel);
        XYItemRenderer mainRenderer = new XYLineAndShapeRenderer(true, false);
        mainRenderer.setDefaultItemLabelGenerator(labelGenerator);
        mainRenderer.setDefaultItemLabelsVisible(true);
        AntJFreeChartXYPlot plot = new AntJFreeChartXYPlot(chartPane, dataset, xAxis, yAxis, mainRenderer, synchronizer);
        plot.setOrientation(PlotOrientation.VERTICAL);
        mainRenderer.setDefaultToolTipGenerator(new StandardXYToolTipGenerator());
        //renderer.setURLGenerator(new StandardXYURLGenerator());
        boolean legend = false;
        AntJFreeChart chart = new AntJFreeChart(chartPane, title, JFreeChart.DEFAULT_TITLE_FONT, plot, legend, synchronizer);
        currentTheme.apply(chart);

        if (subTitle != null) {
            chart.addSubtitle(new TextTitle(subTitle, new Font(fontName, Font.PLAIN, 14)));
        }

        Color[] chartSeriesColors = chartPane.getSeriesColors() != null ?
                                    chartPane.getSeriesColors() :
                                    ColorBrewer.Set1.getColorPalette(ColorBrewer.Set1.getMaximumColorCount());
        for (int series = 0; series < 10; series++) {
            mainRenderer.setSeriesStroke(series, new BasicStroke(4));
            mainRenderer.setSeriesPaint(series, chartSeriesColors[series % chartSeriesColors.length]);
        }
        plot.setRenderer(0, plot.new AntShapeRendererForLineCharts(chart, chartPane, synchronizer));
        plot.setRenderer(1, mainRenderer);
        plot.setDataset(0, plot.getDataset());
        plot.setDataset(1, plot.getDataset());
        formatChart(chart, mainRenderer);

        return chart;
    }

    private static void formatChart(JFreeChart chart, XYItemRenderer mainRenderer) {
        chart.getTitle().setFont(new Font(fontName, Font.BOLD, 18));
        XYPlot plot = (XYPlot) chart.getPlot();
        plot.setDomainPannable(true);
        plot.setRangePannable(false);
        //plot.setDomainCrosshairVisible(true);
        //plot.setRangeCrosshairVisible(true);
        plot.getDomainAxis().setLowerMargin(0.0);
        plot.getDomainAxis().setLabelFont(new Font(fontName, Font.BOLD, 14));
        plot.getDomainAxis().setTickLabelFont(new Font(fontName, Font.PLAIN, 12));
        plot.getRangeAxis().setLabelFont(new Font(fontName, Font.BOLD, 14));
        plot.getRangeAxis().setTickLabelFont(new Font(fontName, Font.PLAIN, 12));
        //chart.getLegend().setHorizontalAlignment(HorizontalAlignment.CENTER);
        //LegendTitle legend = chart.getLegend();
        //legend.setPosition(RectangleEdge.RIGHT);

        LegendTitle legend = new LegendTitle(mainRenderer);
        legend.setBackgroundPaint(Color.WHITE);
        //legend.setFrame(new BlockBorder(Color.WHITE));
        legend.setPosition(RectangleEdge.BOTTOM);
        legend.setItemFont(new Font(fontName, Font.PLAIN, 14));
        legend.setFrame(BlockBorder.NONE);
        legend.setSortOrder(SortOrder.DESCENDING);

        XYTitleAnnotation ta = new XYTitleAnnotation(0.0, 1, legend, RectangleAnchor.TOP_LEFT);
        ta.setMaxWidth(0.45);
        chart.getXYPlot().addAnnotation(ta);

        plot.setRangeGridlinePaint(Color.BLACK);
        plot.setBackgroundPaint(Color.WHITE);
        chart.setBackgroundPaint(new Color(0, 0, 0, 1));
    }
}