/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.classification.classifier;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierChain;
import at.jku.mevss.anttracks.classification.ClassifierException;
import at.jku.mevss.anttracks.classification.Transformer;
import at.jku.mevss.anttracks.classification.annotations.ClassifierProperty;
import at.jku.mevss.anttracks.classification.nodes.GroupingNode;
import at.jku.mevss.anttracks.classification.nodes.ListGroupingNode;
import at.jku.mevss.anttracks.util.ApplicationStatistics;
import at.jku.mevss.anttracks.util.ImagePack;
import at.jku.mevss.anttracks.util.ParallelizationUtil;
import at.jku.mevss.trace.gui.utils.ImageUtil;

import java.util.BitSet;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public abstract class AbstractPointsToClosureTransformer extends Transformer {

    @ClassifierProperty(overviewLevel = 10)
    protected ClassifierChain classifiers = new ClassifierChain();

    public ClassifierChain getClassifiers() {
        return classifiers;
    }

    public void setClassifiers(ClassifierChain classifiers) {
        this.classifiers = classifiers;
    }

    @Override
    public ImagePack[] loadIcons() {
        return new ImagePack[]{ImageUtil.getResourceImagePack("Points to", "points_to.png")};
    }

    @Override
    public String title() {
        return Classifier.CLASSIFICATION_MODE == ClassificationMode.LIST ? "Points To (# Objects)" : "Pointers To (# Pointers)";
    }

    @Override
    public GroupingNode classify(GroupingNode transformerRoot) throws ClassifierException {
        class ThreadLocalListGroupingNodes {
            Map<Thread, ListGroupingNode> nodes = new ConcurrentHashMap<>();

            public ListGroupingNode get() {
                Thread t = Thread.currentThread();
                if (!nodes.containsKey(t)) {
                    nodes.put(t, new ListGroupingNode(null, 0, 0, null, transformerRoot.getKey(), fastHeap()));
                }
                return nodes.get(t);
            }

            public ListGroupingNode combine(ListGroupingNode into) {
                nodes.values().stream().forEach(into::merge);
                return into;
            }
        }

        ApplicationStatistics.Measurement m = ApplicationStatistics.getInstance().createMeasurement("fast heap: points to (closure)");
        BitSet closure = closure();
        ThreadLocalListGroupingNodes tl = new ThreadLocalListGroupingNodes();
        if (closure != null && closure.cardinality() > 0) {
            // TODO filters
            closure.stream().parallel().forEach(idx -> tl.get().classify(fastHeap(), idx, classifiers, null));
        }
        tl.combine((ListGroupingNode) transformerRoot);
        m.end();
        m.end();
        return transformerRoot;
    }

    protected abstract BitSet closure() throws ClassifierException;
}
