/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.classification.classifier;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.annotations.ClassifierProperty;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierSourceCollection;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierType;
import at.jku.mevss.anttracks.heap.IndexBasedHeap;
import at.jku.mevss.anttracks.heap.symbols.Symbols;
import at.jku.mevss.anttracks.util.ImagePack;
import at.jku.mevss.trace.gui.utils.ImageUtil;

import java.util.function.Supplier;

@at.jku.mevss.anttracks.classification.annotations.Classifier(
        name = "Address",
        desc = "This classifier distinguishes objects based on their address.",
        example = "[30,450,670]",
        type = ClassifierType.ONE,
        collection = ClassifierSourceCollection.ALL)
public class AddressClassifier extends Classifier<String> {

    @ClassifierProperty(overviewLevel = 10)
    public Classifier<?> additionalClassifier = null;

    public Classifier<?> getAdditionalClassifier() {
        return additionalClassifier;
    }

    public void setAdditionalClassifier(Classifier<?> additionalClassifier) {
        this.additionalClassifier = additionalClassifier;
        if (additionalClassifier != null) {
            additionalClassifier.setup(() -> symbols(), () -> fastHeap());
        }
    }

    @ClassifierProperty(overviewLevel = 10)
    public boolean prepend = true;

    public boolean getPrepend() {
        return prepend;
    }

    public void setPrepend(boolean prepend) {
        this.prepend = prepend;
    }

    @Override
    public void setup(Supplier<Symbols> symbolsSupplier, Supplier<IndexBasedHeap> fastHeapSupplier) {
        super.setup(symbolsSupplier, fastHeapSupplier);
        if (additionalClassifier != null) {
            additionalClassifier.setup(symbolsSupplier, fastHeapSupplier);
        }
    }

    @Override
    public String classify() {
        try {
            return String.format("%s [%,d] %s",
                                 prepend && additionalClassifier != null ? additionalClassifier.classify(index()) : "",
                                 address(),
                                 !prepend && additionalClassifier != null ? additionalClassifier.classify(index()) : "");
        } catch (Exception ex) {
            ex.printStackTrace();
            return "classification error";
        }

    }

    @Override
    public ImagePack[] loadIcons() {
        return new ImagePack[]{ImageUtil.getResourceImagePack("Address", "address.png")};
    }

}
