/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.classification.classifier;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierException;
import at.jku.mevss.anttracks.classification.annotations.ClassifierProperty;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierSourceCollection;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierType;
import at.jku.mevss.anttracks.heap.roots.RootPtr;

import java.util.List;

@at.jku.mevss.anttracks.classification.annotations.Classifier(name = "In Degree", desc = "This classifier distinguishes objects based on the number of pointer that " +
        "point to them" +
        ".", example = "4", type = ClassifierType.ONE, collection = ClassifierSourceCollection.FASTHEAP)
public class InDegreeClassifier extends Classifier<String> {

    @ClassifierProperty(overviewLevel = 10)
    private boolean checkForRootPointer = true;

    public boolean getCheckForRootPointer() {
        return checkForRootPointer;
    }

    public void setCheckForRootPointer(boolean showNulls) {
        this.checkForRootPointer = showNulls;
    }

    @Override
    public String classify() throws ClassifierException {
        boolean rootPointed = false;
        if (checkForRootPointer) {
            List<RootPtr> roots = rootPointers();
            rootPointed = roots != null && !roots.isEmpty();
        }
        int[] fromPointerIndices = pointedFromIndices();
        return (fromPointerIndices == null ? "Unknown" : String.valueOf(fromPointerIndices.length)) + (checkForRootPointer ?
                                                                                                       (rootPointed ? " (root pointed)" : " (not root pointed)") :
                                                                                                       "");
    }

    /*
    @Override
    public ImagePack[] loadIcons() {
        return new ImagePack[]{
                ImageUtil.getResourceImagePack("Address", "address.png")
        };
    }
    */
}
