/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.classification.classifier;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.annotations.ClassifierProperty;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierSourceCollection;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierType;
import at.jku.mevss.anttracks.util.ImagePack;
import at.jku.mevss.trace.gui.utils.ImageUtil;

@at.jku.mevss.anttracks.classification.annotations.Classifier(name = "Type", desc = "This classifier distinguishes objects based on their type", example = "java.lang" +
        ".String",
        type = ClassifierType.ONE, collection = ClassifierSourceCollection.ALL)
public class TypeClassifier extends Classifier<String> {

    @ClassifierProperty(overviewLevel = 10)
    private boolean showPackage = true;

    public boolean getShowPackage() {
        return showPackage;
    }

    public void setShowPackage(boolean showPackage) {
        this.showPackage = showPackage;
    }

    @ClassifierProperty(overviewLevel = 10)
    private boolean showMirrorType = false;

    public boolean getShowMirrorType() {
        return showMirrorType;
    }

    public void setShowMirrorType(boolean showMirrorType) {
        this.showMirrorType = showMirrorType;
    }

    @Override
    public ImagePack[] loadIcons() {
        return new ImagePack[]{ImageUtil.getResourceImagePack("Type", "type.png")};
    }

    @Override
    public String classify() {
        return type().getExternalName(!showPackage, showMirrorType);
    }
}
