/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.component.treetable;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.logging.Logger;

public abstract class AbstractTreeTableModel<T> implements TreeTableModelInterface<T> {

    private final Logger logger = Logger.getLogger("at.jku.mevss.trace.gui.detail.statistics.treetable.AbstractTreeTableModel");

    protected final TreeNode<T> root;
    private final List<TreeTableModelListener<T>> listeners = new CopyOnWriteArrayList<>();
    private final TreeNodeListener<T> nodeListener = new TreeNodeListener<T>() {
        @Override
        public void expanded(TreeNode<T> node) {
            for (int child = 0; child < node.getChildCount(); child++) {
                try {
                    node.children.get(child).addListener(nodeListener);
                } catch (NullPointerException npe) {
                    logger.warning("Listener could not be installed into child node #" + child + " of " + node + " [ChildrenCount: " + node.getChildCount() + "]\n" +
                                           npe);
                }
            }
            listeners.forEach(l -> l.expanded(node));
        }

        @Override
        public void collapsed(TreeNode<T> node) {
            listeners.forEach(l -> l.collapsed(node));
        }
    };

    public AbstractTreeTableModel(TreeNode<T> root) {
        this.root = root;

        // Forward information from each node listener to the
        // TreeTableModelListener
        if (this.root != null) {
            this.root.addListener(nodeListener);
        }

        this.root.setExpanded(false);
        this.root.setExpanded(true);
    }

    @Override
    public TreeNode<T> getRoot() {
        return root;
    }

    @Override
    public void addListener(TreeTableModelListener<T> l) {
        if (l != null) {
            listeners.add(l);
        }
    }

    @Override
    public void removeListener(TreeTableModelListener<T> l) {
        if (l != null) {
            listeners.remove(l);
        }
    }

    protected void fireStructureChange() {
        listeners.forEach(l -> l.structureChanged());
    }
}
