/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.component.treetable;

/**
 * Objects of this class represent the value for a TreeNode at a given column
 *
 * @param <T> The TreeNode's type this value belongs to
 * @author Markus Weninger
 */
public class TreeNodeValue<T> {

    public final TreeNode<T> node;
    public final Object value;
    @SuppressWarnings("unused")
    private final int column;

    public TreeNodeValue(TreeNode<T> node, Object value, int column) {
        this.node = node;
        this.value = value;
        this.column = column;
    }

    @Override
    public boolean equals(Object other) {
        if (this == other) {
            return true;
        } else if (other != null && getClass().equals(other.getClass())) {
            @SuppressWarnings("unchecked")
            TreeNodeValue<T> that = (TreeNodeValue<T>) other;
            return node == that.node && value.equals(that.value);
        } else {
            return false;
        }
    }

    @Override
    public int hashCode() {
        return value.hashCode();
    }

    @Override
    public String toString() {
        return value.toString();
    }
}
