/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.component.treetableview;

import javafx.scene.Node;

import java.util.Collection;
import java.util.function.Function;
import java.util.stream.Collectors;

public class AutomatedTreeItem<CONTAINER, DATA> extends AntTreeItem<DATA> {
    // private final boolean dataSet = false;
    // private final boolean subTreeLevelSet = false;
    // boolean iconSet = false;

    private final CONTAINER container;

    private final Function<CONTAINER, String> titleFunction;
    private final Function<CONTAINER, DATA> dataFunction;
    private final Function<CONTAINER, Collection<? extends CONTAINER>> childFunction;
    private final Function<CONTAINER, Integer> subTreeLevelFunction;
    private final Function<CONTAINER, Node> iconFunction;

    public AutomatedTreeItem(CONTAINER container,
                             Function<CONTAINER, String> titleFunction,
                             Function<CONTAINER, DATA> dataFunction,
                             Function<CONTAINER, Collection<? extends CONTAINER>> childFunction,
                             Function<CONTAINER, Integer> subTreeLevelFunction,
                             Function<CONTAINER, Node> iconFunction) {
        super(dataFunction.apply(container), iconFunction != null ? iconFunction.apply(container) : null);
        this.container = container;
        this.titleFunction = titleFunction;
        this.dataFunction = dataFunction;
        this.childFunction = childFunction;
        this.subTreeLevelFunction = subTreeLevelFunction;
        this.iconFunction = iconFunction;

        getChildren().addAll(childFunction.apply(container)
                                          .stream()
                                          .map(childContainer -> new AutomatedTreeItem<CONTAINER, DATA>(childContainer,
                                                                                                        titleFunction,
                                                                                                        dataFunction,
                                                                                                        childFunction,
                                                                                                        subTreeLevelFunction,
                                                                                                        iconFunction))
                                          .collect(Collectors.toList()));
    }

    public int getSubTreeLevel() {
        return subTreeLevelFunction.apply(container);
    }
}
