package at.jku.mevss.trace.gui.component.treetableview.cell;

import at.jku.mevss.anttracks.util.ApplicationStatistics;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import javafx.scene.control.TreeTableCell;

import java.lang.reflect.InvocationTargetException;
import java.util.logging.Logger;

public class AntCell<TREE_ITEM_TYPE, COLUMN_ITEM_TYPE, CELL_TYPE extends AntCellContent<COLUMN_ITEM_TYPE>> extends TreeTableCell<TREE_ITEM_TYPE, COLUMN_ITEM_TYPE> {

    Logger LOGGER = Logger.getLogger(AntCell.class.getSimpleName());

    CELL_TYPE content;
    COLUMN_ITEM_TYPE lastItem = null;

    public AntCell(Class<CELL_TYPE> clazz) {
        FXMLUtil.load(this, AntCell.class);
        try {
            content = clazz.getConstructor().newInstance();
            content.init();
        } catch (InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException | NoSuchMethodException | SecurityException e) {
            // TODO: Error handling
            e.printStackTrace();
        }
    }

    @Override
    protected void updateItem(COLUMN_ITEM_TYPE item, boolean empty) {
        ApplicationStatistics.Measurement m = ApplicationStatistics.getInstance().createMeasurement("AntCell: Update Item");
        if (item == lastItem) {
            ApplicationStatistics.getInstance().inc("antcell.earlyreturn");
            m.end();
            return;
        }
        super.updateItem(item, empty);

        if ((empty || item == null)) {
            if (getGraphic() != null) {
                setText(null);
                setGraphic(null);
                setTooltip(null);
            }
        } else {
            content.setInternal(this, item);
            setText(null);
            setGraphic(content);
            setTooltip(content.getTooltip(this, item));
        }
        lastItem = item;
        m.end();
    }
}



