package at.jku.mevss.trace.gui.component.treetableview.cell.barcell;

import at.jku.mevss.trace.gui.component.treetableview.AutomatedTreeItem;
import at.jku.mevss.trace.gui.component.treetableview.cell.AntCell;
import at.jku.mevss.trace.gui.component.treetableview.cell.AntCellContent;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import javafx.beans.property.DoubleProperty;
import javafx.beans.property.SimpleDoubleProperty;
import javafx.fxml.FXML;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.control.Label;
import javafx.scene.layout.StackPane;
import javafx.scene.shape.Rectangle;

import java.awt.*;

public abstract class BarCellContent<T> extends AntCellContent<T> {

    @FXML
    Label label;

    @FXML
    Rectangle filledRect;

    private static final Color[] percentageColors = {new Color(160, 192, 56), new Color(222, 129, 144),
                                                     // new Color(255, 204, 0), WTF YELLOW DOES NOT WORK!?
                                                     new Color(166, 217, 239),
                                                     // new Color( 255, 128, 0) WTF, ORANGE DOES NOT WORK EITHER!!??
                                                     // new Color(222, 128, 0) Aaaand also not working
                                                     new Color(222, 222, 100),};
    int colorId = -1;
    String textToShow = "";
    DoubleProperty fillRatioProperty = new SimpleDoubleProperty(0.0);

    public BarCellContent() {
        FXMLUtil.load(this, BarCellContent.class);
    }

    @Override
    public void init() {
        filledRect.widthProperty().bind(this.widthProperty().multiply(fillRatioProperty));
        filledRect.heightProperty().bind(this.heightProperty());
    }

    @Override
    public void set(AntCell<?, T, ?> parentCell, T item) {
        int oldColorId = colorId;
        if (parentCell.getTreeTableRow() != null && parentCell.getTreeTableRow().getTreeItem() != null && (parentCell.getTreeTableRow()
                                                                                                                     .getTreeItem() instanceof AutomatedTreeItem)) {
            colorId = ((AutomatedTreeItem<?, ?>) parentCell.getTreeTableRow().getTreeItem()).getSubTreeLevel() % percentageColors.length;
        } else {
            colorId = 0;
        }

        // Graphical
        StackPane.setAlignment(filledRect, Pos.CENTER_LEFT);
        StackPane.setMargin(filledRect, new Insets(0, 0, 0, 0));

        if (colorId != oldColorId) {
            // Change style only if necessary, CSS is expensive!
            Color color = percentageColors[colorId];
            filledRect.setFill(javafx.scene.paint.Color.color(color.getRed() / 255.0, color.getGreen() / 255.0, color.getBlue() / 255.0));
        }

        // Bar
        fillRatioProperty.setValue(defineFillRatio(item));

        // Text
        StackPane.setAlignment(label, Pos.CENTER_LEFT);
        StackPane.setMargin(label, new Insets(0, 0, 0, 0));

        textToShow = defineTextToShow(item);
        label.setText(textToShow);
    }

    protected abstract String defineTextToShow(T item);

    protected abstract double defineFillRatio(T item);
}
