package at.jku.mevss.trace.gui.component.treetableview.cell.plotablediffingdata;

import at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.diffing.DiffingStackedAreaJFreeChart;
import at.jku.mevss.trace.gui.component.treetableview.cell.AntCell;
import at.jku.mevss.trace.gui.component.treetableview.cell.AntCellContent;
import at.jku.mevss.trace.gui.frame.main.tab.heapdiffing.model.PlotableDiffingData;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import com.sun.javafx.scene.control.skin.Utils;
import javafx.beans.binding.ObjectBinding;
import javafx.beans.value.ChangeListener;
import javafx.fxml.FXML;
import javafx.scene.control.Label;
import javafx.scene.control.OverrunStyle;
import javafx.scene.control.Tooltip;
import javafx.scene.layout.ColumnConstraints;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.Pane;
import javafx.scene.layout.RowConstraints;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;

import java.lang.reflect.Method;
import java.text.DecimalFormat;

public class PlotableDiffingDataCellContent extends AntCellContent<PlotableDiffingData> {

    @FXML
    Label label;

    @FXML
    Pane diedPane;
    @FXML
    Pane permPane;
    @FXML
    Pane bornPane;
    @FXML
    Pane tempPane;

    @FXML
    GridPane mainPane;

    @FXML
    Label diedLabel;
    @FXML
    Label permLabel;
    @FXML
    Label bornLabel;
    @FXML
    Label tempLabel;

    @FXML
    ColumnConstraints tempColumnConstraint;
    @FXML
    ColumnConstraints leftEmptyColumnConstraint;
    @FXML
    ColumnConstraints diedColumnConstraint;
    @FXML
    ColumnConstraints permColumnConstraint;
    @FXML
    ColumnConstraints bornColumnConstraint;
    @FXML
    ColumnConstraints rightEmptyColumnConstraint;

    public static final java.awt.Color PERM_COLOR = DiffingStackedAreaJFreeChart.DIFFING_COLORS[0];
    public static final java.awt.Color DIED_COLOR = DiffingStackedAreaJFreeChart.DIFFING_COLORS[1];
    public static final java.awt.Color BORN_COLOR = DiffingStackedAreaJFreeChart.DIFFING_COLORS[2];
    public static final java.awt.Color TEMP_COLOR = DiffingStackedAreaJFreeChart.DIFFING_COLORS[4];

    private static final DecimalFormat format = new DecimalFormat("###,###.#");

    PlotableDiffingData item;

    ChangeListener<? super Number> diedWidthListener = (observableValue, oldV, newV) -> {
        if (newV.doubleValue() > 0 && item != null) {
            setText(diedLabel, item.died);
        }
    };

    ChangeListener<? super Number> permWidthListener = (observableValue, oldV, newV) -> {
        if (newV.doubleValue() > 0 && item != null) {
            setText(permLabel, item.perm);
        }
    };

    ChangeListener<? super Number> bornWidthListener = (observableValue, oldV, newV) -> {
        if (newV.doubleValue() > 0 && item != null) {
            setText(bornLabel, item.born);
        }
    };

    ChangeListener<? super Number> tempWidthListener = (observableValue, oldV, newV) -> {
        if (newV.doubleValue() > 0 && item != null) {
            setText(tempLabel, item.temp);
        }
    };

    private static Method computeClippedText = null;

    @SuppressWarnings("rawtypes")
    private void setText(Label label, double value) {
        try {
            if (computeClippedText == null) {
                computeClippedText = Utils.class.getDeclaredMethod("computeClippedText",
                                                                   new Class[]{Font.class, String.class, Double.TYPE, OverrunStyle.class, String.class});
                computeClippedText.setAccessible(true);
            }
        } catch (Throwable t) {
            // ignore silently
        }

        String text = value > 0 ? format.format(value) : "";
        label.textProperty().bind(new ObjectBinding<String>() {

            {
                label.maxWidthProperty().addListener((_0, _1, _2) -> invalidate());
            }

            @Override
            protected String computeValue() {
                try {
                    if (computeClippedText != null) {
                        String clipped = (String) computeClippedText.invoke(null,
                                                                            label.getFont(),
                                                                            text,
                                                                            label.getMaxWidth(),
                                                                            label.getTextOverrun(),
                                                                            label.getEllipsisString());
                        if (clipped.length() < text.length()) {
                            return "";
                        }
                    }
                } catch (Throwable t) {
                    //ignore silently
                }
                return text;
            }

        });
    }

    public PlotableDiffingDataCellContent() {
        FXMLUtil.load(this, PlotableDiffingDataCellContent.class);
    }

    private String hexColor(java.awt.Color color) {
        return String.format("#%02x%02x%02x", color.getRed(), color.getGreen(), color.getBlue());
    }

    @Override
    public void init() {
        diedLabel.maxWidthProperty().bind(diedPane.widthProperty());
        permLabel.maxWidthProperty().bind(permPane.widthProperty());
        bornLabel.maxWidthProperty().bind(bornPane.widthProperty());
        tempLabel.maxWidthProperty().bind(tempPane.widthProperty());

        diedLabel.textOverrunProperty().set(OverrunStyle.CLIP);
        permLabel.textOverrunProperty().set(OverrunStyle.CLIP);
        bornLabel.textOverrunProperty().set(OverrunStyle.CLIP);
        tempLabel.textOverrunProperty().set(OverrunStyle.CLIP);

        diedPane.setStyle("-fx-background-color: " + hexColor(DIED_COLOR));
        permPane.setStyle("-fx-background-color: " + hexColor(PERM_COLOR));
        bornPane.setStyle("-fx-background-color: " + hexColor(BORN_COLOR));
        tempPane.setStyle("-fx-background-color: " + hexColor(TEMP_COLOR));

        diedPane.widthProperty().addListener(diedWidthListener);
        permPane.widthProperty().addListener(permWidthListener);
        bornPane.widthProperty().addListener(bornWidthListener);
        tempPane.widthProperty().addListener(tempWidthListener);
    }

    @Override
    public void set(AntCell<?, PlotableDiffingData, ?> parentCell, PlotableDiffingData item) {
        this.item = item;

        switch (item.plotStyle) {
            case PermDiedBorn:
                final double full = item.maxDied + item.maxAfter;
                final double leftEmpty = (item.maxDied - item.died) / full;
                final double died = item.died / full;
                final double perm = item.perm / full;
                final double born = item.born / full;
                final double rightEmpty = (item.maxAfter - item.after) / full;

                // Reset
                tempColumnConstraint.setPercentWidth(100);
                leftEmptyColumnConstraint.setPercentWidth(100);
                diedColumnConstraint.setPercentWidth(100);
                permColumnConstraint.setPercentWidth(100);
                bornColumnConstraint.setPercentWidth(100);
                rightEmptyColumnConstraint.setPercentWidth(100);

                // Set
                tempColumnConstraint.setPercentWidth(0);
                leftEmptyColumnConstraint.setPercentWidth(leftEmpty * 100);
                diedColumnConstraint.setPercentWidth(died * 100);
                permColumnConstraint.setPercentWidth(perm * 100);
                bornColumnConstraint.setPercentWidth(born * 100);
                rightEmptyColumnConstraint.setPercentWidth(rightEmpty * 100);
                break;
            case Temp:
                final double temp = item.temp / item.maxTemp;

                // Reset
                tempColumnConstraint.setPercentWidth(100);
                leftEmptyColumnConstraint.setPercentWidth(100);
                diedColumnConstraint.setPercentWidth(100);
                permColumnConstraint.setPercentWidth(100);
                bornColumnConstraint.setPercentWidth(100);
                rightEmptyColumnConstraint.setPercentWidth(100);

                // Set
                tempColumnConstraint.setPercentWidth(temp * 100);
                leftEmptyColumnConstraint.setPercentWidth(0);
                diedColumnConstraint.setPercentWidth(0);
                permColumnConstraint.setPercentWidth(0);
                bornColumnConstraint.setPercentWidth(0);
                rightEmptyColumnConstraint.setPercentWidth(0);
                break;
        }
    }

    @Override
    public Tooltip getTooltip(AntCell<?, PlotableDiffingData, ?> parentCell, PlotableDiffingData item) {
        GridPane gridPane = new GridPane();
        gridPane.getColumnConstraints().add(new ColumnConstraints(100));
        gridPane.getColumnConstraints().add(new ColumnConstraints(100));
        gridPane.getRowConstraints().add(new RowConstraints());
        gridPane.getRowConstraints().add(new RowConstraints());
        gridPane.getRowConstraints().add(new RowConstraints());
        gridPane.getRowConstraints().add(new RowConstraints());
        gridPane.getRowConstraints().stream().forEach(constraint -> constraint.setPercentHeight(25));

        Tooltip tooltip = new Tooltip();
        tooltip.setGraphic(gridPane);

        switch (item.plotStyle) {
            case PermDiedBorn:
                // Check if mouse hovers painted area
                // TODO: Make mouse position check
                /*
                if (x >= startXBorn * cellWidth && x <= endXBorn * cellWidth) {
                    return String.format("Born: %12s", format.format(data.born));
                }
                if (x >= startXPerm * cellWidth && x <= endXPerm * cellWidth) {
                    return String.format("Perm: %12s", format.format(data.perm));
                }
                if (x >= startXDied * cellWidth && x <= endXDied * cellWidth) {
                    return String.format("Died: %12s", format.format(data.died));
                }
                */

                Label diedTextLabel = new Label("Died:");
                diedTextLabel.setTextFill(Color.WHITE);
                GridPane.setConstraints(diedTextLabel, 0, 0);
                gridPane.getChildren().add(diedTextLabel);
                Label diedNumberLabel = new Label(format.format(item.died));
                diedNumberLabel.setTextFill(Color.WHITE);
                GridPane.setConstraints(diedNumberLabel, 1, 0);
                gridPane.getChildren().add(diedNumberLabel);

                Label permTextLabel = new Label("Perm:");
                permTextLabel.setTextFill(Color.WHITE);
                GridPane.setConstraints(permTextLabel, 0, 1);
                gridPane.getChildren().add(permTextLabel);
                Label permNumberLabel = new Label(format.format(item.perm));
                permNumberLabel.setTextFill(Color.WHITE);
                GridPane.setConstraints(permNumberLabel, 1, 1);
                gridPane.getChildren().add(permNumberLabel);

                Label bornTextLabel = new Label("Born:");
                bornTextLabel.setTextFill(Color.WHITE);
                GridPane.setConstraints(bornTextLabel, 0, 2);
                gridPane.getChildren().add(bornTextLabel);
                Label bornNumberLabel = new Label(format.format(item.born));
                bornNumberLabel.setTextFill(Color.WHITE);
                GridPane.setConstraints(bornNumberLabel, 1, 2);
                gridPane.getChildren().add(bornNumberLabel);

                if (item.before > 0) {
                    Label ratioTextLabel = new Label("After-Before-Ratio:");
                    ratioTextLabel.setTextFill(Color.WHITE);
                    GridPane.setConstraints(ratioTextLabel, 0, 3);
                    gridPane.getChildren().add(ratioTextLabel);
                    Label ratioNumberLabel = new Label(String.format("%+,.3f%%", ((100.0 * item.after) / (1.0 * item.before)) - 100.0));
                    ratioNumberLabel.setTextFill(Color.WHITE);
                    GridPane.setConstraints(ratioNumberLabel, 1, 3);
                    gridPane.getChildren().add(ratioNumberLabel);
                }
                break;

            case Temp:
                Label tempTextLabel = new Label("Temp:");
                tempTextLabel.setTextFill(Color.WHITE);
                GridPane.setConstraints(tempTextLabel, 0, 0);
                gridPane.getChildren().add(tempTextLabel);
                Label tempNumberLabel = new Label(String.format("%,3f", item.temp));
                tempNumberLabel.setTextFill(Color.WHITE);
                GridPane.setConstraints(tempNumberLabel, 1, 0);
                gridPane.getChildren().add(tempNumberLabel);
                break;

            default:
                return null;
        }
        return tooltip;
    }
}
