/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.dialog.preference;

import at.jku.mevss.trace.gui.utils.FXMLUtil;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.value.ChangeListener;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.DialogPane;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.cell.TextFieldTableCell;

import java.net.URL;
import java.util.Arrays;
import java.util.ResourceBundle;
import java.util.stream.Collectors;

public class PreferencesDialogPane extends DialogPane implements Initializable {

    @FXML
    private TableView<SettingsTableItem> tableView;
    @FXML
    private TableColumn<SettingsTableItem, String> nameTableColumn;
    @FXML
    private TableColumn<SettingsTableItem, String> valueTableColumn;

    private final Settings settings = Settings.getInstance();
    private final ObservableList<SettingsTableItem> data = FXCollections.observableArrayList(Arrays.stream(settings.getAll())
                                                                                                   .filter(p -> p.configurable)
                                                                                                   .map(settingsProperty -> new SettingsTableItem(settings,
                                                                                                                                                  settingsProperty))
                                                                                                   .collect(Collectors.toList()));

    public PreferencesDialogPane() {
        FXMLUtil.load(this, PreferencesDialogPane.class);
    }

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        nameTableColumn.setCellFactory(TextFieldTableCell.<SettingsTableItem>forTableColumn());
        nameTableColumn.setCellValueFactory(cell -> cell.getValue().descriptionProperty);
        valueTableColumn.setCellFactory(TextFieldTableCell.<SettingsTableItem>forTableColumn());
        valueTableColumn.setCellValueFactory(cell -> cell.getValue().valueProperty);
        tableView.setItems(data);
    }

    private class SettingsTableItem {
        private final SimpleStringProperty descriptionProperty;
        private final SimpleStringProperty valueProperty;

        private SettingsTableItem(Settings settings, Settings.Property<?> settingsProperty) {
            descriptionProperty = new SimpleStringProperty(settingsProperty.description);
            valueProperty = new SimpleStringProperty(settingsProperty.get().toString());
            valueProperty.addListener((ChangeListener<String>) (observable, oldValue, newValue) -> {
                settings.set(settingsProperty.name, newValue);
                valueProperty.set(settingsProperty.get().toString());
            });
        }
    }
}