/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main;

import at.jku.mevss.trace.gui.frame.main.component.applicationbase.ApplicationBaseTab;
import at.jku.mevss.trace.gui.frame.main.component.headerpane.HeaderPane;
import at.jku.mevss.trace.gui.frame.main.component.maintabbedpane.MainTabbedPane;
import at.jku.mevss.trace.gui.frame.main.component.menubar.AntTracksMenuBar;
import at.jku.mevss.trace.gui.frame.main.tab.application.ApplicationTab;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.utils.HeaderTextSupport;
import javafx.application.Platform;
import javafx.collections.ListChangeListener;
import javafx.embed.swing.SwingNode;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.Label;
import javafx.scene.control.Tab;
import javafx.scene.layout.Pane;
import javafx.scene.layout.VBox;

import java.util.logging.Logger;

public class MainFrame {

    @FXML
    private Node root;
    @FXML
    private AntTracksMenuBar menuBar;
    @FXML
    private HeaderPane headerPane;
    @FXML
    private MainTabbedPane mainTabbedPane;
    @FXML
    private VBox statusPane;

    private final static Logger LOGGER = Logger.getLogger(MainFrame.class.getSimpleName());

    public void init() {
        mainTabbedPane.init();

        updateHeaderPaneText(mainTabbedPane.getCurrentTab());

        mainTabbedPane.addSelectedItemListener((observable, oldValue, newValue) -> {
            updateHeaderPaneText(newValue);
        });

        mainTabbedPane.addSelectedIndexListener((observable, oldValue, newValue) -> {
            menuBar.setAppsEnabled(mainTabbedPane.getCloseableTabCount() > 0);
        });

        Label noOperationsLabel = new Label("No operations running");
        statusPane.getChildren().add(noOperationsLabel);
        statusPane.getChildren().addListener((ListChangeListener<Node>) c -> {
            Platform.runLater(() -> {
                if (statusPane.getChildren().isEmpty()) {
                    statusPane.getChildren().add(noOperationsLabel);
                } else {
                    if (statusPane.getChildren().size() > 1) {
                        statusPane.getChildren().removeIf(node -> node.equals(noOperationsLabel));
                    }
                }
            });
        });
    }

    private void updateHeaderPaneText(Tab currentTab) {
        if (currentTab instanceof HeaderTextSupport) {
            HeaderTextSupport headerTextSupport = (HeaderTextSupport) currentTab;
            headerPane.setText(headerTextSupport.getHeaderText());
        } else {
            Node tabContent = currentTab.getContent();
            if (tabContent instanceof SwingNode) {
                SwingNode swingNode = (SwingNode) tabContent;
                if (swingNode.getContent() instanceof HeaderTextSupport) {
                    HeaderTextSupport headerTextSupport = (HeaderTextSupport) swingNode.getContent();
                    headerPane.setText(headerTextSupport.getHeaderText());
                }
            }
        }
    }

    public void plotAllApps() {
        LOGGER.fine("Plot all apps");

        mainTabbedPane.getTabs().stream().filter(tab -> tab instanceof ApplicationTab).map(tab -> (ApplicationTab) tab).forEach(applicationTab -> applicationTab.plot());
    }

    public void addTab(Tab tab) {
        mainTabbedPane.addTab(tab);
    }

    public AppInfo getAppInfo() {
        if (mainTabbedPane.getCurrentTab() instanceof ApplicationBaseTab) {
            return ((ApplicationBaseTab) mainTabbedPane.getCurrentTab()).getAppInfo();
        }
        return null;
    }

    public Pane getStatusPane() {
        return statusPane;
    }

    public Node getRoot() {
        return root;
    }
}
