/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.component.applicationbase;

import at.jku.mevss.trace.gui.frame.main.component.headertext.HeaderTextTab;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.model.AppInfo.ChangeType;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import javafx.concurrent.Task;

import java.util.ArrayList;
import java.util.List;

public abstract class ApplicationBaseTab extends HeaderTextTab {

    protected AppInfo appInfo;
    public List<Task<?>> tasks;

    public ApplicationBaseTab() {
        FXMLUtil.load(this, ApplicationBaseTab.class);
    }

    private final AppInfo.AppInfoListener appInfoListener = type -> appInfoChangeAction(type);

    public void init(AppInfo appInfo) {
        this.tasks = new ArrayList<>();
        this.appInfo = appInfo;
        this.appInfo.addListener(appInfoListener);
        this.setOnClosed(event -> {
            tasks.forEach(Task::cancel);
            appInfo.removeListener(appInfoListener);
            cleanupOnClose();
        });
    }

    protected abstract void cleanupOnClose();

    protected abstract void appInfoChangeAction(ChangeType type);

    public AppInfo getAppInfo() {
        return appInfo;
    }
}
