/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.application.component.rootchartpane;

import at.jku.mevss.trace.gui.chart.base.AntChartPane;
import at.jku.mevss.trace.gui.chart.base.ApplicationChartFactory;
import at.jku.mevss.trace.gui.chart.base.ApplicationChartFactory.ApplicationChartType;
import at.jku.mevss.trace.gui.chart.base.ChartSynchronizer;
import at.jku.mevss.trace.gui.frame.main.GUI;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.model.AppInfo.AliveDeadPanelType;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import at.jku.mevss.trace.gui.utils.PlatformUtil;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.Priority;

import java.util.logging.Logger;

public class RootChartPane extends GridPane {
    @SuppressWarnings("unused")
    private final Logger LOGGER = Logger.getLogger(this.getClass().getSimpleName());
    private ChartSynchronizer chartSynchronizer;

    // Object & Memory Panels (1 + 2)
    private AntChartPane<?, AppInfo> objectsApplicationChart;
    private AntChartPane<?, AppInfo> bytesApplicationChart;
    // GC type panel (3)
    private AntChartPane<?, AppInfo> gcTypesApplicationChart;
    // Alive / Dead panels (3 alternatives) (4)
    private AntChartPane<?, AppInfo> aliveDeadApplicationChart;
    private AntChartPane<?, AppInfo> aliveDeadRelativeApplicationChart;
    private AntChartPane<?, AppInfo> aliveDeadObjectsApplicationChart;
    private AntChartPane<?, AppInfo> aliveDeadBytesApplicationChart;
    // // Feature panels (5 + 6)
    private AntChartPane<?, AppInfo> featureObjectsApplicationChart;
    private AntChartPane<?, AppInfo> featureBytesApplicationChart;
    // // Extended info panels (7 + 8)
    private AntChartPane<?, AppInfo> allocatingSubsystemApplicationChart;
    private AntChartPane<?, AppInfo> objectKindsApplicationChart;

    private boolean showFeatures = false;
    private boolean showExtended = false;

    public RootChartPane() {
        FXMLUtil.load(this, RootChartPane.class);
    }

    public void init(ChartSynchronizer chartSynchronizer) {
        this.chartSynchronizer = chartSynchronizer;

        createAllBasicChartPanels();
    }

    @SuppressWarnings("unchecked")
    public final void plot(AppInfo appInfo) {
        boolean somethingChanged = false;
        somethingChanged |= showFeatureChartPanels(appInfo.isShowFeatures());
        somethingChanged |= showExtendedInfo(GUI.CLIENT_INFO.isExtendedChartVisibility());
        somethingChanged |= setAliveDeadPanel(appInfo.getAliveDeadPanelType());
        if (somethingChanged) {
            reorderGrid();
        }

        getChildren().stream().map(node -> (AntChartPane<?, AppInfo>) node).forEach(chart -> {
            try {
                chart.update(appInfo);
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
    }

    private boolean setAliveDeadPanel(AliveDeadPanelType aliveDeadPanelType) {
        AntChartPane<?, AppInfo> newPanel = null;
        switch (aliveDeadPanelType) {
            case RELATIVE:
                newPanel = aliveDeadRelativeApplicationChart;
                break;
            case N_OBJECTS:
                newPanel = aliveDeadObjectsApplicationChart;
                break;
            case BYTE:
                newPanel = aliveDeadBytesApplicationChart;
                break;
        }
        if (aliveDeadApplicationChart != newPanel) {
            aliveDeadApplicationChart = newPanel;
            return true;
        }
        return false;
    }

    private void createAllBasicChartPanels() {
        objectsApplicationChart = ApplicationChartFactory.createChart(ApplicationChartType.OBJECTS, chartSynchronizer);
        bytesApplicationChart = ApplicationChartFactory.createChart(ApplicationChartType.BYTES, chartSynchronizer);
        gcTypesApplicationChart = ApplicationChartFactory.createChart(ApplicationChartType.GC_TYPES, chartSynchronizer);
        aliveDeadRelativeApplicationChart = ApplicationChartFactory.createChart(ApplicationChartType.ALIVE_DEAD_RELATIVE, chartSynchronizer);
        aliveDeadObjectsApplicationChart = ApplicationChartFactory.createChart(ApplicationChartType.ALIVE_DEAD_OBJECTS, chartSynchronizer);
        aliveDeadBytesApplicationChart = ApplicationChartFactory.createChart(ApplicationChartType.ALIVE_DEAD_BYTES, chartSynchronizer);
        setAliveDeadPanel(AliveDeadPanelType.RELATIVE);
        featureObjectsApplicationChart = ApplicationChartFactory.createChart(ApplicationChartType.FEATURE_OBJECTS, chartSynchronizer);
        featureBytesApplicationChart = ApplicationChartFactory.createChart(ApplicationChartType.FEATURE_BYTES, chartSynchronizer);
        allocatingSubsystemApplicationChart = ApplicationChartFactory.createChart(ApplicationChartType.ALLOCATING_SUBSYSTEM, chartSynchronizer);
        objectKindsApplicationChart = ApplicationChartFactory.createChart(ApplicationChartType.OBJECT_KINDS, chartSynchronizer);
        reorderGrid();

        // alternate colors for each chart row
        // TODO replace hardcoded colors with -fx-color-50/100
        /*
        //this alternating looks bad with only two rows, it makes one row stick out although there is nothing special about them ...
		//it only makes sense for 4+ rows, otherwise its not alternating but highlighting
		objectsApplicationChart.chart.setStyle("-fx-background-color: #eceff1;");
		bytesApplicationChart.chart.setStyle("-fx-background-color: #eceff1;");

		gcTypesApplicationChart.chart.setStyle("-fx-background-color:  #cfd8dc;");
		aliveDeadRelativeApplicationChart.chart.setStyle("-fx-background-color:  #cfd8dc;");
		aliveDeadObjectsApplicationChart.chart.setStyle("-fx-background-color:  #cfd8dc;");
		aliveDeadBytesApplicationChart.chart.setStyle("-fx-background-color:  #cfd8dc;");

		featureObjectsApplicationChart.chart.setStyle("-fx-background-color: #eceff1;");
		featureBytesApplicationChart.chart.setStyle("-fx-background-color: #eceff1;");

		allocatingSubsystemApplicationChart.chart.setStyle("-fx-background-color:  #cfd8dc;");
		objectKindsApplicationChart.chart.setStyle("-fx-background-color:  #cfd8dc;");
		 */
    }

    private void reorderGrid() {
        PlatformUtil.runAndWait(() -> {
            getChildren().clear();
            getRowConstraints().clear();

            addRow(getRowCount(), objectsApplicationChart, bytesApplicationChart);
            addRow(getRowCount(), gcTypesApplicationChart, aliveDeadApplicationChart);
            if (showFeatures) {
                addRow(getRowCount(), featureObjectsApplicationChart, featureBytesApplicationChart);
            }
            if (showExtended) {
                addRow(getRowCount(), allocatingSubsystemApplicationChart, objectKindsApplicationChart);
            }

            getChildren().forEach(child -> {
                setVgrow(child, Priority.ALWAYS);
                setHgrow(child, Priority.ALWAYS);
            });
        });
    }

    private int getRowCount() {
        return getChildren().size() / 2;
    }

    public boolean showFeatureChartPanels(boolean show) {
        if (showFeatures != show) {
            showFeatures = show;
            return true;
        }
        return false;
    }

    public boolean showExtendedInfo(boolean show) {
        if (showExtended != show) {
            showExtended = show;
            return true;
        }
        return false;
    }

    private void updateMarker() {
        getChildren().stream().map(node -> (AntChartPane<?, ?>) node).forEach(antChart -> antChart.updateMarker());
    }

    @SuppressWarnings("unchecked")
    private void zoomX(double lowerBound, double upperBound) {
        getChildren().stream().map(node -> (AntChartPane<?, AppInfo>) node).forEach(baseChartPane -> {
            if (baseChartPane.isZoomSynchronized()) {
                baseChartPane.zoomX(lowerBound, upperBound);
            }
        });
    }
}
