/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.heapdiffing;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierChain;
import at.jku.mevss.anttracks.classification.Filter;
import at.jku.mevss.trace.gui.chart.base.ApplicationChartFactory.MemoryConsumptionUnit;
import at.jku.mevss.trace.gui.chart.base.ChartSynchronizer;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.diffing.DiffingStackedAreaJFreeChart;
import at.jku.mevss.trace.gui.frame.main.component.applicationbase.ApplicationBaseTab;
import at.jku.mevss.trace.gui.frame.main.tab.heapdiffing.component.HeapDiffingConfigurationPane;
import at.jku.mevss.trace.gui.frame.main.tab.heapdiffing.component.heapstatisticstreetableview.DiffingTreeTableView;
import at.jku.mevss.trace.gui.frame.main.tab.heapdiffing.model.DiffingData;
import at.jku.mevss.trace.gui.model.AppInfo.ChangeType;
import at.jku.mevss.trace.gui.model.DiffStatisticsInfo;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import javafx.fxml.FXML;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.scene.layout.VBox;

import java.util.List;

public class HeapDiffingTab extends ApplicationBaseTab {

    @FXML
    private DiffingTreeTableView treeTableView;
    @FXML
    private VBox borderPane;

    @FXML
    private HeapDiffingConfigurationPane configurationPane;

    @FXML
    private HBox chartBox;
    @FXML
    private DiffingStackedAreaJFreeChart chartObjects;
    @FXML
    private DiffingStackedAreaJFreeChart chartBytes;

    private boolean isLoading;
    private DiffStatisticsInfo diffStatisticsInfo;

    private boolean showAsTextualTable = false;
    private boolean showObjects = true;
    private boolean showBytes = true;

    public HeapDiffingTab() {
        FXMLUtil.load(this, HeapDiffingTab.class);
    }

    public void init(DiffStatisticsInfo diffStatisticsInfo, long startTime, long endTime) {
        super.init(diffStatisticsInfo.getDiffInfo().getAppInfo());
        this.diffStatisticsInfo = diffStatisticsInfo;

        initializeTab();

        configurationPane.init(diffStatisticsInfo, this);

        initializeChart();
        initializeTable();

        setDefaultClassification();
    }

    private void initializeTab() {
        setText(String.format("Heap diff - %s @ %.3fs - %.3fs",
                              getAppInfo().getAppName(),
                              (diffStatisticsInfo.getDiffInfo().getTime(0) / 1000.0),
                              (diffStatisticsInfo.getDiffInfo().getTime(1) / 1000.0)));
        // setGraphic(ImageUtil.getIconNode(Consts.DIFFING_IMAGE));
        setHeaderText(String.format("Diffing - %s from %fs to %fs",
                                    getAppInfo().getAppName(),
                                    (diffStatisticsInfo.getDiffInfo().getTime(0) / 1000.0),
                                    (diffStatisticsInfo.getDiffInfo().getTime(1) / 1000.0)));
        setClosable(true);
    }

    private void initializeChart() {
        HBox.setHgrow(chartObjects, Priority.ALWAYS);
        HBox.setHgrow(chartBytes, Priority.ALWAYS);
        VBox.setVgrow(treeTableView, Priority.ALWAYS);
        ChartSynchronizer synchronizer = new ChartSynchronizer();
        chartObjects.init(synchronizer, MemoryConsumptionUnit.OBJECTS);
        chartBytes.init(synchronizer, MemoryConsumptionUnit.BYTES);
    }

    private void initializeTable() {
        treeTableView.init();
    }

    public void setLoadingMode(boolean isLoading) {
        this.isLoading = isLoading;
        configurationPane.setLoadingMode(isLoading);

        if (isLoading) {
            setAppropriateConfigView();
        }
    }

    public void updateChart(List<DiffingData> data) throws Exception {
        chartObjects.update(data);
        chartBytes.update(data);
    }

    public void setShowAsTextualTable(boolean show) {
        showAsTextualTable = show;
        updateTable();
    }

    public boolean isShowAsTextualTable() {
        return showAsTextualTable;
    }

    public boolean getShowObjects() {
        return showObjects;
    }

    public boolean getShowBytes() {
        return showBytes;
    }

    public void setShowObjects(boolean b) {
        showObjects = b;
        updateTable();
    }

    public void setShowBytes(boolean b) {
        showBytes = b;
        updateTable();
    }

    public void updateTable() {
        if (diffStatisticsInfo.getGrouping() == null) {
            return;
        }

        treeTableView.setRoot(diffStatisticsInfo.getGrouping());
    }

    public void acceptEdit() {
        ClassifierChain chain = new ClassifierChain(configurationPane.getClassifierSelectionPanel().getSelected());
        List<Filter> filter = configurationPane.getFilterSelectionPanel().getSelected();
        diffStatisticsInfo.setSelectedClassifier(chain);
        diffStatisticsInfo.setSelectedFilters(filter);
        diffStatisticsInfo.getSelectedConditionalClassifier().clear();
    }

    public void cancelEdit() {
        configurationPane.getClassifierSelectionPanel().resetSelected(diffStatisticsInfo.getSelectedClassifiers().getList());
    }

    public boolean isLoading() {
        return isLoading;
    }

    public void setAppropriateConfigView() {
        configurationPane.setAppropriateConfigView();
    }

    public void setDefaultClassification() {
        Classifier<?> types = diffStatisticsInfo.getDiffInfo().getClassifier("Type");
        Classifier<?> allocSites = diffStatisticsInfo.getDiffInfo().getClassifier("Allocation Site");
        Classifier<?> callSites = diffStatisticsInfo.getDiffInfo().getClassifier("Call Sites");
        if (types != null && allocSites != null && callSites != null) {
            diffStatisticsInfo.setSelectedClassifier(new ClassifierChain(types, allocSites, callSites));
        }
        dataToUI();
    }

    private void dataToUI() {
        configurationPane.getFilterSelectionPanel().resetSelected(diffStatisticsInfo.getSelectedFilters());
        configurationPane.getClassifierSelectionPanel().resetSelected(diffStatisticsInfo.getSelectedClassifiers().getList());
        setAppropriateConfigView();
    }

    @Override
    protected void appInfoChangeAction(ChangeType type) {
        // TODO Auto-generated method stub

    }

    @Override
    protected void cleanupOnClose() {

    }
}
