/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.heapdiffing.component;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierFactory;
import at.jku.mevss.anttracks.classification.Filter;
import at.jku.mevss.anttracks.classification.FilterFactory;
import at.jku.mevss.trace.gui.classification.component.configurationpane.ConfigurationPane;
import at.jku.mevss.trace.gui.classification.component.selectionpane.ClassificationSelectionPane;
import at.jku.mevss.trace.gui.classification.component.selectionpane.ClassificationSelectionPane.ClassificationSelectionListener;
import at.jku.mevss.trace.gui.frame.main.tab.heapdiffing.HeapDiffingTab;
import at.jku.mevss.trace.gui.model.DiffStatisticsInfo;

public class HeapDiffingConfigurationPane extends ConfigurationPane {

    private HeapDiffingTab heapDiffingTab;
    private HeapDiffingOperationPane actionPane;
    private DiffStatisticsInfo diffStatisticsInfo;

    public HeapDiffingConfigurationPane() {}

    public void init(DiffStatisticsInfo diffStatisticsInfo, HeapDiffingTab heapDiffingTab) {
        super.init(diffStatisticsInfo.getDiffInfo().getAppInfo(), diffStatisticsInfo.getDiffInfo(), diffStatisticsInfo);
        this.diffStatisticsInfo = diffStatisticsInfo;
        this.heapDiffingTab = heapDiffingTab;
        actionPane = new HeapDiffingOperationPane(diffStatisticsInfo, heapDiffingTab);
        getExtension().add(actionPane);
        getDataStructureSwitch().setDisable(true);
    }

    @Override
    protected ClassificationSelectionListener<Classifier<?>, ClassifierFactory> createClassifierListener() {
        return new ClassificationSelectionListener<Classifier<?>, ClassifierFactory>() {

            @Override
            public void selected(ClassificationSelectionPane<Classifier<?>, ClassifierFactory> sender, Classifier<?> x) {
                actionPane.setClassifierUpdateEnabled(true);
                actionPane.setExportEnabled(!heapDiffingTab.isLoading() && diffStatisticsInfo.getGrouping() != null);
            }

            @Override
            public void deselected(ClassificationSelectionPane<Classifier<?>, ClassifierFactory> sender, Classifier<?> x) {
                actionPane.setClassifierUpdateEnabled(sender.getSelected().size() > 0);
                actionPane.setExportEnabled(!heapDiffingTab.isLoading() && diffStatisticsInfo.getGrouping() != null);
            }

            @Override
            public void propertiesChanged(ClassificationSelectionPane<Classifier<?>, ClassifierFactory> sender, Classifier<?> x) {
                // Nothing to do if just property changed
            }
        };
    }

    @Override
    protected ClassificationSelectionListener<Filter, FilterFactory> createFilterListener() {
        return new ClassificationSelectionListener<Filter, FilterFactory>() {
            @Override
            public void selected(ClassificationSelectionPane<Filter, FilterFactory> sender, Filter x) {
                // TODO Auto-generated method stub
            }

            @Override
            public void deselected(ClassificationSelectionPane<Filter, FilterFactory> sender, Filter x) {
                // TODO Auto-generated method stub
            }

            @Override
            public void propertiesChanged(ClassificationSelectionPane<Filter, FilterFactory> sender, Filter x) {
                // TODO Auto-generated method stub
            }
        };
    }

    @Override
    public void setLoadingMode(boolean isLoading) {
        actionPane.setExportEnabled(!isLoading && diffStatisticsInfo.getGrouping() != null);
    }

    @Override
    public void setAppropriateConfigView() {
        if (diffStatisticsInfo != null && diffStatisticsInfo.getSelectedClassifiers().length() > 0) {
            if (isMaxMode) {
                // Switch to minimal view
                switchMode();
            }
        } else {
            if (!isMaxMode) {
                // Switch to max view
                switchMode();
            }
        }
    }

    @Override
    public void switchMode() {
        super.switchMode();
        if (actionPane != null) {
            actionPane.setVisible(isMaxMode);
            actionPane.setManaged(isMaxMode);
        }
    }

}
