/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.heapstate.tab.statistics;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierChain;
import at.jku.mevss.anttracks.classification.ConditionalClassifier;
import at.jku.mevss.anttracks.classification.Filter;
import at.jku.mevss.anttracks.classification.nodes.GroupingNode;
import at.jku.mevss.anttracks.classification.trees.ClassificationTree;
import at.jku.mevss.anttracks.util.ThreadUtil;
import at.jku.mevss.trace.gui.classification.dialog.properties.ClassificationPropertiesDialog;
import at.jku.mevss.trace.gui.component.treetable.TreeNode;
import at.jku.mevss.trace.gui.frame.main.component.applicationbase.ApplicationBaseTab;
import at.jku.mevss.trace.gui.frame.main.tab.heapstate.HeapStateTab;
import at.jku.mevss.trace.gui.frame.main.tab.heapstate.tab.statistics.component.classificationtreetableview.ClassificationTreeTableView;
import at.jku.mevss.trace.gui.frame.main.tab.heapstate.tab.statistics.component.configurationpane.HeapStatisticsConfigurationPane;
import at.jku.mevss.trace.gui.frame.main.tab.heapstate.tab.statistics.component.heapmetricstable.HeapMetricsTable;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.model.HeapStateStatisticsInfo;
import at.jku.mevss.trace.gui.model.SelectedClassifierInfo;
import at.jku.mevss.trace.gui.utils.Consts;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import at.jku.mevss.trace.gui.utils.ImageUtil;
import javafx.fxml.FXML;

import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.function.Consumer;
import java.util.logging.Logger;

public class HeapStatisticsTab extends ApplicationBaseTab implements HeapStateTab.HeapStateSubTab {

    @FXML
    private HeapStatisticsConfigurationPane configurationPane;
    @FXML
    private ClassificationTreeTableView treeTableView;
    @FXML
    private HeapMetricsTable heapMetricsTable;

    private SelectedClassifierInfo classifierConfigurationSavepoint;

    Logger LOGGER = Logger.getLogger(this.getClass().getSimpleName());

    private HeapStateStatisticsInfo statisticsInfo;

    private boolean isLoading;

    public HeapStatisticsTab() {
        FXMLUtil.load(this, HeapStatisticsTab.class);
    }

    @Override
    protected void appInfoChangeAction(AppInfo.ChangeType type) {
    }

	/*
    private final Consumer<MouseEvent> drillDown = me -> {
		Platform.runLater(() -> {
			ClassifierSelectionDialog dialog = new ClassifierSelectionDialog(statisticsInfo.getDetailsInfo(), statisticsInfo);
			dialog.showAndWait().ifPresent(result -> {
				if (dialog.getSelectedClassifiers() != null && dialog.getSelectedClassifiers().length() > 0) {
					addConditionalClassifierAndRunGroupingTask(
							table.getValueAt(table.getSelectionModel().getAnchorSelectionIndex(), 0).node, dialog.getSelectedClassifiers
							());
				}
			});
		});
	};
	*/

    private void addConditionalClassifierAndRunGroupingTask(TreeNode<GroupingNode> node, ClassifierChain objectClassifierChain) {
        List<String> conditions = new ArrayList<>();
        while (node != null) {
            conditions.add(0, node.getData().getKey().toString());
            node = node.getParent();
        }

        String[] conditionArray = conditions.toArray(new String[conditions.size()]);
        LOGGER.info(Arrays.toString(conditionArray));

        // Appending! Order of conditional classifiers is important (FIFC -
        // First in, first checked)
        statisticsInfo.getSelectedConditionalClassifier().add(new ConditionalClassifier() {
            @Override
            public Object[] getConditions() {
                return conditionArray;
            }

            @Override
            public ClassifierChain getClassifiers() {
                return objectClassifierChain;
            }
        });

        (new Thread((new GroupingTask(statisticsInfo, this)))).start();

        configurationPane.setAppropriateConfigView();
    }

    private final Consumer<MouseEvent> inspect = me -> {
        // TODO
        // int row = table.rowAtPoint(table.getContextMenuOpenedAtLocation());
        // TreeNode<HeapObjectGroupingNode> node = table.getValueAt(row, 0).node;
        // if (node == null) {
        // return;
        // }
        // List<TreeNode<HeapObjectGroupingNode>> nodes = new ArrayList<>();
        // nodes.add(node);
        //
        // while (node.getParent() != null) {
        // node = node.getParent();
        // nodes.add(0, node);
        // }
        //
        // List<HeapObjectFilter> filters = new ArrayList<>(statisticsInfo.getSelectedFilters());
        // filters.add(
        // new @Filter(name = "Inspection Filter", desc = "A filter dynamically created during inspection") HeapObjectFilter() {
        //
        // List<TreeNode<HeapObjectGroupingNode>> refNodes = nodes;
        // HeapObjectClassifierChain referenceClassifier = new HeapObjectClassifierChain(
        // statisticsInfo.getSelectedClassifiers());
        // ConditionalHeapObjectClassifierList referenceConditionalClassifier = new ConditionalHeapObjectClassifierList(
        // statisticsInfo.getSelectedConditionalClassifier());
        //
        // @Override
        // public boolean filter(long address, SpaceInfo space, AllocatedType type, int size, boolean isArray,
        // int arrayLength, AllocationSite allocationSite, long[] pointedFrom, long[] pointsTo,
        // EventType eventType) {
        // HeapObjectGroupingNode g = new HeapObjectMapGrouping();
        // g.classify(address, space, type, size, isArray, arrayLength, allocationSite, pointedFrom, pointsTo,
        // eventType, referenceClassifier, referenceConditionalClassifier);
        // List<HeapObjectGroupingNode> findMatch = new ArrayList<>();
        // findMatch.add(g);
        // for (int i = 0; i < refNodes.size(); i++) {
        // HeapObjectGroupingNode reference = refNodes.get(i).getData();
        // HeapObjectGroupingNode match = findMatch.stream()
        // .filter(possibleMatch -> reference.getClassifier().getName()
        // .equals(possibleMatch.getClassifier().getName())
        // && reference.getKey().equals(possibleMatch.getKey()))
        // .findFirst().orElse(null);
        //
        // if (match == null) {
        // return false;
        // }
        // findMatch.clear();
        // findMatch.addAll(match.getChildren());
        // }
        // return true;
        // }
        // });
        // DetailedStatisticsView inspectView = new
        // DetailedStatisticsView(new
        // StatisticsInfo(statisticsInfo.getDetailsInfo(),
        // new
        // HeapObjectClassifierChain(statisticsInfo.getSelectedClassifiers()),
        // new
        // ConditionalHeapObjectClassifierList(statisticsInfo.getSelectedConditionalClassifier()),
        // filters));
        //
        // HeapStatisticsTab inspectView = new HeapStatisticsTab(
        // new StatisticsInfo(statisticsInfo.getDetailsInfo(),
        // new HeapObjectClassifierChain(statisticsInfo.getSelectedClassifiers()),
        // new ConditionalHeapObjectClassifierList(statisticsInfo.getSelectedConditionalClassifier()), filters),
        // heapStateTab);
        // inspectView.setInfoClassification();
        // getTabPane().getTabs().add(inspectView);
        // // heapStateTab.addTab(inspectView);
        // // statisticsInfo.getDetailsInfo().getAppInfo().getClientInfo().getMainFrame().addTab(inspectView);
        // });
    };

    public void init(HeapStateStatisticsInfo statisticsInfo) {
        super.init(statisticsInfo.getDetailsInfo().getAppInfo());
        this.statisticsInfo = statisticsInfo;

        initializeTab();
        initializeConfigurationPane();
        initializeTable();
        initializeMetricsTable();
        initDataStructureSwitch();

        dataToUI();
    }

    private void initializeTab() {
        setGraphic(ImageUtil.getIconNode(Consts.STATISTICS_IMAGE));
    }

    private void initializeConfigurationPane() {
        configurationPane.init(statisticsInfo, this);
    }

    private void initializeTable() {
        treeTableView.init(appInfo,
                           configurationPane.getAvailableClassifierInfo(),
                           classifier -> ClassificationPropertiesDialog.showDialogForClassifier(classifier, statisticsInfo.getDetailsInfo(), this.getContent()),
                           (tab) -> {
                               if (!getTabPane().getTabs().contains(tab)) {
                                   getTabPane().getTabs().add(tab);
                               }
                           },
                           (tab) -> getTabPane().getSelectionModel().select(tab),
                           () -> getTabPane().getSelectionModel().select(this),
                           (newClassifierChain) -> configurationPane.getClassifierSelectionPanel().resetSelected(newClassifierChain.getList()));
    }

    private void initDataStructureSwitch() {
        // set initial state of DS switch according to given filters
        Filter onlyDSHeadsFilter = statisticsInfo.getDetailsInfo().getAvailableFilter().get("Only data structure heads");
        if(statisticsInfo.getSelectedFilters().removeAll(Collections.singleton(onlyDSHeadsFilter))) {
            configurationPane.getDataStructureSwitch().setSelected(true);
            // hidden filter is applied now
        }

        configurationPane.getDataStructureSwitch().selectedProperty().addListener((obs, wasSelected, isSelected) -> {
            if (wasSelected != isSelected) {
                // take currently selected classifiers and filters (including hidden ds heads filter)
                acceptEdit();

                // run grouping with modified filters
                GroupingTask groupingTask = new GroupingTask(statisticsInfo, this) {
                    @Override
                    public void succeeded() {
                        super.succeeded();
                        // update visible columns
                        treeTableView.getDataStructureSizeColumn().setVisible(isSelected);
                        treeTableView.getDeepDataStructureSizeColumn().setVisible(isSelected);
                        treeTableView.getShallowSizeColumn().setVisible(!isSelected);
                    }

                    @Override
                    public void failed() {
                        super.failed();
                        // reset ds switch
                        configurationPane.getDataStructureSwitch().setSelected(wasSelected);
                    }
                };
                tasks.add(groupingTask);
                ThreadUtil.startTask(groupingTask);
            }
        });
    }

    private void initializeMetricsTable() {
        heapMetricsTable.init();
    }

    public void updateGrouping(ClassificationTree classificationTree) {
        statisticsInfo.setGrouping(classificationTree);
        treeTableView.setRoot(statisticsInfo.getGrouping(), false, false);

        getHeapMetricsTable().treeNodesMetric.valueProperty.set(treeTableView.getClassificationTree().getnNodes());
        getHeapMetricsTable().dataTreeNodesMetric.valueProperty.set(treeTableView.getClassificationTree().getnDataNodes());
        getHeapMetricsTable().treeNodeDataCollectionPartsMetric.valueProperty.set(treeTableView.getClassificationTree().getnDataCollectionParts());

        getHeapMetricsTable().avgTreeNodeDataCollectionPartsPerNodeMetric.valueProperty.set(treeTableView.getClassificationTree()
                                                                                                         .getAvgTreeNodeDataCollectionPartsPerNode());
        getHeapMetricsTable().avgTreeNodeDataCollectionPartsPerDataNodeMetric.valueProperty.set(treeTableView.getClassificationTree()
                                                                                                             .getAvgTreeNodeDataCollectionPartsPerDataNode());
        getHeapMetricsTable().avgObjectsPerNodeMetric.valueProperty.set(treeTableView.getClassificationTree().getAvgObjectsPerNode());
        getHeapMetricsTable().avgObjectsPerDataNodeMetric.valueProperty.set(treeTableView.getClassificationTree().getAvgObjectsPerDataNode());
        getHeapMetricsTable().avgObjectsPerTreeNodeDataCollectionPartMetric.valueProperty.set(treeTableView.getClassificationTree()
                                                                                                           .getAvgObjectsPerTreeNodeDataCollectionPart());
    }

    public void setLoadingMode(boolean isLoading) {
        this.isLoading = isLoading;
        configurationPane.setLoadingMode(isLoading);
        treeTableView.setLoadingMode(isLoading);
        // Stay on current view if loading has finished (i.e., stay on maximized view if user switched to it during loading)
        if (isLoading) {
            setAppropriateConfigView();
        }
    }

    public void setAppropriateConfigView() {
        configurationPane.setAppropriateConfigView();
    }

    public void setDefaultClassification() {
        Classifier<?> types = statisticsInfo.getDetailsInfo().getClassifier("Type");
        Classifier<?> allocSites = statisticsInfo.getDetailsInfo().getClassifier("Allocation Site");
        Classifier<?> callSites = statisticsInfo.getDetailsInfo().getClassifier("Call Sites");
        if (types != null && allocSites != null && callSites != null) {
            statisticsInfo.setSelectedClassifier(new ClassifierChain(types, allocSites, callSites));
        }

    }

    public void applyClassifiers() {
        dataToUI();
        configurationPane.getOperationPanel().apply();
    }

    private void dataToUI() {
        configurationPane.getFilterSelectionPanel().resetSelected(statisticsInfo.getSelectedFilters());
        configurationPane.getClassifierSelectionPanel().resetSelected(statisticsInfo.getSelectedClassifiers().getList());
        setAppropriateConfigView();
    }

    public boolean isLoading() {
        return isLoading;
    }

    public void acceptEdit() {
        ClassifierChain chain = new ClassifierChain(configurationPane.getClassifierSelectionPanel().getSelected());
        List<Filter> filters = configurationPane.getFilterSelectionPanel().getSelected();
        statisticsInfo.setSelectedClassifier(chain);
        statisticsInfo.setSelectedFilters(filters);
        statisticsInfo.getSelectedConditionalClassifier().clear();
    }

    public void cancelEdit() {
        configurationPane.getFilterSelectionPanel().resetSelected(statisticsInfo.getSelectedFilters());
        configurationPane.getClassifierSelectionPanel().resetSelected(statisticsInfo.getSelectedClassifiers().getList());
    }

    public void setConfigurationSavepoint() {
        // successful grouping with current filter/classifier configuration
        classifierConfigurationSavepoint = new SelectedClassifierInfo(new ClassifierChain(configurationPane.getClassifierSelectionPanel().getSelected()),
                                                                      null,
                                                                      configurationPane.getFilterSelectionPanel().getSelected());
    }

    public void rollbackConfiguration() {
        // restore configuration to previous savepoint
        if (classifierConfigurationSavepoint != null) {
            configurationPane.getClassifierSelectionPanel().resetSelected(classifierConfigurationSavepoint.getSelectedClassifiers().getList());
            configurationPane.getFilterSelectionPanel().resetSelected(classifierConfigurationSavepoint.getSelectedFilters());
        }
    }

    public HeapMetricsTable getHeapMetricsTable() {
        return heapMetricsTable;
    }

    @Override
    protected void cleanupOnClose() {

    }

    @Override
    public void heapStateChanged() {
        // rerun classification
        configurationPane.getOperationPanel().apply();
    }
}
