/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.heapstate.tab.statistics;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierChain;
import at.jku.mevss.anttracks.classification.Filter;
import at.jku.mevss.anttracks.classification.nodes.GroupingNode;
import at.jku.mevss.anttracks.classification.nodes.IndexCollection;
import at.jku.mevss.anttracks.classification.trees.ListClassificationTree;
import at.jku.mevss.anttracks.heap.Closures;
import at.jku.mevss.anttracks.heap.IndexBasedHeap;
import at.jku.mevss.anttracks.heap.datastructures.DataStructure;
import at.jku.mevss.anttracks.heap.roots.RootPtr;
import at.jku.mevss.anttracks.util.Counter;
import at.jku.mevss.anttracks.util.Dot;
import at.jku.mevss.anttracks.util.ParallelizationUtil;
import at.jku.mevss.anttracks.util.ThreadUtil;
import at.jku.mevss.anttracks.util.Tuple;
import at.jku.mevss.trace.gui.classification.classifier.DirectGCRootsClassifier;
import at.jku.mevss.trace.gui.frame.main.component.applicationbase.ApplicationBaseTab;
import at.jku.mevss.trace.gui.frame.main.tab.heapstate.HeapStateTab;
import at.jku.mevss.trace.gui.frame.main.tab.heapstate.tab.statistics.component.classificationtreetableview.ClassificationTreeTableView;
import at.jku.mevss.trace.gui.frame.main.tab.heapstate.tab.statistics.component.heapmetricstable.HeapMetric;
import at.jku.mevss.trace.gui.graph.Graph;
import at.jku.mevss.trace.gui.graph.GraphVis;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.model.AvailableClassifierInfo;
import at.jku.mevss.trace.gui.utils.*;
import javafx.application.Platform;
import javafx.beans.binding.Bindings;
import javafx.beans.property.SimpleStringProperty;
import javafx.concurrent.Task;
import javafx.fxml.FXML;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.stage.Stage;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.stream.Collectors;

public class ObjectGroupInfoTab extends ApplicationBaseTab implements HeapStateTab.HeapStateSubTab {

    @FXML
    private ScrollPane objectGroupInfoTabSP;
    @FXML
    private VBox inspectedObjectGroupsVB;
    @FXML
    private TableView<Tuple<String, Number>> metricsTV;
    @FXML
    private TitledPane transitiveObjectsTP;
    @FXML
    private ClassificationTreeTableView transitiveObjectsTTV;
    @FXML
    private TitledPane retainedObjectsTP;
    @FXML
    private ClassificationTreeTableView retainedObjectsTTV;
    @FXML
    private TitledPane directRootsTP;
    @FXML
    private ClassificationTreeTableView directRootsTTV;
    @FXML
    private TitledPane indirectRootsTP;
    @FXML
    private ClassificationTreeTableView indirectRootsTTV;
    @FXML
    private TitledPane dataStructureHeadsTP;
    @FXML
    private ClassificationTreeTableView dataStructureHeadsTTV;

    @FXML
    private Button visPathsToAllRootsBT;
    // root path visualization options
    @FXML
    private CheckBox visPathsToAllRootsInvertEdgesCB;
    @FXML
    private CheckBox visPathsToAllRootsAggregateObjectsCB;
    @FXML
    private CheckBox visPathsToAllRootsAggregateDataStructuresCB;
    @FXML
    private CheckBox visPathsToAllRootsObjRootInfoCB;
    @FXML
    private CheckBox visPathToAllRootsObjIdxCB;
    @FXML
    private CheckBox visPathToAllRootsObjAddrCB;
    @FXML
    private CheckBox visPathToAllRootsObjTypeCB;
    @FXML
    private CheckBox visPathToAllRootsObjAllocSiteCB;
    @FXML
    private CheckBox visPathToAllRootsObjSizeCB;
    @FXML
    private CheckBox visPathToAllRootsObjDeepSizeCB;
    @FXML
    private CheckBox visPathToAllRootsObjRetainedSizeCB;
    @FXML
    private CheckBox visPathToAllRootsDSObjCountCB;
    @FXML
    private CheckBox visPathsToAllRootsStoreCB;
    @FXML
    private TextField visPathsToAllRootsFilenameTF;

    private IndexBasedHeap fastHeap;
    private GroupingNode[] nodes;
    private int[] objectGroup;
    private BitSet directRootObjectIndices;
    private BitSet indirectRootObjectIndices;
    private BitSet dataStructureHeadIndices;

    private AvailableClassifierInfo availableClassifierInfo;
    private Consumer<GroupingNode> showOriginalNode;
    private Function<Classifier<?>, Boolean> performClassifierSelection;
    private Consumer<ApplicationBaseTab> addTab;
    private Consumer<ApplicationBaseTab> switchToTab;

    public ObjectGroupInfoTab() {
        FXMLUtil.load(this, ObjectGroupInfoTab.class);
    }

    public void init(AppInfo appInfo,
                     GroupingNode[] nodes,
                     Function<Classifier<?>, Boolean> performClassifierSelection,
                     AvailableClassifierInfo availableClassifierInfo,
                     Consumer<ApplicationBaseTab> addTab,
                     Consumer<ApplicationBaseTab> switchToTab,
                     Consumer<GroupingNode> showOriginalNode,
                     boolean showTabImmediately) {
        if (nodes.length <= 0) {
            throw new IllegalArgumentException("Given node array must contain at least one element!");
        }

        super.init(appInfo);

        this.nodes = nodes;
        fastHeap = availableClassifierInfo.getFastHeapSupplier().get();
        this.availableClassifierInfo = availableClassifierInfo;
        this.showOriginalNode = showOriginalNode;
        this.performClassifierSelection = performClassifierSelection;
        this.addTab = addTab;
        this.switchToTab = switchToTab;
        directRootObjectIndices = IndexBasedHeap.bitSetSupplier.get();
        indirectRootObjectIndices = IndexBasedHeap.bitSetSupplier.get();
        dataStructureHeadIndices = IndexBasedHeap.bitSetSupplier.get();

        Task<Void> initTask = new InitTask(showTabImmediately);
        tasks.add(initTask);
        ParallelizationUtil.submitTask(initTask);
    }

    private void visualizePathsToAllRoots() {
        Stage stage = new Stage();
        BorderPane pane = new BorderPane();
        GraphVis<Integer> graphVis = new GraphVis<>();
        ScrollPane scroller = new ScrollPane(graphVis);
        Scene scene = new Scene(pane);

        pane.setCenter(scroller);
        graphVis.init(new Graph<>(
            // root node
            -1,

            // children
            idx -> {
                List<Integer> childObjects;

                if (idx == -1) {
                    // root node has initial object group as children
                    childObjects = Arrays.stream(objectGroup).boxed().collect(Collectors.toList());

                } else {
                    // other object's children are all their successor objects in their root paths
                    childObjects = fastHeap.traceAllRootsDFS(idx, Integer.MAX_VALUE, false).stream()
                                           .filter(rootInfo -> rootInfo.path.length > 1)
                                           .map(rootInfo -> rootInfo.path[1])
                                           .distinct()
                                           .collect(Collectors.toList());
                }

                if (visPathsToAllRootsAggregateObjectsCB.isSelected()) {
                    // map child objects to their data structures (i.e. the head objects)
                    childObjects = childObjects.stream()
                                               .map(objIndex -> {
                                                   DataStructure[] dataStructures = fastHeap.getDataStructures(objIndex,
                                                                                                               visPathsToAllRootsAggregateDataStructuresCB
                                                                                                                   .isSelected(),
                                                                                                               visPathsToAllRootsAggregateDataStructuresCB
                                                                                                                   .isSelected());
                                                   if (dataStructures != null) {
                                                       return Arrays.stream(dataStructures).mapToInt(ds -> ds.headIdx).toArray();
                                                   }

                                                   // objIndex is not part of a data structure => don't transform it
                                                   return new int[]{objIndex};
                                               })
                                               .flatMap(headIndices -> Arrays.stream(headIndices).boxed())
                                               .distinct()
                                               .collect(Collectors.toList());
                }

                return childObjects;
            },

            // node title
            idx -> idx == -1 ? "Selected Objects" : fastHeap.getType(idx).getExternalName(true, true),

            // node color
            idx -> idx == -1 ? Color.YELLOW : fastHeap.getRoot(idx) == null ? Color.WHITE : Color.RED,

            // tooltip
            idx -> idx == -1 ? "Click me to see the selected objects" : fastHeap.getType(idx).getExternalName(false, true),

            // node icon
            idx -> idx == -1 ? null : ImageUtil.getResourceImagePack("Type", "type.png").getAsNewNode(),

            // css
            idx -> null,

            // additional text
            idx -> idx == -1 ? null : getPathToRootNodeAdditionalText(idx)));

        stage.setTitle("Paths to all roots visualization");
        stage.setScene(scene);
        stage.show();
        stage.setMaximized(true);

        // old visualization task
        Task<Void> visTask = new AntTask<Void>() {

            Process xdot;

            @Override
            protected Void backgroundWork() throws Exception {
                updateTitle("Visualize paths to all roots");
                updateMessage("Tracing paths...");

                Path dotFile = Paths.get(visPathsToAllRootsFilenameTF.getText());
                Dot.DotBuilder dotBuilder = Dot.builder(dotFile);

                Arrays.stream(objectGroup).forEach(objIndex -> {
                    String[] initialNodeStrings = getPathToRootNodeStrings(objIndex);
                    Arrays.stream(initialNodeStrings).forEach(nodeString -> dotBuilder.addNode(nodeString, new String[]{"style=filled", "fillcolor=yellow"}));
                });

                List<RootPtr.RootInfo> pathsToAllRoots = fastHeap.traceAllRootsDFS(objectGroup, Integer.MAX_VALUE, false);

                updateMessage("Creating graph...");
                Counter progressCounter = new Counter();
                Set<Tuple<Integer, Integer>> closedSet = new HashSet<>();
                pathsToAllRoots.stream().map(rootInfo -> rootInfo.path).forEach(pathToRoot -> {
                    for (int i = 0; i < pathToRoot.length - 1; i++) {
                        int fromIdx = visPathsToAllRootsInvertEdgesCB.isSelected() ? pathToRoot[i + 1] : pathToRoot[i];
                        int toIdx = visPathsToAllRootsInvertEdgesCB.isSelected() ? pathToRoot[i] : pathToRoot[i + 1];

                        // don't process two index combinations twice! (paths to roots can be very overlapping)
                        if (closedSet.add(new Tuple<>(fromIdx, toIdx))) {
                            String[] fromNodes = getPathToRootNodeStrings(fromIdx);
                            String[] toNodes = getPathToRootNodeStrings(toIdx);

                            for (String fromNode : fromNodes) {
                                for (String toNode : toNodes) {
                                    dotBuilder.addEdge(fromNode, toNode);
                                }
                            }
                        }
                    }

                    progressCounter.inc();
                    updateProgress(progressCounter.get(), pathsToAllRoots.size());
                });

                try {
                    dotBuilder.build();
                    updateMessage("Opening visualization...");
                    xdot = Runtime.getRuntime().exec("xdot " + dotFile.toString());

                    // wait for process to finish
                    xdot.waitFor();

                    // delete graph file if no longer needed
                    if (!visPathsToAllRootsStoreCB.isSelected()) {
                        Files.deleteIfExists(dotFile);
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }

                return null;
            }

            @Override
            protected void suceeded() {

            }

            @Override
            protected void cancelled() {
                super.cancelled();
                if (xdot != null) {
                    // stop xdot process when vis task is cancelled
                    xdot.destroy();
                }
            }
        };

        //        tasks.add(visTask);
        //        ThreadUtil.startTask(visTask);
    }

    private String[] getPathToRootNodeStrings(int objIndex) {
        if (visPathsToAllRootsAggregateObjectsCB.isSelected()) {
            DataStructure[] dataStructures = fastHeap.getDataStructures(objIndex,
                                                                        visPathsToAllRootsAggregateDataStructuresCB.isSelected(),
                                                                        visPathsToAllRootsAggregateDataStructuresCB.isSelected());

            if (dataStructures != null && dataStructures.length > 0) {
                List<String> dsNodeStrings = new ArrayList<>();
                for (DataStructure ds : dataStructures) {
                    dsNodeStrings.add(getPathToRootNodeString(ds));
                }

                return dsNodeStrings.toArray(new String[dsNodeStrings.size()]);

            } else {
                // retrieve single object nodes instead
                return new String[]{getPathToRootNodeString(objIndex)};
            }

        } else {
            // retrieve single object nodes instead
            return new String[]{getPathToRootNodeString(objIndex)};
        }
    }

    private String getPathToRootNodeAdditionalText(int objIndex) {
        StringBuilder additionalTextBuilder = new StringBuilder();

        if (visPathsToAllRootsObjRootInfoCB.isSelected()) {
            if (fastHeap.getRoot(objIndex) != null) {
                fastHeap.getRoot(objIndex).forEach(rootPtr -> {
                    additionalTextBuilder.append("ROOT PTR: ").append(rootPtr.toShortString());
                    additionalTextBuilder.append("\n");
                });

                additionalTextBuilder.append("\n");
            }
        }
        if (visPathToAllRootsObjIdxCB.isSelected()) {
            additionalTextBuilder.append("IDX: ").append(objIndex).append("\n\n");
        }
        if (visPathToAllRootsObjAddrCB.isSelected()) {
            additionalTextBuilder.append("ADDR: ").append(fastHeap.getAddress(objIndex)).append("\n\n");
        }
        if (visPathToAllRootsObjAllocSiteCB.isSelected()) {
            additionalTextBuilder.append("ALLOCATION SITE: ").append(fastHeap.getAllocationSite(objIndex).callSites[0]).append("\n\n");
        }
        if (visPathToAllRootsObjSizeCB.isSelected()) {
            additionalTextBuilder.append("SHALLOW SIZE: ").append(fastHeap.getSize(objIndex)).append(" bytes").append("\n");
        }
        Closures closures = null;
        if (visPathToAllRootsObjDeepSizeCB.isSelected()) {
            closures = fastHeap.getClosures(objIndex);
            additionalTextBuilder.append("DEEP SIZE: ").append(closures.getTransitiveClosureByteCount()).append(" bytes").append("\n");
        }
        if (visPathToAllRootsObjRetainedSizeCB.isSelected()) {
            if (closures == null) {
                closures = fastHeap.getClosures(objIndex);
            }
            additionalTextBuilder.append("RETAINED SIZE: ").append(closures.getGCClosureByteCount()).append(" bytes").append("\n");
        }

        return additionalTextBuilder.toString();
    }

    private String getPathToRootNodeString(int objIndex) {
        StringBuilder nodeStringBuilder = new StringBuilder();

        if (fastHeap.getRoot(objIndex) != null) {
            nodeStringBuilder.append("ROOT OBJECT");
        } else {
            nodeStringBuilder.append("OBJECT");
        }

        if (visPathsToAllRootsObjRootInfoCB.isSelected()) {
            if (fastHeap.getRoot(objIndex) != null) {
                fastHeap.getRoot(objIndex).forEach(rootPtr -> {
                    nodeStringBuilder.append("\\n");
                    nodeStringBuilder.append("ROOT PTR: ").append(rootPtr.toShortString());
                });
            }
        }
        if (visPathToAllRootsObjIdxCB.isSelected()) {
            nodeStringBuilder.append("\\n");
            nodeStringBuilder.append("IDX: ").append(objIndex);
        }
        if (visPathToAllRootsObjAddrCB.isSelected()) {
            nodeStringBuilder.append("\\n");
            nodeStringBuilder.append("ADDR: ").append(fastHeap.getAddress(objIndex));
        }
        if (visPathToAllRootsObjTypeCB.isSelected()) {
            nodeStringBuilder.append("\\n");
            nodeStringBuilder.append("TYPE: ").append(fastHeap.getType(objIndex).getExternalName(false, true));
        }
        if (visPathToAllRootsObjAllocSiteCB.isSelected()) {
            nodeStringBuilder.append("\\n");
            nodeStringBuilder.append("ALLOCATION SITE: ").append(fastHeap.getAllocationSite(objIndex).callSites[0]);
        }
        if (visPathToAllRootsObjSizeCB.isSelected()) {
            nodeStringBuilder.append("\\n");
            nodeStringBuilder.append("SHALLOW SIZE: ").append(fastHeap.getSize(objIndex)).append(" bytes");
        }
        Closures closures = null;
        if (visPathToAllRootsObjDeepSizeCB.isSelected()) {
            closures = fastHeap.getClosures(objIndex);
            nodeStringBuilder.append("\\n");
            nodeStringBuilder.append("DEEP SIZE: ").append(closures.getTransitiveClosureByteCount()).append(" bytes");
        }
        if (visPathToAllRootsObjRetainedSizeCB.isSelected()) {
            if (closures == null) {
                closures = fastHeap.getClosures(objIndex);
            }
            nodeStringBuilder.append("\\n");
            nodeStringBuilder.append("RETAINED SIZE: ").append(closures.getGCClosureByteCount()).append(" bytes");
        }

        return nodeStringBuilder.toString();
    }

    private String getPathToRootNodeString(DataStructure ds) {
        StringBuilder nodeStringBuilder = new StringBuilder();

        if (fastHeap.getRoot(ds.headIdx) != null) {
            nodeStringBuilder.append("ROOT DATA STRUCTURE");
        } else {
            nodeStringBuilder.append("DATA STRUCTURE");
        }

        if (visPathsToAllRootsObjRootInfoCB.isSelected()) {
            if (fastHeap.getRoot(ds.headIdx) != null) {
                fastHeap.getRoot(ds.headIdx).forEach(rootPtr -> {
                    nodeStringBuilder.append("\\n");
                    nodeStringBuilder.append("ROOT PTR: ").append(rootPtr.toShortString());
                });
            }
        }
        if (visPathToAllRootsObjIdxCB.isSelected()) {
            nodeStringBuilder.append("\\n");
            nodeStringBuilder.append("HEAD IDX: ").append(ds.headIdx);
        }
        if (visPathToAllRootsObjAddrCB.isSelected()) {
            nodeStringBuilder.append("\\n");
            nodeStringBuilder.append("HEAD ADDR: ").append(fastHeap.getAddress(ds.headIdx));
        }
        if (visPathToAllRootsObjTypeCB.isSelected()) {
            nodeStringBuilder.append("\\n");
            nodeStringBuilder.append("TYPE: ").append(fastHeap.getType(ds.headIdx).getExternalName(false, true));
        }
        if (visPathToAllRootsObjAllocSiteCB.isSelected()) {
            nodeStringBuilder.append("\\n");
            nodeStringBuilder.append("ALLOCATION SITE: ").append(fastHeap.getAllocationSite(ds.headIdx).callSites[0]);
        }
        if (visPathToAllRootsDSObjCountCB.isSelected()) {
            nodeStringBuilder.append("\\n");
            nodeStringBuilder.append("DS OBJECT COUNT: ").append(ds.getObjectCount());
        }
        // TODO if needed can be implemented through ds visitor (...expensive)
        //        if(visPathToAllRootsObjSizeCB.isSelected()) {
        //            nodeStringBuilder.append("\\n");
        //            nodeStringBuilder.append("SHALLOW SIZE: ").append(fastHeap.getSize(objIndex)).append(" bytes");
        //        }
        //        Closures closures = null;
        //        if(visPathToAllRootsObjDeepSizeCB.isSelected()) {
        //            closures = fastHeap.getClosures(objIndex);
        //            nodeStringBuilder.append("\\n");
        //            nodeStringBuilder.append("DEEP SIZE: ").append(closures.getTransitiveClosureByteCount()).append(" bytes");
        //        }
        //        if(visPathToAllRootsObjRetainedSizeCB.isSelected()) {
        //            if(closures == null) {
        //                closures = fastHeap.getClosures(objIndex);
        //            }
        //            nodeStringBuilder.append("\\n");
        //            nodeStringBuilder.append("RETAINED SIZE: ").append(closures.getGCClosureByteCount()).append(" bytes");
        //        }

        return nodeStringBuilder.toString();
    }

    @Override
    protected void cleanupOnClose() {

    }

    @Override
    protected void appInfoChangeAction(AppInfo.ChangeType type) {

    }

    @Override
    public void heapStateChanged() {
        // reopen tab...
        ObjectGroupInfoTab updatedTab = new ObjectGroupInfoTab();
        updatedTab.init(appInfo, nodes, performClassifierSelection, availableClassifierInfo, addTab, switchToTab, showOriginalNode, false);

        // ...and close this one
        TabUtil.closeTab(this);
    }

    private class InitTask extends AntTask<Void> {

        private boolean showTabImmediately;

        public InitTask(boolean showTabImmediately) {
            this.showTabImmediately = showTabImmediately;
        }

        @Override
        protected Void backgroundWork() throws Exception {
            updateTitle("Calculate object group info");

            // init data
            updateMessage("Gather objects...");
            IndexCollection objectGroupIC = new IndexCollection(fastHeap);
            objectGroupIC.unionWith(Arrays.stream(nodes).map(node -> node.getData(true)).toArray(IndexCollection[]::new));
            objectGroup = new int[objectGroupIC.getCount()];
            for (int i = 0; i < objectGroup.length; i++) {
                objectGroup[i] = objectGroupIC.get(i);
            }
            updateMessage("Calculate closures...");
            Closures closures = fastHeap.getClosures(objectGroup);

            // closures
            ClassifierChain defaultClassifiers = new ClassifierChain();
            defaultClassifiers.getList().add(availableClassifierInfo.getClassifier("Type"));
            defaultClassifiers.getList().add(availableClassifierInfo.getClassifier("Allocation Site"));
            defaultClassifiers.getList().add(availableClassifierInfo.getClassifier("Call Sites"));
            updateMessage("Classify transitive objects with " + defaultClassifiers.toString() + "...");
            ListClassificationTree transitiveObjectsGrouping = initTransitiveObjectsGrouping(closures, defaultClassifiers);
            transitiveObjectsGrouping.initClosureTask().run();
            updateMessage("Classify retained objects with " + defaultClassifiers.toString() + "...");
            ListClassificationTree retainedObjectsGrouping = initRetainedObjectGrouping(closures, defaultClassifiers);
            retainedObjectsGrouping.initClosureTask().run();

            // data structures
            updateMessage("Gather and classify data structures ...");
            ListClassificationTree dataStructureHeadsGrouping = initDataStructureHeadsGrouping();
            dataStructureHeadsGrouping.initClosureTask().run();

            // roots
            ClassifierChain rootClassifiers = new ClassifierChain();
            DirectGCRootsClassifier directGCRootsClassifier = (DirectGCRootsClassifier) availableClassifierInfo.getClassifier("GC Roots: Direct");
            directGCRootsClassifier.setOnlyShowVariables(true);
            rootClassifiers.getList().add(directGCRootsClassifier);
            updateMessage("Gather and classify direct roots ...");
            ListClassificationTree directRootObjectsGrouping = initDirectRootObjectsGrouping(rootClassifiers);
            transitiveObjectsGrouping.initClosureTask().run();
            updateMessage("Gather and classify indirect roots ...");
            ListClassificationTree indirectRootObjectsGrouping = initIndirectRootObjectsGrouping(rootClassifiers);
            indirectRootObjectsGrouping.initClosureTask().run();

            if (!isCancelled()) {
                // init tab UI
                initUI(objectGroupIC,
                       closures,
                       transitiveObjectsGrouping,
                       retainedObjectsGrouping,
                       dataStructureHeadsGrouping,
                       directRootObjectsGrouping,
                       indirectRootObjectsGrouping);

                // show tab
                Platform.runLater(() -> {
                    updateMessage("Opening representation...");
                    addTab.accept(ObjectGroupInfoTab.this);
                    if (showTabImmediately) {
                        switchToTab.accept(ObjectGroupInfoTab.this);
                    }
                });
            }

            return null;
        }

        private ListClassificationTree initIndirectRootObjectsGrouping(ClassifierChain rootClassifiers) {
            Filter indirectGCRootFilter = new Filter() {
                @Override
                protected Boolean classify() throws Exception {
                    return indirectRootObjectIndices.get(index());
                }
            };
            indirectGCRootFilter.setup(() -> fastHeap.getSymbols(), () -> fastHeap);
            fastHeap.indirectGCRoots(objectGroup, Integer.MAX_VALUE).forEach(root -> indirectRootObjectIndices.set(root.getIdx()));
            Counter c = new Counter();
            return fastHeap.groupListParallel(new Filter[]{indirectGCRootFilter},
                                              rootClassifiers,
                                              null,
                                              objectCount -> {
                                                  c.add(objectCount);
                                                  updateProgress(c.get(), fastHeap.getObjectCount());
                                              },
                                              cancelProperty);
        }

        private ListClassificationTree initDirectRootObjectsGrouping(ClassifierChain rootClassifiers) {
            Filter directGCRootFilter = new Filter() {
                @Override
                protected Boolean classify() throws Exception {
                    return directRootObjectIndices.get(index());
                }
            };
            directGCRootFilter.setup(() -> fastHeap.getSymbols(), () -> fastHeap);
            Arrays.stream(objectGroup).forEach(objIndex -> {
                if (!isCancelled()) {
                    List<RootPtr> roots = fastHeap.getRoot(objIndex);
                    if (roots != null) {
                        directRootObjectIndices.set(objIndex);
                    }
                }
            });
            Counter c = new Counter();
            return fastHeap.groupListParallel(new Filter[]{directGCRootFilter},
                                              rootClassifiers,
                                              null,
                                              objectCount -> {
                                                  c.add(objectCount);
                                                  updateProgress(c.get(), fastHeap.getObjectCount());
                                              },
                                              cancelProperty);
        }

        private ListClassificationTree initDataStructureHeadsGrouping() {
            ClassifierChain dataStructureClassifiers = new ClassifierChain();
            dataStructureClassifiers.getList().add(availableClassifierInfo.getClassifier("Type"));
            Arrays.stream(objectGroup)
                  .filter(objIndex -> fastHeap.isDataStructureHead(objIndex))
                  .forEach(headIndex -> dataStructureHeadIndices.set(headIndex));
            Filter dataStructureHeadFilter = new Filter() {
                @Override
                protected Boolean classify() throws Exception {
                    return dataStructureHeadIndices.get(index());
                }
            };
            Counter c = new Counter();
            return fastHeap.groupListParallel(new Filter[]{dataStructureHeadFilter},
                                              dataStructureClassifiers,
                                              null,
                                              objectCount -> {
                                                  c.add(objectCount);
                                                  updateProgress(c.get(), fastHeap.getObjectCount());
                                              },
                                              cancelProperty);
        }

        private ListClassificationTree initRetainedObjectGrouping(Closures closures, ClassifierChain defaultClassifiers) {
            Filter retainedObjectFilter = new Filter() {
                @Override
                protected Boolean classify() throws Exception {
                    return closures.getGCClosure().get(index());
                }
            };
            retainedObjectFilter.setup(() -> fastHeap.getSymbols(), () -> fastHeap);
            Counter c = new Counter();
            return fastHeap.groupListParallel(new Filter[]{retainedObjectFilter},
                                              defaultClassifiers,
                                              null,
                                              objectCount -> {
                                                  c.add(objectCount);
                                                  updateProgress(c.get(), fastHeap.getObjectCount());
                                              },
                                              cancelProperty);
        }

        private ListClassificationTree initTransitiveObjectsGrouping(Closures closures, ClassifierChain defaultClassifiers) {
            Filter transitiveObjectFilter = new Filter() {
                @Override
                protected Boolean classify() throws Exception {
                    return closures.getTransitiveClosure().get(index());
                }
            };
            transitiveObjectFilter.setup(() -> fastHeap.getSymbols(), () -> fastHeap);
            Counter c = new Counter();
            return fastHeap.groupListParallel(new Filter[]{transitiveObjectFilter},
                                              defaultClassifiers,
                                              null,
                                              objectCount -> {
                                                  c.add(objectCount);
                                                  updateProgress(c.get(), fastHeap.getObjectCount());
                                              },
                                              cancelProperty);
        }

        private void initUI(IndexCollection objectGroupIC,
                            Closures closures,
                            ListClassificationTree transitiveObjectsGrouping,
                            ListClassificationTree retainedObjectsGrouping,
                            ListClassificationTree dataStructureHeadsGrouping,
                            ListClassificationTree directRootObjectsGrouping,
                            ListClassificationTree indirectRootObjectsGrouping) {
            Platform.runLater(() -> {
                // tab title
                StringBuilder tabTitle = new StringBuilder(nodes[0].getKey().toString());
                if (tabTitle.length() > 25) {
                    tabTitle.delete(25, tabTitle.length());
                    tabTitle.append("...");
                }
                if (nodes.length > 1) {
                    tabTitle.append(" and ").append(nodes.length - 1).append(" more");
                }
                ObjectGroupInfoTab.this.setText(tabTitle.toString());

                // inspected object groups
                Arrays.stream(nodes).forEach(node -> {
                    HBox hbox = new HBox();
                    hbox.setAlignment(Pos.CENTER);
                    hbox.getChildren().add(new Label(Arrays.stream(node.getFullKey()).reduce((key1, key2) -> key1 + " -> " + key2).get().toString()));
                    Button jumpBackToNode = new Button("?");
                    jumpBackToNode.setPadding(new Insets(5));
                    jumpBackToNode.setTranslateX(10);
                    jumpBackToNode.setOnAction(evt -> showOriginalNode.accept(node));
                    hbox.getChildren().add(jumpBackToNode);
                    inspectedObjectGroupsVB.getChildren().add(hbox);
                });

                // metrics
                TableColumn<Tuple<String, Number>, String> labelColumn = new TableColumn<>("Metric");
                TableColumn<Tuple<String, Number>, String> valueColumn = new TableColumn<>("Value");
                labelColumn.setCellValueFactory(param -> new SimpleStringProperty(param.getValue().a));
                valueColumn.setCellValueFactory(param -> new SimpleStringProperty(HeapMetric.formatter.format(param.getValue().b)));
                labelColumn.prefWidthProperty().bind(metricsTV.widthProperty().divide(2).subtract(10));
                valueColumn.prefWidthProperty().bind(metricsTV.widthProperty().divide(2).subtract(10));
                metricsTV.getColumns().add(labelColumn);
                metricsTV.getColumns().add(valueColumn);
                //metricsTV.setColumnResizePolicy(TableView.CONSTRAINED_RESIZE_POLICY);   // no horizontal scrollbar
                metricsTV.getItems().add(new Tuple<>("Shallow size [objects]", objectGroupIC.getCount()));
                metricsTV.getItems().add(new Tuple<>("Shallow size [bytes]", objectGroupIC.getBytes()));
                metricsTV.getItems().add(new Tuple<>("Deep size [objects]", closures.getTransitiveClosureObjectCount()));
                metricsTV.getItems().add(new Tuple<>("Deep size [bytes]", closures.getTransitiveClosureByteCount()));
                metricsTV.getItems().add(new Tuple<>("Retained size [objects]", closures.getGCClosureObjectCount()));
                metricsTV.getItems().add(new Tuple<>("Retained size [bytes]", closures.getGCClosureByteCount()));
                metricsTV.setFixedCellSize(25);
                metricsTV.prefHeightProperty().bind(metricsTV.fixedCellSizeProperty().multiply(Bindings.size(metricsTV.getItems()).add(1.25)));

                // closures
                transitiveObjectsTTV.init(appInfo,
                                          availableClassifierInfo,
                                          performClassifierSelection,
                                          addTab,
                                          switchToTab,
                                          () -> {
                                              addTab.accept(ObjectGroupInfoTab.this);
                                              switchToTab.accept(ObjectGroupInfoTab.this);
                                          },
                                          (newClassifierChain) -> {});
                transitiveObjectsTTV.setRoot(transitiveObjectsGrouping, false, false);
                retainedObjectsTTV.init(appInfo,
                                        availableClassifierInfo,
                                        performClassifierSelection,
                                        addTab,
                                        switchToTab,
                                        () -> {
                                            addTab.accept(ObjectGroupInfoTab.this);
                                            switchToTab.accept(ObjectGroupInfoTab.this);
                                        },
                                        (newClassifierChain) -> {});
                retainedObjectsTTV.setRoot(retainedObjectsGrouping, false, false);

                // data structure heads
                dataStructureHeadsTTV.init(appInfo,
                                           availableClassifierInfo,
                                           performClassifierSelection,
                                           addTab,
                                           switchToTab,
                                           () -> {
                                               addTab.accept(ObjectGroupInfoTab.this);
                                               switchToTab.accept(ObjectGroupInfoTab.this);
                                           },
                                           (newClassifierChain) -> {});
                dataStructureHeadsTTV.setRoot(dataStructureHeadsGrouping, false, false);
                dataStructureHeadsTTV.getShallowSizeColumn().setVisible(false);
                dataStructureHeadsTTV.getDataStructureSizeColumn().setVisible(true);
                dataStructureHeadsTTV.getDeepDataStructureSizeColumn().setVisible(true);

                // roots
                directRootsTTV.init(appInfo,
                                    availableClassifierInfo,
                                    performClassifierSelection,
                                    addTab,
                                    switchToTab,
                                    () -> {
                                        addTab.accept(ObjectGroupInfoTab.this);
                                        switchToTab.accept(ObjectGroupInfoTab.this);
                                    },
                                    (newClassifierChain) -> {});
                directRootsTTV.setRoot(directRootObjectsGrouping, false, false);

                indirectRootsTTV.init(appInfo,
                                      availableClassifierInfo,
                                      performClassifierSelection,
                                      addTab,
                                      switchToTab,
                                      () -> {
                                          addTab.accept(ObjectGroupInfoTab.this);
                                          switchToTab.accept(ObjectGroupInfoTab.this);
                                      },
                                      (newClassifierChain) -> {});
                indirectRootsTTV.setRoot(indirectRootObjectsGrouping, false, false);

                // allow scrolling of tree tables only when they are focused (and not just hovered)
                NodeUtil.ignoreScrollingUnlessFocused(transitiveObjectsTTV, objectGroupInfoTabSP);
                NodeUtil.ignoreScrollingUnlessFocused(retainedObjectsTTV, objectGroupInfoTabSP);
                NodeUtil.ignoreScrollingUnlessFocused(dataStructureHeadsTTV, objectGroupInfoTabSP);
                NodeUtil.ignoreScrollingUnlessFocused(directRootsTTV, objectGroupInfoTabSP);
                NodeUtil.ignoreScrollingUnlessFocused(indirectRootsTTV, objectGroupInfoTabSP);

                visPathsToAllRootsBT.setOnAction(evt -> visualizePathsToAllRoots());
                // disallow the combination of not aggregating objects but aggregating data structures:
                // disable aggregate ds checkbox when aggregate obj checkbox is not selected
                visPathsToAllRootsAggregateDataStructuresCB.disableProperty().bind(visPathsToAllRootsAggregateObjectsCB.selectedProperty().not());
                // unselect aggregate ds checkbox when aggregate obj checkbox is unselected
                visPathsToAllRootsAggregateObjectsCB.selectedProperty().addListener((obs, wasSelected, isSelected) -> {
                    if (!isSelected) {
                        visPathsToAllRootsAggregateDataStructuresCB.setSelected(false);
                    }
                });
                // disallow the combination of not aggregating objects but showing data structure size in nodes
                // disable show ds size checkbox when aggregate obj checkbox is not selected
                visPathToAllRootsDSObjCountCB.disableProperty().bind(visPathsToAllRootsAggregateObjectsCB.selectedProperty().not());
                // unselect show ds size checkbox when aggregate obj checkbox is unselected
                visPathsToAllRootsAggregateObjectsCB.selectedProperty().addListener((obs, wasSelected, isSelected) -> {
                    if (!isSelected) {
                        visPathToAllRootsDSObjCountCB.setSelected(false);
                    }
                });
                visPathsToAllRootsFilenameTF.setText("paths_to_all_roots_" + System.currentTimeMillis() + ".dot");
            });
        }

        @Override
        protected void suceeded() {
//
//            System.out.println("testing dominators...");
//            fastHeap.calculateDominators();
//            AntTracksDominators antTracksDominators = new AntTracksDominators(fastHeap);
//            AntTracksDominators2 antTracksDominators2 = new AntTracksDominators2(fastHeap);
//            fastHeap.stream().forEach(i -> {
//                if (antTracksDominators.getImmediateDominator(i) != fastHeap.getImmediateDominator(i) ||
//                    antTracksDominators2.getImmediateDominator(i) != fastHeap.getImmediateDominator(i)) {
//                    System.out.print(String.format("Immediate dominator missmatch for index %d: %d vs %d vs %d",
//                                                   i,
//                                                   fastHeap.getImmediateDominator(i),
//                                                   antTracksDominators.getImmediateDominator(i),
//                                                   antTracksDominators2.getImmediateDominator(i)));
//                    if (fastHeap.getRoot(i) != null) {
//                        System.out.println(" (rooted)");
//                    } else {
//                        System.out.println();
//                    }
//                }
//            });
//            System.out.println("done testing dominators!");

        }
    }
}
