/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.heapstate.tab.statistics.component.classificationtreetableview;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierChain;
import at.jku.mevss.anttracks.classification.nodes.GroupingNode;
import at.jku.mevss.anttracks.classification.nodes.ListGroupingNode;
import at.jku.mevss.anttracks.classification.trees.ClassificationTree;
import at.jku.mevss.anttracks.classification.trees.ListClassificationTree;
import at.jku.mevss.anttracks.heap.ObjectStream;
import at.jku.mevss.anttracks.util.ApplicationStatistics;
import at.jku.mevss.anttracks.util.Counter;
import at.jku.mevss.anttracks.util.ParallelizationUtil;
import at.jku.mevss.trace.gui.classification.classifier.LocalClassifier;
import at.jku.mevss.trace.gui.component.treetableview.AntTreeTableView;
import at.jku.mevss.trace.gui.component.treetableview.AutomatedTreeItem;
import at.jku.mevss.trace.gui.component.treetableview.cell.AntCell;
import at.jku.mevss.trace.gui.component.treetableview.cell.keycell.KeyCellContent;
import at.jku.mevss.trace.gui.frame.main.component.applicationbase.ApplicationBaseTab;
import at.jku.mevss.trace.gui.frame.main.tab.heapstate.tab.statistics.ObjectGroupInfoTab;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.model.ApproximateDouble;
import at.jku.mevss.trace.gui.model.AvailableClassifierInfo;
import at.jku.mevss.trace.gui.model.Percentage;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import com.sun.javafx.scene.control.skin.TreeTableViewSkin;
import com.sun.javafx.scene.control.skin.VirtualFlow;
import javafx.application.Platform;
import javafx.beans.binding.Bindings;
import javafx.beans.binding.NumberExpression;
import javafx.beans.binding.ObjectBinding;
import javafx.beans.property.ReadOnlyObjectWrapper;
import javafx.beans.property.ReadOnlyStringWrapper;
import javafx.beans.property.SimpleLongProperty;
import javafx.beans.value.ChangeListener;
import javafx.concurrent.Task;
import javafx.geometry.Insets;
import javafx.scene.Node;
import javafx.scene.control.*;
import javafx.scene.image.ImageView;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.scene.layout.*;
import javafx.scene.paint.Color;
import javafx.stage.Popup;
import javafx.util.Duration;
import org.controlsfx.control.Notifications;
import org.controlsfx.control.action.Action;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;

import static at.jku.mevss.trace.gui.frame.main.GUI.CLIENT_INFO;

public class ClassificationTreeTableView extends AntTreeTableView<GroupingNode> {
    private AutomatedTreeItem<GroupingNode, GroupingNode> root;

    // Timeline tableRefresher = null;

    private Function<GroupingNode, String> titleFunction = c -> c.getKey().toString();
    private Function<GroupingNode, GroupingNode> dataFunction = c -> c;
    private Function<GroupingNode, Collection<? extends GroupingNode>> childFunction = GroupingNode::getChildren;
    private Function<GroupingNode, Integer> subTreeLevelFunction = GroupingNode::getSubTreeLevel;
    // Root has no classifier, every other Grouping has a classifier
    private Function<GroupingNode, Node> iconFunction = c -> {
        List<ImageView> iconNodes = new ArrayList<>();
        if (c.getClassifier() != null) {
            ImageView iconNode = c.getClassifier().getIconNode(c.getKey());
            if (iconNode != null) {
                iconNodes.add(new ImageView(iconNode.getImage()));
            }
            if (c.getSubTreeLevel() > 0) {
                GroupingNode current = c.getParent();

                while (current.getParent() != null) {
                    if (current.getParent().getSubTreeLevel() != current.getSubTreeLevel()) {
                        iconNode = current.getClassifier().getIconNode(current.getKey());
                        if (iconNode != null) {
                            iconNodes.add(0, new ImageView(iconNode.getImage()));
                        }
                    }
                    current = current.getParent();
                }
            }
        }

        HBox hBox = new HBox(2, iconNodes.toArray(new ImageView[iconNodes.size()]));
        return hBox;
    };

    protected TreeTableColumn<GroupingNode, String> nameColumn = new TreeTableColumn<>("Name");

    protected TreeTableColumn<GroupingNode, ApproximateDouble> objectsColumn = new TreeTableColumn<>("Objects");
    protected TreeTableColumn<GroupingNode, ApproximateDouble> objectsCountColumn = new TreeTableColumn<>("#");
    protected TreeTableColumn<GroupingNode, Percentage> objectsPercentColumn = new TreeTableColumn<>("%");

    protected TreeTableColumn<GroupingNode, String> shallowSizeColumn = new TreeTableColumn<>("Shallow size");
    protected TreeTableColumn<GroupingNode, Number> bytesCountColumn = new TreeTableColumn<>("#");
    protected TreeTableColumn<GroupingNode, Percentage> bytesPercentColumn = new TreeTableColumn<>("%");

    protected TreeTableColumn<GroupingNode, String> transitiveClosureSizeColumn = new TreeTableColumn<>("Deep size");
    protected TreeTableColumn<GroupingNode, Number> closureSizeCountColumn = new TreeTableColumn<>("#");
    protected TreeTableColumn<GroupingNode, Percentage> closureSizePercentColumn = new TreeTableColumn<>("%");

    protected TreeTableColumn<GroupingNode, String> retainedSizeColumn = new TreeTableColumn<>("Retained size");
    protected TreeTableColumn<GroupingNode, Number> gcSizeCountColumn = new TreeTableColumn<>("#");
    protected TreeTableColumn<GroupingNode, Percentage> gcSizePercentColumn = new TreeTableColumn<>("%");

    protected TreeTableColumn<GroupingNode, String> dataStructureSizeColumn = new TreeTableColumn<>("Data structure size");
    protected TreeTableColumn<GroupingNode, Number> dataStructureSizeCountColumn = new TreeTableColumn<>("#");
    protected TreeTableColumn<GroupingNode, Percentage> dataStructureSizePercentColumn = new TreeTableColumn<>("%");

    protected TreeTableColumn<GroupingNode, String> deepDataStructureSizeColumn = new TreeTableColumn<>("Deep data structure size");
    protected TreeTableColumn<GroupingNode, Number> deepDataStructureSizeCountColumn = new TreeTableColumn<>("#");
    protected TreeTableColumn<GroupingNode, Percentage> deepDataStructureSizePercentColumn = new TreeTableColumn<>("%");

    private ClassificationTree classificationTree;

    // handle sorting of columns with expensive metrics
    private long lastSortingNotificationTime = 0;
    private ChangeListener<TreeTableColumn.SortType> closureSortListener = (obs, oldVal, newVal) -> {
        if (classificationTree != null && classificationTree instanceof ListClassificationTree) {
            ListClassificationTree tree = (ListClassificationTree) classificationTree;
            if (!tree.isClosuresCalculated() && System.currentTimeMillis() - lastSortingNotificationTime > 3_000) {
                org.controlsfx.control.Notifications.create()
                                                    .title("Sorting")
                                                    .text("Not all closures have been calculated yet!")
                                                    .hideAfter(new Duration(3_000))
                                                    .showWarning();
                lastSortingNotificationTime = System.currentTimeMillis();
            }
        }
    };

    private AppInfo appInfo;
    private AvailableClassifierInfo availableClassifierInfo;

    /**
     * Sets the classifierchain-property of a given classifier (local classifier/pointsto/pointedfrom) and returns a boolean indicating success
     */
    private Function<Classifier<?>, Boolean> selectLocalClassifiers;

    /**
     * Takes a tab and integrates it into the UI
     */
    private Consumer<ApplicationBaseTab> addTab;

    /**
     * Takes a tab and switches to it (if it exists)
     */
    private Consumer<ApplicationBaseTab> switchToTab;

    /**
     * When called makes sure that this treetableview becomes visible on the screen (e.g., by switching to the containing tab)
     */
    private Runnable showTTVOnScreen;

    /**
     * Adjusts the UI such that the given classifierchain is shown as the selected one
     */
    private Consumer<ClassifierChain> setSelectedClassifiers;

    private ContextMenu contextMenu = new ContextMenu();
    private MenuItem inspectObjectGroup = new MenuItem("Inspect object group");
    private MenuItem calculateClosuresNow = new MenuItem("Calculate closures now");
    private MenuItem localClassify = new MenuItem("Classify...");
    private MenuItem localPointsTo = new MenuItem("Points to...");
    private MenuItem localPointedFrom = new MenuItem("Pointed from...");
    private MenuItem localClassifyOnSameSubTreeLevel = new MenuItem("Classify on same sub-tree level... (experimental)");
    private MenuItem localPointsToOnSameSubTreeLevel = new MenuItem("Points to on same sub-tree level... (experimental)");
    private MenuItem localPointedFromOnSameSubTreeLevel = new MenuItem("Pointed from on same sub-tree level... (experimental)");
    private Menu localClassifyOnLowerSubTreeLevel = new Menu("Classify on lower sub-tree level... (experimental)");
    private Menu localPointsToOnLowerSubTreeLevel = new Menu("Points to on lower sub-tree level... (experimental)");
    private Menu localPointedFromOnLowerSubTreeLevel = new Menu("Pointed from on lower sub-tree level... (experimental)");

    public ClassificationTreeTableView() {
        FXMLUtil.load(this, ClassificationTreeTableView.class);
    }

    public void init(AppInfo appInfo,
                     AvailableClassifierInfo availableClassifierInfo,
                     Function<Classifier<?>, Boolean> selectLocalClassifiers,
                     Consumer<ApplicationBaseTab> addTab,
                     Consumer<ApplicationBaseTab> switchToTab,
                     Runnable showTTVOnScreen,
                     Consumer<ClassifierChain> setSelectedClassifiers) {
        this.appInfo = appInfo;
        this.availableClassifierInfo = availableClassifierInfo;
        this.selectLocalClassifiers = selectLocalClassifiers;
        this.addTab = addTab;
        this.switchToTab = switchToTab;
        this.showTTVOnScreen = showTTVOnScreen;
        this.setSelectedClassifiers = setSelectedClassifiers;

        // setup default context menu
        initContextMenu(availableClassifierInfo);

        // add textual search feature (CTRL+F)
        initSearchFunction();

        // Define columns
        initColumns();
    }

    private void initContextMenu(AvailableClassifierInfo availableClassifierInfo) {
        calculateClosuresNow.setOnAction(
                click -> getSelectionModel().getSelectedItems()
                                            .stream()
                                            .map(ti -> ti.getValue())
                                            .forEach(GroupingNode::fillClosureSizes));
        //.forEach(node -> ParallelizationUtil.submitFuture("Force closure calculation", () -> node.fillClosureSizes(), true)));
        inspectObjectGroup.setOnAction(click -> inspectObjectGroup(getSelectionModel().getSelectedItems()));
        localClassify.setOnAction(click -> handleLocalClassification(new ClassifierChain(new LocalClassifier())));
        localPointsTo.setOnAction(click -> handleLocalClassification(new ClassifierChain(availableClassifierInfo.getClassifier("Points To (Direct)"))));
        localPointedFrom.setOnAction(click -> handleLocalClassification(new ClassifierChain(availableClassifierInfo.getClassifier("Pointed From"))));
        localClassifyOnSameSubTreeLevel.setOnAction(click -> handleLocalClassification(new ClassifierChain(new LocalClassifier())));
        localPointsToOnSameSubTreeLevel.setOnAction(click -> handleLocalClassification(new ClassifierChain(availableClassifierInfo.getClassifier("Points To (Direct)"))));
        localPointedFromOnSameSubTreeLevel.setOnAction(click -> handleLocalClassification(new ClassifierChain(availableClassifierInfo.getClassifier("Pointed From"))));
        contextMenu.getItems()
                   .addAll(inspectObjectGroup,
                           calculateClosuresNow,
                           localClassify,
                           localPointsTo,
                           localPointedFrom,
                           localClassifyOnSameSubTreeLevel,
                           localPointsToOnSameSubTreeLevel,
                           localPointedFromOnSameSubTreeLevel,
                           localClassifyOnLowerSubTreeLevel,
                           localPointsToOnLowerSubTreeLevel,
                           localPointedFromOnLowerSubTreeLevel);
    }

    private void initColumns() {
        nameColumn.setCellValueFactory(param -> new ReadOnlyStringWrapper(param.getValue().getValue().getKey().toString()));
        nameColumn.setCellFactory(param -> new AntCell<>(KeyCellContent.class));

        objectsCountColumn.setCellValueFactory(param -> new ReadOnlyObjectWrapper<>(new ApproximateDouble(param.getValue().getValue().getObjectCount(),
                                                                                                          !param.getValue().getValue().isCalculatedOnSampling())));
        objectsPercentColumn.setCellValueFactory(param -> new ReadOnlyObjectWrapper<>(getPercentage(getParent(param.getValue().getValue()).getObjectCount(),
                                                                                                    param.getValue().getValue().getObjectCount())));
        objectsColumn.getColumns().add(objectsCountColumn);
        objectsColumn.getColumns().add(objectsPercentColumn);

        bytesCountColumn.setCellValueFactory(param -> new SimpleLongProperty(param.getValue().getValue().getByteCount()));
        bytesPercentColumn.setCellValueFactory(param -> new ReadOnlyObjectWrapper<>(getPercentage(getParent(param.getValue().getValue()).getByteCount(),
                                                                                                  param.getValue().getValue().getByteCount())));
        shallowSizeColumn.getColumns().add(bytesCountColumn);
        shallowSizeColumn.getColumns().add(bytesPercentColumn);

        closureSizeCountColumn.setCellValueFactory(param -> {
            ApplicationStatistics.getInstance().inc("closureSizeCountColumn CellValueFactory");

            ApplicationStatistics.Measurement minner = ApplicationStatistics.getInstance().createMeasurement("cellvaluefactory.closuresize.inner");
            if (param.getValue().getValue().getAndSetIsClosuresBeingCalculated(true) && !param.getValue().getValue().isClosureSizeCalculated()) {
                ParallelizationUtil.submitFuture("UI-triggered Transitive Closure Size Calculation for " + param.getValue().getValue().getKey(), () -> {
                    ApplicationStatistics.Measurement m = ApplicationStatistics.getInstance().createMeasurement("cellvaluefactory.closuresize.future");
                    //param.getValue().getValue().fillClosureSizes();
                    param.getValue().getValue().getAndSetIsClosuresBeingCalculated(false);
                    m.end();
                }, CLIENT_INFO.meterRegistry);
            }
            minner.end();
            return param.getValue().getValue().transitiveClosureSizeProperty();
        });
        closureSizePercentColumn.setCellValueFactory(param -> getPercentageCellValueFactory(param.getValue().getValue().transitiveClosureSizeProperty(),
                                                                                            getParent(param.getValue().getValue()).transitiveClosureSizeProperty()));

        transitiveClosureSizeColumn.getColumns().add(closureSizeCountColumn);
        transitiveClosureSizeColumn.getColumns().add(closureSizePercentColumn);

        gcSizeCountColumn.setCellValueFactory(param -> {
            ApplicationStatistics.getInstance().inc("gcSizeCountColumn CellValueFactory");

            ApplicationStatistics.Measurement minner = ApplicationStatistics.getInstance().createMeasurement("cellvaluefactory.gcSizeCount.inner");
            if (param.getValue().getValue().getAndSetIsClosuresBeingCalculated(true) && !param.getValue().getValue().isClosureSizeCalculated()) {
                ParallelizationUtil.submitFuture("UI-triggered GC Closure Size Calculation for " + param.getValue().getValue().getKey(), () -> {
                    ApplicationStatistics.Measurement m = ApplicationStatistics.getInstance().createMeasurement("cellvaluefactory.gcSizeCount.future");
                    //param.getValue().getValue().fillClosureSizes();
                    param.getValue().getValue().getAndSetIsClosuresBeingCalculated(false);
                    m.end();
                }, CLIENT_INFO.meterRegistry);
            }
            minner.end();
            return param.getValue().getValue().retainedSizeProperty();
        });
        gcSizePercentColumn.setCellValueFactory(param -> getPercentageCellValueFactory(param.getValue().getValue().retainedSizeProperty(),
                                                                                       getParent(param.getValue().getValue()).retainedSizeProperty()));

        retainedSizeColumn.getColumns().add(gcSizeCountColumn);
        retainedSizeColumn.getColumns().add(gcSizePercentColumn);

        dataStructureSizeCountColumn.setCellValueFactory(param -> {
            ApplicationStatistics.getInstance().inc("dataStructureSizeCountColumn CellValueFactory");

            ApplicationStatistics.Measurement minner = ApplicationStatistics.getInstance().createMeasurement("cellvaluefactory.dataStructureSizeCount.inner");
            if (param.getValue().getValue().getAndSetIsClosuresBeingCalculated(true) && !param.getValue().getValue().isClosureSizeCalculated()) {
                ParallelizationUtil.submitFuture("UI-triggered Data Structure Closure Size Calculation for " + param.getValue().getValue().getKey(), () -> {
                    ApplicationStatistics.Measurement m = ApplicationStatistics.getInstance().createMeasurement("cellvaluefactory.dataStructureSizeCount.future");
                    //param.getValue().getValue().fillClosureSizes();
                    param.getValue().getValue().getAndSetIsClosuresBeingCalculated(false);
                    m.end();
                }, CLIENT_INFO.meterRegistry);
            }
            minner.end();
            return param.getValue().getValue().dataStructureSizeProperty();
        });
        dataStructureSizePercentColumn.setCellValueFactory(param -> getPercentageCellValueFactory(param.getValue().getValue().dataStructureSizeProperty(),
                                                                                                  getParent(param.getValue().getValue()).dataStructureSizeProperty()));

        dataStructureSizeColumn.getColumns().add(dataStructureSizeCountColumn);
        dataStructureSizeColumn.getColumns().add(dataStructureSizePercentColumn);

        deepDataStructureSizeCountColumn.setCellValueFactory(param -> {
            ApplicationStatistics.getInstance().inc("deepDataStructureSizeCountColumn CellValueFactory");

            ApplicationStatistics.Measurement minner = ApplicationStatistics.getInstance().createMeasurement("cellvaluefactory.deepDataStructureSizeCount.inner");
            if (param.getValue().getValue().getAndSetIsClosuresBeingCalculated(true) && !param.getValue().getValue().isClosureSizeCalculated()) {
                ParallelizationUtil.submitFuture("UI-triggered Deep Data Structure Closure Size Calculation for " + param.getValue().getValue().getKey(), () -> {
                    ApplicationStatistics.Measurement m = ApplicationStatistics.getInstance().createMeasurement("cellvaluefactory.deepDataStructureSizeCount.future");
                    //param.getValue().getValue().fillClosureSizes();
                    param.getValue().getValue().getAndSetIsClosuresBeingCalculated(false);
                    m.end();
                }, CLIENT_INFO.meterRegistry);
            }
            minner.end();
            return param.getValue().getValue().deepDataStructureSizeProperty();
        });
        deepDataStructureSizePercentColumn.setCellValueFactory(param -> getPercentageCellValueFactory(param.getValue().getValue().deepDataStructureSizeProperty(),
                                                                                                      getParent(param.getValue()
                                                                                                                     .getValue()).deepDataStructureSizeProperty()));

        deepDataStructureSizeColumn.getColumns().add(deepDataStructureSizeCountColumn);
        deepDataStructureSizeColumn.getColumns().add(deepDataStructureSizePercentColumn);

        getColumns().add(nameColumn);
        getColumns().add(objectsColumn);
        getColumns().add(shallowSizeColumn);
        getColumns().add(transitiveClosureSizeColumn);
        getColumns().add(retainedSizeColumn);
        getColumns().add(dataStructureSizeColumn);
        getColumns().add(deepDataStructureSizeColumn);

        // Define column widths
        nameColumn.prefWidthProperty().bind(widthProperty().multiply(7).divide(15) /* aaaand subtract scroll bar width, otherwise binding properties to width does not
                                       work at all */.subtract(20)); // = 7/15
        objectsColumn.prefWidthProperty().bind(widthProperty().divide(7.5)); // 1/7.5 = 2/15
        objectsCountColumn.prefWidthProperty().bind(objectsColumn.widthProperty().divide(2));
        objectsPercentColumn.prefWidthProperty().bind(objectsColumn.widthProperty().divide(2));
        shallowSizeColumn.prefWidthProperty().bind(widthProperty().divide(7.5)); // 1/7.5 = 2/15
        bytesCountColumn.prefWidthProperty().bind(shallowSizeColumn.widthProperty().divide(2));
        bytesPercentColumn.prefWidthProperty().bind(shallowSizeColumn.widthProperty().divide(2));
        transitiveClosureSizeColumn.prefWidthProperty().bind(widthProperty().divide(7.5)); // 1/7.5 = 2/15
        closureSizeCountColumn.prefWidthProperty().bind(transitiveClosureSizeColumn.widthProperty().divide(2));
        closureSizePercentColumn.prefWidthProperty().bind(transitiveClosureSizeColumn.widthProperty().divide(2));
        retainedSizeColumn.prefWidthProperty().bind(widthProperty().divide(7.5)); // 1/7.5 = 2/15
        gcSizeCountColumn.prefWidthProperty().bind(retainedSizeColumn.widthProperty().divide(2));
        gcSizePercentColumn.prefWidthProperty().bind(retainedSizeColumn.widthProperty().divide(2));
        dataStructureSizeColumn.prefWidthProperty().bind(widthProperty().divide(7.5)); // 1/7.5 = 2/15
        dataStructureSizeCountColumn.prefWidthProperty().bind(dataStructureSizeColumn.widthProperty().divide(2));
        dataStructureSizePercentColumn.prefWidthProperty().bind(deepDataStructureSizeColumn.widthProperty().divide(2));
        deepDataStructureSizeColumn.prefWidthProperty().bind(widthProperty().divide(7.5)); // 1/7.5 = 2/15
        deepDataStructureSizeCountColumn.prefWidthProperty().bind(deepDataStructureSizeColumn.widthProperty().divide(2));
        deepDataStructureSizePercentColumn.prefWidthProperty().bind(deepDataStructureSizeColumn.widthProperty().divide(2));

        // must explicitly reset sort types such that sort type change listener triggers on first sort action
        closureSizeCountColumn.setSortType(null);
        closureSizePercentColumn.setSortType(null);
        gcSizeCountColumn.setSortType(null);
        gcSizePercentColumn.setSortType(null);
        dataStructureSizeCountColumn.setSortType(null);
        dataStructureSizePercentColumn.setSortType(null);
        deepDataStructureSizeCountColumn.setSortType(null);
        deepDataStructureSizePercentColumn.setSortType(null);

        closureSizeCountColumn.sortTypeProperty().addListener(closureSortListener);
        closureSizePercentColumn.sortTypeProperty().addListener(closureSortListener);
        gcSizeCountColumn.sortTypeProperty().addListener(closureSortListener);
        gcSizePercentColumn.sortTypeProperty().addListener(closureSortListener);
        dataStructureSizeCountColumn.sortTypeProperty().addListener(closureSortListener);
        dataStructureSizePercentColumn.sortTypeProperty().addListener(closureSortListener);
        deepDataStructureSizeCountColumn.sortTypeProperty().addListener(closureSortListener);
        deepDataStructureSizePercentColumn.sortTypeProperty().addListener(closureSortListener);

        // not shown by default
        transitiveClosureSizeColumn.setVisible(false);
        dataStructureSizeColumn.setVisible(false);
        deepDataStructureSizeColumn.setVisible(false);
    }

    private void initSearchFunction() {
        final Popup popup = new Popup();
        final VBox layout = new VBox();
        final Label titleLabel = new Label("Search for text...");
        final Label instructionsLabel = new Label("(Press ENTER for next or SHIFT+ENTER for previous match)");
        final TextField searchTextField = new TextField();
        final Label resultsLabel = new Label();

        popup.getContent().add(layout);
        popup.setAutoHide(true);
        layout.setBackground(new Background(new BackgroundFill(Color.BLACK.deriveColor(0, 1, 1, 0.6), new CornerRadii(10), new Insets(-5))));
        layout.getChildren().addAll(titleLabel, instructionsLabel, searchTextField, resultsLabel);
        titleLabel.setTextFill(Color.WHITE);
        instructionsLabel.setTextFill(Color.WHITE);
        resultsLabel.setTextFill(Color.WHITE);
        resultsLabel.setManaged(false);

        StringBuilder lastSearchTerm = new StringBuilder();    // mutable string for lambda, needed to detect whether new search or same as last time
        List<TreeItem<GroupingNode>> matches = new ArrayList<>();
        Counter currentMatchIndex = new Counter(true);

        this.addEventHandler(KeyEvent.KEY_RELEASED, evt -> {
            if (!popup.isShowing() && evt.getCode() == KeyCode.F && evt.isControlDown()) {
                // reset search state
                lastSearchTerm.setLength(0);
                matches.clear();
                currentMatchIndex.reset();
                resultsLabel.setManaged(false);
                resultsLabel.setVisible(false);

                // show search box
                popup.show(this, localToScreen(getBoundsInLocal()).getMaxX() - 300, localToScreen(getBoundsInLocal()).getMaxY() - 100);
            }
        });

        searchTextField.addEventHandler(KeyEvent.KEY_RELEASED, evt -> {
            if (evt.getCode() == KeyCode.ENTER && searchTextField.getText().length() > 0) {
                if (!searchTextField.getText().equals(lastSearchTerm.toString())) {
                    // new search -> filter
                    lastSearchTerm.setLength(0);
                    lastSearchTerm.append(searchTextField.getText());
                    matches.clear();
                    currentMatchIndex.reset();
                    gatherMatchingItems(getRoot(), lastSearchTerm.toString(), matches);
                    if (matches.size() > 0) {
                        resultsLabel.setText(String.format("Showing matching item #%d (%d in total)", currentMatchIndex.get() + 1, matches.size()));
                        showAndSelect(matches.get((int) currentMatchIndex.get()));
                    } else {
                        resultsLabel.setText("No matching items found!");
                    }

                    resultsLabel.setVisible(true);
                    resultsLabel.setManaged(true);

                } else if (matches.size() > 0) {
                    // move within matches
                    if (evt.isShiftDown()) {
                        // previous match
                        currentMatchIndex.dec();
                        if (currentMatchIndex.get() < 0) {
                            currentMatchIndex.reset();
                            currentMatchIndex.add(matches.size() - 1);
                        }
                    } else {
                        // next match
                        currentMatchIndex.inc();
                        if (currentMatchIndex.get() >= matches.size()) {
                            currentMatchIndex.reset();
                        }
                    }

                    resultsLabel.setText(String.format("Showing matching item #%d (%d in total)", currentMatchIndex.get() + 1, matches.size()));
                    showAndSelect(matches.get((int) currentMatchIndex.get()));
                }

            } else if (evt.getCode() == KeyCode.ENTER && searchTextField.getText().length() == 0) {
                resultsLabel.setText("Please enter at least one character...");
                resultsLabel.setManaged(true);
                resultsLabel.setVisible(true);

            } else if (evt.getCode() == KeyCode.ESCAPE) {
                popup.hide();
            }
        });
    }

    private void adjustContextMenu(TreeTableRow<GroupingNode> row) {
        contextMenu.getItems().forEach(item -> item.setVisible(false));

        // inspect object group is always possible
        inspectObjectGroup.setVisible(true);

        // enable instant closure size calculation if at least one selected node has not been calculated yet
        if (getSelectionModel().getSelectedItems().stream().map(ti -> ti.getValue()).anyMatch(node -> !node.isClosureSizeCalculated())) {
            calculateClosuresNow.setVisible(true);
        }

        if (row.getItem() != null && row.getItem().getSubTreeLevel() == 0) {
            // regular node
            localClassify.setVisible(true);
            localPointsTo.setVisible(true);
            localPointedFrom.setVisible(true);

        } else if (row.getItem() != null) {
            // transformer node
            localClassifyOnSameSubTreeLevel.setVisible(true);
            localPointsToOnSameSubTreeLevel.setVisible(true);
            localPointedFromOnSameSubTreeLevel.setVisible(true);
            // TODO classification on lower sub-tree level does not work correctly for now
            //            localClassifyOnLowerSubTreeLevel.setVisible(true);
            //            localPointsToOnLowerSubTreeLevel.setVisible(true);
            //            localPointedFromOnLowerSubTreeLevel.setVisible(true);
            //
            //            localClassifyOnLowerSubTreeLevel.getItems().clear();
            //            localPointsToOnLowerSubTreeLevel.getItems().clear();
            //            localPointedFromOnLowerSubTreeLevel.getItems().clear();
            //            for (int i = 0; i < row.getItem().getSubTreeLevel(); i++) {
            //                // generate submenu items for each possible subtreelevel
            //                final int targetSubTreeLevel = i;
            //
            //                MenuItem subTreeLevelSelectionClassify = new MenuItem(Integer.toString(i));
            //                subTreeLevelSelectionClassify.setOnAction(click -> handleLocalClassification(new ClassifierChain(new LocalClassifier()),
            // targetSubTreeLevel));
            //                localClassifyOnLowerSubTreeLevel.getItems().add(subTreeLevelSelectionClassify);
            //
            //                MenuItem subTreeLevelSelectionPointsTo = new MenuItem(Integer.toString(i));
            //                subTreeLevelSelectionPointsTo.setOnAction(click -> handleLocalClassification(new ClassifierChain(availableClassifierInfo.getClassifier
            // ("Points To
            // (Direct)")),
            //                                                                                             targetSubTreeLevel));
            //                localPointsToOnLowerSubTreeLevel.getItems().add(subTreeLevelSelectionPointsTo);
            //
            //                MenuItem subTreeLevelSelectionPointedFrom = new MenuItem(Integer.toString(i));
            //                subTreeLevelSelectionPointedFrom.setOnAction(click -> handleLocalClassification(new ClassifierChain(availableClassifierInfo.getClassifier
            // ("Pointed
            // From")),
            //                                                                                                targetSubTreeLevel));
            //                localPointedFromOnLowerSubTreeLevel.getItems().add(subTreeLevelSelectionPointedFrom);
            //            }
        }
    }

    /**
     * Used for search functionality to collect nodes that match the given string
     *
     * @param node          current node to check
     * @param searchText    text that should be matched
     * @param matchingItems matches found so far
     */
    private void gatherMatchingItems(TreeItem<GroupingNode> node, String searchText, List<TreeItem<GroupingNode>> matchingItems) {
        if (node.getValue().getKey().toString().contains(searchText)) {
            matchingItems.add(node);
        }

        node.getChildren().forEach(child -> gatherMatchingItems(child, searchText, matchingItems));
    }

    private void showAndSelect(TreeItem<GroupingNode> item) {
        // make tree table visible on the screen
        showTTVOnScreen.run();

        // expand all parents
        for (TreeItem<GroupingNode> ti = item.getParent(); ti != null; ti = ti.getParent()) {
            ti.setExpanded(true);
        }

        // select
        this.getSelectionModel().clearSelection();
        this.getSelectionModel().select(item);

        // scroll if necessary (scroll to 5 items above if possible - adds some context to the selected item)
        if (!isItemInView(item)) {
            this.scrollTo(this.getRow(item) - 5);
        }
    }

    /**
     * Scrolls to and selects the first row in the tree table that matches the given search text
     *
     * @param searchText search text that the name cell of a row has to match
     */
    public void showAndSelect(String searchText) {
        List<TreeItem<GroupingNode>> matches = new ArrayList<>();
        gatherMatchingItems(getRoot(), searchText, matches);

        if (!matches.isEmpty()) {
            showAndSelect(matches.get(0));
        }
    }

    private void inspectObjectGroup(List<TreeItem<GroupingNode>> items) {
        List<TreeItem<GroupingNode>> stableItemList = new ArrayList<>(items);   // copy list! given list might be modified (usually an observable list of selections...)
        ObjectGroupInfoTab objectGroupInfoTab = new ObjectGroupInfoTab();
        objectGroupInfoTab.init(appInfo,
                                items.stream().map(TreeItem::getValue).toArray(GroupingNode[]::new),
                                selectLocalClassifiers,
                                availableClassifierInfo,
                                addTab,
                                switchToTab,
                                (node) -> {
                                    TreeItem<GroupingNode> targetItem = stableItemList.stream().filter(item -> item.getValue() == node).findFirst().orElse(null);
                                    if (targetItem == null) {
                                        throw new IllegalArgumentException("Jumping back is only allowed for the originally given items!");
                                    }
                                    showAndSelect(targetItem);
                                },
                                true);
    }

    private void handleLocalClassification(ClassifierChain classifierChain) {
        // classify on same subtreelevel
        handleLocalClassification(classifierChain, -1);
    }

    private void handleLocalClassification(ClassifierChain classifierChain, int targetSubTreeLevel) {
        TreeItem<GroupingNode> selectedItem = getSelectionModel().getSelectedItem();

        if (!(selectedItem.getValue() instanceof ListGroupingNode)) {
            LOGGER.warning("Local classification only works for ListGroupingNodes!");
            return;
        }

        ListGroupingNode node = (ListGroupingNode) getSelectionModel().getSelectedItem().getValue();

        // show classifier selection
        if (selectLocalClassifiers != null && classifierChain != null && classifierChain.getList().size() > 0 && classifierChain.get(0) != null) {

            if (selectLocalClassifiers.apply(classifierChain.get(0))) {
                // successfully selected some classifier(s)
                AtomicBoolean cancellationToken = new AtomicBoolean(false);
                Task<TreeItem<Void>> t = new Task<TreeItem<Void>>() {
                    @Override
                    protected TreeItem<Void> call() throws Exception {
                        updateTitle("Local classification");
                        updateMessage(String.format("%,d objects in Node %s with:/n",
                                                    node.getObjectCount(),
                                                    node.getKey(),
                                                    (classifierChain.get(0) instanceof LocalClassifier
                                                     ? ((LocalClassifier) classifierChain.get(0)).getClassifiers().toString()
                                                     : classifierChain.toString())));

                        Counter objectsProcessed = new Counter();
                        ObjectStream.IterationListener iterationListener = (oc) -> {
                            objectsProcessed.add(oc);
                            updateProgress(objectsProcessed.get(), node.getObjectCount());
                        };

                        // extract selected classifiers from LocalClassifier (only used for easy selection dialog) if necessary
                        ClassifierChain finalClassifierChain = classifierChain.get(0) instanceof LocalClassifier ?
                                                               ((LocalClassifier) classifierChain.get(0)).getClassifiers() :
                                                               classifierChain;

                        if (node.getSubTreeLevel() == 0) {
                            // regular node
                            node.locallyClassify(finalClassifierChain, iterationListener, cancellationToken);
                        } else {
                            // transformer node
                            if (targetSubTreeLevel >= 0) {
                                // valid target stl given
                                node.locallyClassifyTransformerOnLowerSubtreeLevel(targetSubTreeLevel, finalClassifierChain, iterationListener, cancellationToken);
                            } else {
                                node.locallyClassifyTransformerOnSameSubTreeLevel(finalClassifierChain, iterationListener, cancellationToken);
                            }
                        }
                        node.sampleTopDown();

                        if (!cancellationToken.get()) {
                            // display result of local classification
                            Platform.runLater(() -> {
                                TreeItem<GroupingNode> newItem;
                                // if the classified items was selected we'll have to reselect the new item later
                                boolean reselect = getSelectionModel().isSelected(getRow(selectedItem));
                                // unselect the classified item because otherwise removing it from the treetable causes weird selection behaviour
                                getSelectionModel().clearSelection(getRow(selectedItem));

                                if (getRoot() == selectedItem) {
                                    // the overall node is reclassified
                                    setRoot(new ListClassificationTree(node,
                                                                       classificationTree.getFilters(),
                                                                       finalClassifierChain,
                                                                       classificationTree.getConditionalClassifiers()),
                                            true,
                                            true);
                                    newItem = getRoot();
                                    setSelectedClassifiers.accept(finalClassifierChain);
                                } else {
                                    // some descendant of overall is locally classified
                                    newItem = new AutomatedTreeItem<>(node, titleFunction, dataFunction, childFunction, subTreeLevelFunction, iconFunction);
                                    TreeItem<GroupingNode> selectionParent = selectedItem.getParent();
                                    int idx = selectionParent.getChildren().indexOf(selectedItem);
                                    selectionParent.getChildren().remove(idx);
                                    selectionParent.getChildren().add(idx, newItem);
                                }

                                if (reselect) {
                                    getSelectionModel().select(getRow(newItem));
                                }
                                // expand the new node to show the results of the classification
                                newItem.setExpanded(true);

                                // if the user scrolled the selected node out of view, provide a popup to jump back to the results of the classification
                                if (!isItemInView(newItem)) {
                                    Notifications.create()
                                                 .title("Success")
                                                 .text("Local classification of " + newItem.getValue().getKey().toString() + " succeeded!")
                                                 .action(new Action("Show me", actionEvent -> {
                                                     if (showTTVOnScreen != null) {
                                                         showTTVOnScreen.run();
                                                         scrollTo(getRow(newItem));
                                                         getSelectionModel().clearAndSelect(getRow(newItem));
                                                     }
                                                 }))
                                                 .showInformation();
                                }
                            });
                        }

                        return null;
                    }

                    @Override
                    protected void cancelled() {
                        super.cancelled();
                        cancellationToken.set(true);
                    }
                };

                CLIENT_INFO.operationManager.addNewOperation(t);
                ParallelizationUtil.submitTask(t);
            }
        }
    }

    private boolean isItemInView(TreeItem<GroupingNode> item) {
        int[] visibleRange = getVisibleRange();

        return getRow(item) >= visibleRange[0] && getRow(item) <= visibleRange[1];
    }

    private int[] getVisibleRange() {
        TreeTableViewSkin<?> skin = (TreeTableViewSkin<?>) getSkin();
        if (skin == null) {
            return new int[]{0, 0};
        }
        VirtualFlow<?> flow = (VirtualFlow<?>) skin.getChildren().get(1);
        int indexFirst;
        int indexLast;
        if (flow != null && flow.getFirstVisibleCellWithinViewPort() != null && flow.getLastVisibleCellWithinViewPort() != null) {
            indexFirst = flow.getFirstVisibleCellWithinViewPort().getIndex();
            indexLast = flow.getLastVisibleCellWithinViewPort().getIndex();
        } else {
            indexFirst = 0;
            indexLast = 0;
        }
        return new int[]{indexFirst, indexLast};
    }

    private ObjectBinding<Percentage> getPercentageCellValueFactory(NumberExpression source, NumberExpression parent) {
        return Bindings.when(source.greaterThanOrEqualTo(0))
                       .then(Bindings.when(parent.greaterThanOrEqualTo(0)).then(getPercentage(parent.doubleValue(), source.doubleValue())).otherwise(new Percentage(-1)))
                       .otherwise(new Percentage(-1));
    }

    public Percentage getPercentage(double total, double x) {
        if (total == 0 || x == 0) {
            return new Percentage(0.0);
        }
        return new Percentage(new BigDecimal(x / total * 100).setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue());
    }

    public void setLoadingMode(boolean isLoading) {
        setDisabled(isLoading);
    }

    private GroupingNode getParent(GroupingNode heapObjectGroupingNode) {
        int subTreeLevel = heapObjectGroupingNode.getSubTreeLevel();
        GroupingNode lastMatching = null;
        while (heapObjectGroupingNode != null) {
            if (heapObjectGroupingNode.getSubTreeLevel() == subTreeLevel) {
                lastMatching = heapObjectGroupingNode;
            }
            heapObjectGroupingNode = heapObjectGroupingNode.getParent();
        }
        return lastMatching;
    }

    public void setRoot(ClassificationTree grouping, boolean preserveSortOrder, boolean preserveExpansions) {
        this.classificationTree = grouping;
        // remember sorting
        List<TreeTableColumn<GroupingNode, ?>> rememberedSortOrder = new ArrayList<>();
        rememberedSortOrder.addAll(getSortOrder());
        TreeSortMode rememberedSortMode = getSortMode();
        List<TreeTableColumn.SortType> rememberedSortTypes = new ArrayList<>();
        rememberedSortOrder.forEach(col -> rememberedSortTypes.add(col.getSortType()));

        rememberExpansion();
        root = new AutomatedTreeItem<>(grouping.getRoot(), titleFunction, dataFunction, childFunction, subTreeLevelFunction, iconFunction);
        // filterChanged(node -> node.getByteCount() > 10000);
        setRoot(root);
        setShowRoot(true);
        getRoot().setExpanded(true);

        if (preserveExpansions) {
            restoreExpansion();
        }
        if (preserveSortOrder) {
            // restore sorting
            getSortOrder().addAll(rememberedSortOrder);
            setSortMode(rememberedSortMode);
            for (int i = 0; i < rememberedSortOrder.size(); i++) {
                rememberedSortOrder.get(i).setSortType(rememberedSortTypes.get(i));
            }
        } else {
            // standard sorting
            getSortOrder().clear();
            setSortMode(TreeSortMode.ALL_DESCENDANTS);
            objectsCountColumn.setSortType(TreeTableColumn.SortType.DESCENDING);
            getSortOrder().add(objectsCountColumn);
        }


        /*
        if (tableRefresher != null) {
            tableRefresher.stop();
            tableRefresher = null;
        }
        tableRefresher = new Timeline(
                new KeyFrame(Duration.seconds(0)),
                new KeyFrame(Duration.seconds(10), actionEvent -> {
                    if (grouping.closuresInitialized()) {
                        System.out.println("Stop refreshing");
                        ClassificationTreeTableView.this.tableRefresher.stop();
                        ClassificationTreeTableView.this.tableRefresher = null;
                    } else {
                        System.out.println("Refresh");
                        ClassificationTreeTableView.this.refresh();
                    }
                }));
        tableRefresher.setCycleCount(Animation.INDEFINITE);
        tableRefresher.play();
        */
    }

    @Override
    protected TreeTableRow<GroupingNode> createTreeTableRow() {
        TreeTableRow<GroupingNode> row = super.createTreeTableRow();
        // context menu
        row.setContextMenu(contextMenu);
        row.setOnMousePressed(evt -> {
            if (evt.isSecondaryButtonDown()) {
                adjustContextMenu(row);
            }
        });

        return row;
    }

    private void filterChanged(Predicate<GroupingNode> filter) {
        TreeItem<GroupingNode> filteredRoot = new TreeItem<>();
        filteredRoot.setValue(root.getValue());
        filteredRoot.setExpanded(true);
        filter(root, filter, filteredRoot);
        setRoot(filteredRoot);
    }

    private void filter(TreeItem<GroupingNode> root, Predicate<GroupingNode> filter, TreeItem<GroupingNode> filteredRoot) {
        for (TreeItem<GroupingNode> child : root.getChildren()) {
            TreeItem<GroupingNode> filteredChild = new TreeItem<>();
            filteredChild.setValue(child.getValue());
            filteredChild.setExpanded(true);
            filter(child, filter, filteredChild);
            if (!filteredChild.getChildren().isEmpty() || isMatch(filteredChild.getValue(), filter)) {
                System.out.println(Arrays.toString(filteredChild.getValue().getFullKey()) + " matches.");
                filteredRoot.getChildren().add(filteredChild);
            }
        }
    }

    private boolean isMatch(GroupingNode value, Predicate<GroupingNode> filter) {
        return filter.test(value);
    }

    public ClassificationTree getClassificationTree() {
        return classificationTree;
    }

    public TreeTableColumn<GroupingNode, String> getDataStructureSizeColumn() {
        return dataStructureSizeColumn;
    }

    public TreeTableColumn<GroupingNode, String> getShallowSizeColumn() {
        return shallowSizeColumn;
    }

    public TreeTableColumn<GroupingNode, String> getDeepDataStructureSizeColumn() {
        return deepDataStructureSizeColumn;
    }
}