/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.heapstate.tab.statistics.component.configurationpane;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierFactory;
import at.jku.mevss.anttracks.classification.Filter;
import at.jku.mevss.anttracks.classification.FilterFactory;
import at.jku.mevss.trace.gui.classification.component.configurationpane.ConfigurationPane;
import at.jku.mevss.trace.gui.classification.component.selectionpane.ClassificationSelectionPane;
import at.jku.mevss.trace.gui.classification.component.selectionpane.ClassificationSelectionPane.ClassificationSelectionListener;
import at.jku.mevss.trace.gui.frame.main.tab.heapstate.tab.statistics.HeapStatisticsTab;
import at.jku.mevss.trace.gui.frame.main.tab.heapstate.tab.statistics.component.configurationpane.operationpane.HeapStatisticsOperationPane;
import at.jku.mevss.trace.gui.model.HeapStateStatisticsInfo;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import javafx.fxml.FXML;

public class HeapStatisticsConfigurationPane extends ConfigurationPane {

    private HeapStatisticsTab heapStatisticsTab;

    @FXML
    private HeapStatisticsOperationPane actionPanel;

    private HeapStateStatisticsInfo statisticsInfo;

    public HeapStatisticsConfigurationPane() {
        FXMLUtil.load(this, HeapStatisticsConfigurationPane.class);
    }

    public void init(HeapStateStatisticsInfo statisticsInfo, HeapStatisticsTab heapStatisticsTab) {
        super.init(statisticsInfo.getDetailsInfo().getAppInfo(), statisticsInfo.getDetailsInfo(), statisticsInfo);
        this.statisticsInfo = statisticsInfo;
        this.heapStatisticsTab = heapStatisticsTab;
        actionPanel.init(statisticsInfo, heapStatisticsTab);
    }

    @Override
    protected ClassificationSelectionListener<Classifier<?>, ClassifierFactory> createClassifierListener() {
        return new ClassificationSelectionListener<Classifier<?>, ClassifierFactory>() {
            @Override
            public void selected(ClassificationSelectionPane<Classifier<?>, ClassifierFactory> sender, Classifier<?> x) {
                actionPanel.setExecuteEnabled(!heapStatisticsTab.isLoading());
                actionPanel.setExportEnabled(!heapStatisticsTab.isLoading() && statisticsInfo.getGrouping() != null);
            }

            @Override
            public void deselected(ClassificationSelectionPane<Classifier<?>, ClassifierFactory> sender, Classifier<?> x) {
                actionPanel.setExecuteEnabled(!heapStatisticsTab.isLoading());
                actionPanel.setExportEnabled(!heapStatisticsTab.isLoading() && statisticsInfo.getGrouping() != null);
            }

            @Override
            public void propertiesChanged(ClassificationSelectionPane<Classifier<?>, ClassifierFactory> sender, Classifier<?> x) {
                // Nothing to do if property changed
            }
        };
    }

    @Override
    protected ClassificationSelectionListener<Filter, FilterFactory> createFilterListener() {
        return new ClassificationSelectionListener<Filter, FilterFactory>() {
            @Override
            public void selected(ClassificationSelectionPane<Filter, FilterFactory> sender, Filter x) {
                actionPanel.setExecuteEnabled(!heapStatisticsTab.isLoading());
            }

            @Override
            public void deselected(ClassificationSelectionPane<Filter, FilterFactory> sender, Filter x) {
                actionPanel.setExecuteEnabled(!heapStatisticsTab.isLoading());
            }

            @Override
            public void propertiesChanged(ClassificationSelectionPane<Filter, FilterFactory> sender, Filter x) {
                // Nothing to do if property changed
            }
        };
    }

    @Override
    public void setLoadingMode(boolean isLoading) {
        actionPanel.setExecuteEnabled(!isLoading);
        actionPanel.setExportEnabled(!isLoading && statisticsInfo.getGrouping() != null);
        // CR Please don't remove for now, we might want to change it back??
        // actionPanel.setVisualizeEnabled(!isLoading);
    }

    @Override
    public void switchMode() {
        super.switchMode();
        if (actionPanel != null) {
            actionPanel.setVisible(isMaxMode);
            actionPanel.setManaged(isMaxMode);
        }
        // repaint();
        // revalidate();
    }

    @Override
    public void setAppropriateConfigView() {
        if (!isMaxMode) {
            switchMode();
        }
        // Switch to minimal view
        switchMode();
    }

    public HeapStatisticsOperationPane getOperationPanel() {
        return actionPanel;
    }

}
