/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.heapstate.tab.statistics.component.configurationpane.operationpane;

import at.jku.mevss.anttracks.util.ThreadUtil;
import at.jku.mevss.trace.gui.frame.main.tab.heapstate.tab.statistics.GroupingTask;
import at.jku.mevss.trace.gui.frame.main.tab.heapstate.tab.statistics.HeapStatisticsTab;
import at.jku.mevss.trace.gui.model.HeapStateStatisticsInfo;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import at.jku.mevss.trace.gui.utils.WindowUtil;
import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.Button;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Region;

public class HeapStatisticsOperationPane extends HBox {

    // Info objects
    private HeapStateStatisticsInfo statisticsInfo;

    // References to view and other panels
    private HeapStatisticsTab heapStatisticsTab;

    // Buttons
    @FXML
    private Button applyButton;
    @FXML
    private Button exportAsGefxButton;
    @FXML
    private Button cancelButton;

    public HeapStatisticsOperationPane() {
        FXMLUtil.load(this, HeapStatisticsOperationPane.class);
    }

    public void init(HeapStateStatisticsInfo statisticsInfo, HeapStatisticsTab heapStatisticsTab) {
        this.statisticsInfo = statisticsInfo;
        this.heapStatisticsTab = heapStatisticsTab;

        applyButton.setOnAction(ae -> apply());
        cancelButton.setOnAction(ae -> cancelEdit());
        exportAsGefxButton.setOnAction(ae -> {
            Alert alert = new Alert(AlertType.INFORMATION);
            alert.setTitle("Export currently not supported!");
            alert.getDialogPane().setMinHeight(Region.USE_PREF_SIZE);
            alert.setContentText(".gexf export is not yet supported.");
            WindowUtil.centerInMainFrame(alert);
            alert.showAndWait();
        });

    }

    public void setStatisticsInfo(HeapStateStatisticsInfo statisticsInfo) {
        this.statisticsInfo = statisticsInfo;
    }

    public void setHeapStatisticsTab(HeapStatisticsTab heapStatisticsTab) {
        this.heapStatisticsTab = heapStatisticsTab;
    }

    /**
     * Runs a {@link GroupingTask} which updates the UI on finishing
     */
    public void apply() {
        heapStatisticsTab.acceptEdit();
        heapStatisticsTab.setAppropriateConfigView();
        if (statisticsInfo.getDetailsInfo().getFastHeapSupplier().get() != null) {
            GroupingTask groupingTask = new GroupingTask(statisticsInfo, heapStatisticsTab);
            ThreadUtil.startTask(groupingTask);
        }
    }

    //    private void openVisualization() {
    //        StatisticsInfo statisticsCopy = new StatisticsInfo(statisticsInfo.getDetailsInfo(), null,
    //                                                           statisticsInfo.getSelectedConditionalClassifier(), null);
    //
    //        HeapVisualizationTab heapVisualizationTab = new HeapVisualizationTab();
    //        heapVisualizationTab.init(statisticsCopy);
    //        heapStatisticsTab.getTabPane().getTabs().add(heapVisualizationTab);
    //        HeapVisualizationTask task = new HeapVisualizationTask(statisticsCopy, heapVisualizationTab);
    //        ThreadUtil.startTask(task);
    //    }

    private void cancelEdit() {
        heapStatisticsTab.cancelEdit();
        heapStatisticsTab.setAppropriateConfigView();
    }

    public void setExecuteEnabled(boolean b) {
        applyButton.setDisable(!b);
    }

    public void setExportEnabled(boolean b) {
        exportAsGefxButton.setDisable(!b);
    }
}
