package at.jku.mevss.trace.gui.frame.main.tab.heapvisualization;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.trace.gui.frame.main.tab.heapvisualization.model.ObjectVisualizationData;
import at.jku.mevss.trace.gui.model.HeapVisualizationStatisticsInfo;
import at.jku.mevss.trace.gui.utils.AntTask;

import java.util.function.BinaryOperator;

public class HeapVisualizationUpdateTask extends AntTask<ObjectVisualizationData> {
    private final HeapVisualizationStatisticsInfo statisticsInfo;
    private final HeapVisualizationTab heapVisualizationTab;

    public HeapVisualizationUpdateTask(HeapVisualizationStatisticsInfo statisticsInfo, HeapVisualizationTab heapVisualizationTab) {
        this.statisticsInfo = statisticsInfo;
        this.heapVisualizationTab = heapVisualizationTab;
    }

    @Override
    protected ObjectVisualizationData backgroundWork() throws Exception {
        BinaryOperator<String> op = (a, b) -> a + " => " + b;
        updateTitle("Visualization");
        updateMessage("Generate pixel map " + statisticsInfo.getSelectedClassifiers().getList()
                                                            .stream()
                                                            .map(Classifier::getName)
                                                            .reduce(op)
                                                            .orElse("") + " " + (statisticsInfo.getDetailsInfo().getTime() / 1000.0) + "s");

        ObjectVisualizationData data = new ObjectVisualizationData(statisticsInfo, this);
        data.generateData();

        LOGGER.info("Generation of pixel map finished.");

        return data;
    }

    @Override
    protected void suceeded() {
        ObjectVisualizationData data = getValue();
        heapVisualizationTab.setCurrentHeapVisualizationTask(null);
        PaintTask paintTask = new PaintTask(statisticsInfo, heapVisualizationTab, data, heapVisualizationTab.getPixelMap(), PaintTask.PaintOperation.PAINT);
        heapVisualizationTab.getTabPane().getSelectionModel().select(heapVisualizationTab);
        (new Thread(paintTask)).start();
        heapVisualizationTab.setCurrentPaintTask(paintTask);
    }

}
