/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.heapvisualization;

import at.jku.mevss.trace.gui.frame.main.tab.heapvisualization.model.*;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.model.DetailedHeapInfo;
import at.jku.mevss.trace.gui.model.HeapVisualizationStatisticsInfo;
import at.jku.mevss.trace.gui.utils.AntTask;

public class PaintTask extends AntTask<PixelMap> {

    public static enum PaintOperation {
        PAINT,
        ZOOM_IN,
        ZOOM_OUT
    }

    private final AppInfo appInfo;
    protected final DetailedHeapInfo detailsInfo;
    private final HeapVisualizationStatisticsInfo statisticsInfo;

    private final HeapVisualizationTab heapVisualizationTab;
    private final ObjectVisualizationData data;
    private PixelMap pixelMap;

    private final PaintOperation paintOperation;

    public PaintTask(HeapVisualizationStatisticsInfo statisticsInfo,
                     HeapVisualizationTab heapVisualizationTab,
                     ObjectVisualizationData data,
                     PixelMap pixelMap,
                     PaintOperation operation) {
        appInfo = statisticsInfo.getDetailsInfo().getAppInfo();
        detailsInfo = statisticsInfo.getDetailsInfo();
        this.statisticsInfo = statisticsInfo;
        this.heapVisualizationTab = heapVisualizationTab;
        this.data = data;
        this.pixelMap = pixelMap;
        paintOperation = operation;
    }

    @Override
    protected PixelMap backgroundWork() throws Exception {
        updateTitle("Visualization");
        updateMessage("Paint heap " + (detailsInfo.getTime() / 1000.0) + "s");

        switch (paintOperation) {
            case PAINT:
                if (pixelMap == null) {
                    if (heapVisualizationTab.getClusterLevel() == ClusterLevel.OBJECTS) {
                        pixelMap = new ObjectPixelMap(detailsInfo, heapVisualizationTab.getScrollPane(), data, false);
                    } else {
                        pixelMap = new BytePixelMap(detailsInfo, heapVisualizationTab.getScrollPane(), data, false);
                    }
                } else if (!heapVisualizationTab.getClusterLevel().value.isInstance(pixelMap)) {
                    if (heapVisualizationTab.getClusterLevel() == ClusterLevel.OBJECTS) {
                        pixelMap = new ObjectPixelMap(detailsInfo, heapVisualizationTab.getScrollPane(), data, pixelMap.showPointers());
                    } else if (heapVisualizationTab.getClusterLevel() == ClusterLevel.BYTES) {
                        pixelMap = new BytePixelMap(detailsInfo, heapVisualizationTab.getScrollPane(), data, pixelMap.showPointers());
                    }
                } else {
                    pixelMap.setData(data);
                }
                pixelMap.paintMap(heapVisualizationTab.getStalledClusterSize(), this);
                break;
            case ZOOM_IN:
                pixelMap.zoomIn(this);
                break;
            case ZOOM_OUT:
                pixelMap.zoomOut(this);
                break;
            default:
                break;
        }
        LOGGER.info("Painting of heap finished.");
        return pixelMap;
    }

    @Override
    protected void suceeded() {
        PixelMap pixelMap = getValue();
        if (!pixelMap.showPointers()) {
            heapVisualizationTab.resetObjectInfo();
        }
        heapVisualizationTab.updateAfterPainting(pixelMap);
        heapVisualizationTab.setCurrentPaintTask(null);
    }

}
