/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.heapvisualization;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.trace.gui.frame.main.tab.heapvisualization.PaintTask.PaintOperation;
import at.jku.mevss.trace.gui.frame.main.tab.heapvisualization.model.PixelMap;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.model.DetailedHeapInfo;
import at.jku.mevss.trace.gui.model.HeapVisualizationStatisticsInfo;
import at.jku.mevss.trace.gui.utils.AntTask;

import java.util.function.BinaryOperator;

public class PointersTask extends AntTask<PixelMap> {

    public static enum PointerOperation {
        TO_POINTERS,
        FROM_POINTERS,
        BOTH;
    }

    private final DetailedHeapInfo detailsInfo;
    private final AppInfo appInfo;
    private final HeapVisualizationTab heapVisualizationTab;
    private final PixelMap pixelMap;
    private final HeapVisualizationStatisticsInfo statisticsInfo;
    private final PointerOperation pointerOperation;

    public PointersTask(HeapVisualizationStatisticsInfo statisticsInfo, HeapVisualizationTab heapVisualizationTab, PixelMap pixelMap, PointerOperation pointerOperation) {
        detailsInfo = statisticsInfo.getDetailsInfo();
        this.statisticsInfo = statisticsInfo;
        appInfo = detailsInfo.getAppInfo();
        this.heapVisualizationTab = heapVisualizationTab;
        this.pixelMap = pixelMap;
        this.pointerOperation = pointerOperation;
    }

    @Override
    protected PixelMap backgroundWork() throws Exception {
        BinaryOperator<String> op = (a, b) -> a + " => " + b;
        updateTitle("Visualization: ");
        updateMessage("Retrieve pointers " + statisticsInfo.getSelectedClassifiers().getList()
                                                           .stream()
                                                           .map(Classifier::getName)
                                                           .reduce(op)
                                                           .orElse("") + " " + (detailsInfo.getTime() / 1000.0) + "s");
        switch (pointerOperation) {
            case TO_POINTERS:
                pixelMap.generatePointers(heapVisualizationTab.getPointsToLevel(), true, this);
                if (pixelMap.getData().getFromPointersLevel() != heapVisualizationTab.getPointersFromLevel()) {
                    pixelMap.generatePointers(heapVisualizationTab.getPointersFromLevel(), false, this);
                }
                break;
            case FROM_POINTERS:
                pixelMap.generatePointers(heapVisualizationTab.getPointersFromLevel(), false, this);
                if (pixelMap.getData().getPointersLevel() != heapVisualizationTab.getPointsToLevel()) {
                    pixelMap.generatePointers(heapVisualizationTab.getPointsToLevel(), true, this);
                }
                break;
            case BOTH:
                pixelMap.generatePointers(heapVisualizationTab.getPointsToLevel(), true, this);
                pixelMap.generatePointers(heapVisualizationTab.getPointersFromLevel(), false, this);
        }
        LOGGER.info("Retrieval of pointers finished.");
        return pixelMap;
    }

    @Override
    protected void suceeded() {
        PixelMap data = getValue();
        heapVisualizationTab.setCurrentPointersTask(null);
        PaintTask paintTask = new PaintTask(statisticsInfo, heapVisualizationTab, data.getData(), data, PaintOperation.PAINT);
        heapVisualizationTab.setCurrentPaintTask(paintTask);
        (new Thread(paintTask)).start();
    }

}
