/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization;

import at.jku.mevss.anttracks.classification.ClassifierFactoryList;
import at.jku.mevss.anttracks.classification.FilterFactoryList;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierSourceCollection;
import at.jku.mevss.anttracks.heap.IndexBasedHeap;
import at.jku.mevss.anttracks.heap.symbols.Symbols;
import at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization.visualization.LifetimeVisualizationDiagramTask;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.model.AvailableClassifierInfo;
import at.jku.mevss.trace.gui.model.SelectedClassifierInfo;
import at.jku.mevss.trace.gui.utils.Consts;

import java.util.function.Supplier;

public class LifetimeVisualizationInfo extends AvailableClassifierInfo {
    private final AppInfo appInfo;
    private final long startTime;
    private final long endTime;
    private SelectedClassifierInfo selectedClassifierInfo;

    public LifetimeVisualizationInfo(AppInfo appInfo, long startTime, long endTime, SelectedClassifierInfo selectedClassifierInfo) {
        this.appInfo = appInfo;
        this.startTime = startTime;
        this.endTime = endTime;
        this.selectedClassifierInfo = selectedClassifierInfo;

        ClassifierSourceCollection classifierSourceCollection = LifetimeVisualizationDiagramTask.USE_INDEX_BASED_HEAP ?
                                                                ClassifierSourceCollection.FASTHEAP :
                                                                ClassifierSourceCollection.DETAILEDHEAP;

        availableClassifier = new ClassifierFactoryList(getSymbolsSupplier(), getFastHeapSupplier(), Consts.CLASSIFIERS_DIRECTORY, classifierSourceCollection);
        availableFilter = new FilterFactoryList(getSymbolsSupplier(), getFastHeapSupplier(), Consts.FILTERS_DIRECTORY, classifierSourceCollection);
    }

    public LifetimeVisualizationInfo(AppInfo appInfo,
                                     long startTime,
                                     long endTime,
                                     SelectedClassifierInfo selectedClassifierInfo,
                                     ClassifierFactoryList availableClassifier,
                                     FilterFactoryList availableFilter) {
        this.appInfo = appInfo;
        this.startTime = startTime;
        this.endTime = endTime;
        this.selectedClassifierInfo = selectedClassifierInfo;
        this.availableClassifier = availableClassifier;
        this.availableFilter = availableFilter;
    }

    public long getStartTime() {
        return startTime;
    }

    public long getEndTime() {
        return endTime;
    }

    public SelectedClassifierInfo getSelectedClassifierInfo() {
        return selectedClassifierInfo;
    }

    public void setSelectedClassifierInfo(SelectedClassifierInfo selectedClassifierInfo) {
        this.selectedClassifierInfo = selectedClassifierInfo;
    }

    public AppInfo getAppInfo() {
        return appInfo;
    }

    @Override
    public Supplier<IndexBasedHeap> getFastHeapSupplier() {
        return null;
    }

    @Override
    public Supplier<Symbols> getSymbolsSupplier() {
        return appInfo::getSymbols;
    }
}
