/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierChain;
import at.jku.mevss.anttracks.classification.Filter;
import at.jku.mevss.anttracks.classification.nodes.GroupingNode;
import at.jku.mevss.anttracks.util.ThreadUtil;
import at.jku.mevss.trace.gui.chart.base.ApplicationChartFactory;
import at.jku.mevss.trace.gui.chart.base.ChartSynchronizer;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.lifetime.LifetimeStackedAreaJFreeChart;
import at.jku.mevss.trace.gui.frame.main.component.applicationbase.ApplicationBaseTab;
import at.jku.mevss.trace.gui.frame.main.tab.heapstate.HeapStateTask;
import at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization.configurationpane.LifetimeVisualizationConfigurationPane;
import at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization.visualization.DiagramMetric;
import at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization.visualization.LifetimeVisualizationDiagramInfo;
import at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization.visualization.LifetimeVisualizationDiagramTask;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.model.ChartSelection;
import at.jku.mevss.trace.gui.model.SelectedClassifierInfo;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import at.jku.mevss.trace.gui.utils.NodeUtil;
import at.jku.mevss.trace.gui.utils.WindowUtil;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.fxml.FXML;
import javafx.geometry.Insets;
import javafx.scene.control.*;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.scene.layout.VBox;
import javafx.scene.text.Font;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static at.jku.mevss.trace.gui.frame.main.GUI.CLIENT_INFO;

public class LifetimeVisualizationTab extends ApplicationBaseTab {

    @FXML
    private HBox outdatedHeapStateHBox;
    @FXML
    private Label outdatedHeapStateLabel;
    @FXML
    private Button outdatedHeapStateButton;

    @FXML
    private LifetimeVisualizationConfigurationPane configurationPane;

    @FXML
    private Button generateBtn;

    @FXML
    private RadioButton parseAllBtn;

    @FXML
    private RadioButton parseXthHeapBtn;

    @FXML
    private RadioButton parseHeapAllXSecondsBtn;

    @FXML
    private Spinner<Integer> parseHeapAllXSecondsSpinner;

    @FXML
    private Spinner<Integer> parseXthHeapComboSpinner;

    @FXML
    private VBox chartContainerBox;

    @FXML
    private VBox settings;

    @FXML
    private ScrollPane scrollPane;

    private List<LifetimeStackedAreaJFreeChart> curCharts = new ArrayList<>();

    private LifetimeVisualizationInfo lifetimeVisualizationInfo;

    private List<LifetimeVisualizationTab> subTabs = new ArrayList<>();

    private LifetimeVisualizationTab parent;

    private HashMap<LifetimeStackedAreaJFreeChart, ComboBox<String>> selectKeyComboBoxes = new HashMap<>();
    private HashMap<LifetimeStackedAreaJFreeChart, Button> subChartButtons = new HashMap<>();
    private HashMap<LifetimeStackedAreaJFreeChart, Button> subChartNewTabButtons = new HashMap<>();
    private List<Button> examineHeapButtons = new ArrayList<>();

    private static final String stackedChartType = "Stacked";
    private static final String lineChartType = "Line";

    public LifetimeVisualizationTab() {
        FXMLUtil.load(this, LifetimeVisualizationTab.class);
    }

    @Override
    protected void cleanupOnClose() {

    }

    public void init(LifetimeVisualizationInfo lifetimeVisualizationInfo, LifetimeVisualizationTab parent) {
        init(lifetimeVisualizationInfo);
        this.parent = parent;
    }

    public void closeConfigurationSelection() {
        if (configurationPane.isMaxMode()) {
            configurationPane.switchMode();
        }
        settings.setVisible(false);
        settings.setManaged(false);
    }

    public void init(LifetimeVisualizationInfo lifetimeVisualizationInfo) {
        super.init(lifetimeVisualizationInfo.getAppInfo());
        this.lifetimeVisualizationInfo = lifetimeVisualizationInfo;
        configurationPane.setTab(this);
        configurationPane.init(appInfo, lifetimeVisualizationInfo, lifetimeVisualizationInfo.getSelectedClassifierInfo(), true);
        configurationPane.getClassifierSelectionPanel().resetSelected(lifetimeVisualizationInfo.getSelectedClassifierInfo().getSelectedClassifiers().getList());
        configurationPane.getFilterSelectionPanel().resetSelected(lifetimeVisualizationInfo.getSelectedClassifierInfo().getSelectedFilters());
        chartContainerBox.setPadding(new Insets(5));
        scrollPane.setVisible(false);
        disableBtn();
        parseHeapAllXSecondsBtn.selectedProperty().addListener((ObservableValue<? extends Boolean> observable, Boolean oldValue, Boolean newValue) -> {
            if (newValue) {
                parseHeapAllXSecondsSpinner.setDisable(false);
            } else {
                parseHeapAllXSecondsSpinner.setDisable(true);
            }
        });
        parseXthHeapBtn.selectedProperty().addListener((ObservableValue<? extends Boolean> observable, Boolean oldValue, Boolean newValue) -> {
            if (newValue) {
                parseXthHeapComboSpinner.setDisable(false);
            } else {
                parseXthHeapComboSpinner.setDisable(true);
            }
        });
        parseHeapAllXSecondsSpinner.setDisable(true);
        parseHeapAllXSecondsSpinner.focusedProperty().addListener((observable, oldValue, newValue) -> {
            if (!newValue) {
                parseXthHeapComboSpinner.increment(0); // won't change value, but will commit editor
            }
        });
        parseXthHeapComboSpinner.focusedProperty().addListener((observable, oldValue, newValue) -> {
            if (!newValue) {
                parseXthHeapComboSpinner.increment(0); // won't change value, but will commit editor
            }
        });
        parseXthHeapComboSpinner.setDisable(true);
        initDataStructureSwitch();
    }

    private void initDataStructureSwitch() {
        configurationPane.getDataStructureSwitch().setDisable(true);    // no classifiers selected yet
        configurationPane.getDataStructureSwitch().selectedProperty().addListener((obs, wasSelected, isSelected) -> {
            if (wasSelected != isSelected) {
                generateDiagrams();
            }
        });
    }

    public void enableSettings() {
        settings.setVisible(true);
        settings.setManaged(true);
    }

    public void generateDiagrams() {
        //clear old diagram data
        subTabs.clear();
        curCharts.clear();
        selectKeyComboBoxes.clear();
        chartContainerBox.getChildren().clear();
        scrollPane.setVisible(true);
        //unregister from updates through parent
        if (parent != null) {
            removeFromChildTabUpdates(this);
            //we set parent to null here, as we do not have any connection anymore - this also ensures that removeFromChildTabUpdates is not called multiple times
            parent = null;
        }

        generateBtn.setDisable(true);
        closeConfigurationSelection();
        List<Classifier<?>> selectedClassifiers = configurationPane.getClassifierSelectionPanel().getSelected();
        List<Filter> selectedFilters = configurationPane.getFilterSelectionPanel().getSelected();
        ClassifierChain chain = new ClassifierChain(selectedClassifiers);
        Filter[] filters = selectedFilters.toArray(new Filter[0]);
        SelectedClassifierInfo selectedClassifierInfo = new SelectedClassifierInfo(chain, null, selectedFilters);
        lifetimeVisualizationInfo.setSelectedClassifierInfo(selectedClassifierInfo);
        LifetimeVisualizationDiagramInfo diagramInfo = new LifetimeVisualizationDiagramInfo(lifetimeVisualizationInfo,
                                                                                            true,
                                                                                            5,
                                                                                            SeriesSort.AVG,
                                                                                            DiagramMetric.OBJECTS,
                                                                                            new ArrayList<>(),
                                                                                            chain,
                                                                                            filters,
                                                                                            true);
        //create initial chart without data
        LifetimeStackedAreaJFreeChart chart = new LifetimeStackedAreaJFreeChart();
        chart.init(this, diagramInfo, new ChartSynchronizer(), new HashMap<>());
        chart.update(appInfo);
        addDiagramToUi(chart);

        boolean parseXthHeap = parseXthHeapBtn.isSelected();
        boolean parseHeapAllXSeconds = parseHeapAllXSecondsBtn.isSelected();
        int val = parseXthHeap ? parseXthHeapComboSpinner.getValue() : parseHeapAllXSeconds ?
                                                                       (parseHeapAllXSecondsSpinner.getValue()) * 1000 : 0;
        ThreadUtil.startTask(new LifetimeVisualizationDiagramTask(appInfo, chain, filters, lifetimeVisualizationInfo, diagramInfo, this, parseXthHeap,
                                                                  parseHeapAllXSeconds, val));
    }

    private String getClassificationText(LifetimeVisualizationDiagramInfo diagramInfo) {
        StringBuilder sb = new StringBuilder("Level ");
        sb.append(diagramInfo.getSelectedKeys().size() + 1);
        sb.append(": ");
        sb.append(diagramInfo.getClassifierChain().get(0).toString());
        for (int i = 0; i < diagramInfo.getSelectedKeys().size(); i++) {
            sb.append(": ");
            sb.append(diagramInfo.getSelectedKeys().get(i));
            sb.append(", ");
            sb.append(diagramInfo.getClassifierChain().get(i + 1));
        }
        return sb.toString();
    }

    public boolean isDiagramAlreadyOpen(List<String> selectedKeys) {
        for (LifetimeStackedAreaJFreeChart chart : curCharts) {
            if (chart.getDiagramInfo().getSelectedKeys().equals(selectedKeys)) {
                return true;
            }
        }
        return false;
    }

    public void removeAllSubDiagramsOf(LifetimeStackedAreaJFreeChart chart) {
        //remove charts from curCharts list
        int chartIndex = -1;
        for (int i = 0; i < curCharts.size(); i++) {
            LifetimeStackedAreaJFreeChart curChart = curCharts.get(i);
            //as soon as we found the base chart, remove all following charts
            if (chartIndex != -1) {
                curCharts.remove(i);
                selectKeyComboBoxes.remove(curChart);
                i--;
                continue;
            }
            //save index of base chart
            if (curChart == chart) {
                chartIndex = i;
            }
        }

        //remove charts from ui
        //we multiply chartIndex by 4, because for each chart 4 children are inserted
        // and we add 4, as we dont want to remove that chart, but just all children
        chartContainerBox.getChildren().remove(chartIndex * 4 + 4, chartContainerBox.getChildren().size());
    }

    public void openNewTabAndShowCharts(LifetimeStackedAreaJFreeChart baseChart, LifetimeStackedAreaJFreeChart newChart, LifetimeVisualizationTab newTab) {

        subTabs.add(newTab);
        for (int i = 0; i < curCharts.size(); i++) {
            LifetimeStackedAreaJFreeChart curChart = curCharts.get(i);
            LifetimeStackedAreaJFreeChart copiedChart = createCopyOfChart(curChart, newTab, newChart.getChartSynchronizer());
            newTab.addDiagramToUi(copiedChart);
            copiedChart.update(appInfo);
            //we stop copying charts after the base chart
            if (curChart == baseChart) {
                break;
            }
        }
        newTab.addDiagramToUi(newChart);
        CLIENT_INFO.getMainFrame().addTab(newTab);
    }

    private LifetimeStackedAreaJFreeChart createCopyOfChart(LifetimeStackedAreaJFreeChart chartToCopy, LifetimeVisualizationTab newTab,
                                                            ChartSynchronizer newChartSynchronizer) {
        LifetimeStackedAreaJFreeChart copiedChart = new LifetimeStackedAreaJFreeChart();
        LifetimeVisualizationDiagramInfo oldDiagramInfo = chartToCopy.getDiagramInfo();
        LifetimeVisualizationDiagramInfo newDiagramInfo = new LifetimeVisualizationDiagramInfo(newTab.getLifetimeVisualizationInfo(),
                                                                                               oldDiagramInfo.isUseLineChart(),
                                                                                               oldDiagramInfo.getSeriesCount(),
                                                                                               oldDiagramInfo.getSeriesSort(),
                                                                                               oldDiagramInfo.getDiagramMetric(),
                                                                                               oldDiagramInfo.getSelectedKeys(),
                                                                                               oldDiagramInfo.getClassifierChain(),
                                                                                               oldDiagramInfo.getFilters(),
                                                                                               oldDiagramInfo.isShowOtherSeries());
        copiedChart.init(newTab, newDiagramInfo, newChartSynchronizer,
                         chartToCopy.getGroupings());
        return copiedChart;
    }

    public void addDiagramToUi(LifetimeStackedAreaJFreeChart chart) {
        curCharts.add(chart);

        chartContainerBox.getChildren().add(createClassificationLabel(chart));
        HBox curChartComponentContainer = new HBox(createSettingsVBox(chart), createChartContainerBox(chart));
        chartContainerBox.getChildren().add(curChartComponentContainer);
        chartContainerBox.getChildren().add(chart.getTable());
        chartContainerBox.getChildren().add(createButtonRow(chart));

        chart.setOnMouseClicked(evt -> chart.requestFocus());   // for some reason chart does not receive focus when clicked...
        NodeUtil.ignoreScrollingUnlessFocused(chart, scrollPane);
        NodeUtil.ignoreScrollingUnlessFocused(chart.getTable(), scrollPane);
    }

    public void setExamineButtonsDisabled(boolean disabled) {
        for (Button curBtn : examineHeapButtons) {
            curBtn.setDisable(disabled);
        }
    }

    private HBox createButtonRow(LifetimeStackedAreaJFreeChart chart) {
        HBox hBox = new HBox();
        hBox.setSpacing(10);
        hBox.setPadding(new Insets(10, 0, 10, 0));

        Button examineHeapBtn = new Button("Examine Heap");
        List<Classifier<?>> selectedClassifiers = configurationPane.getClassifierSelectionPanel().getSelected();
        List<Filter> selectedFilters = configurationPane.getFilterSelectionPanel().getSelected();
        ClassifierChain chain = new ClassifierChain(selectedClassifiers);
        examineHeapBtn.setOnAction(event -> ThreadUtil.startTask(new HeapStateTask(appInfo,
                                                                                   (long) chart.getChartSynchronizer().getSelection().get(0).x,
                                                                                   chain, null, selectedFilters)));
        examineHeapBtn.setDisable(true);
        examineHeapButtons.add(examineHeapBtn);
        hBox.getChildren().add(examineHeapBtn);

        Label chooseSeriesLbl = new Label("Choose Series:");
        chooseSeriesLbl.setPadding(new Insets(5, 0, 0, 0));
        hBox.getChildren().add(chooseSeriesLbl);

        ComboBox<String> keyComboBox = new ComboBox<>();
        keyComboBox.setPrefWidth(200);
        keyComboBox.setDisable(true);
        selectKeyComboBoxes.put(chart, keyComboBox);
        hBox.getChildren().add(keyComboBox);

        Button openSubChartBtn = new Button("Open Sub-Chart");
        openSubChartBtn.setOnAction(event -> chart.openSubChart(keyComboBox.getSelectionModel().getSelectedItem()));
        subChartButtons.put(chart, openSubChartBtn);
        hBox.getChildren().add(openSubChartBtn);

        Button openSubChartNewTabBtn = new Button("Open Sub-Chart in New Tab");
        openSubChartNewTabBtn.setOnAction(event -> chart.openChartInNewTab(keyComboBox.getSelectionModel().getSelectedItem()));
        subChartNewTabButtons.put(chart, openSubChartNewTabBtn);
        hBox.getChildren().add(openSubChartNewTabBtn);
        updateKeySelectionComboBox(chart);
        return hBox;
    }

    public void updateKeySelectionComboBox(LifetimeStackedAreaJFreeChart chart) {
        ComboBox<String> curKeyComboBox = selectKeyComboBoxes.get(chart);
        ArrayList<String> selectedSeries = chart.getSelectedSeriesKeys();
        if (curKeyComboBox != null && selectedSeries != null) {
            curKeyComboBox.getItems().clear();
            curKeyComboBox.getItems().addAll(selectedSeries);
            boolean empty = curKeyComboBox.getItems().size() == 0;
            boolean canOpenSubChart = !empty && chart.hasSubLevel();
            Button subChartBtn = subChartButtons.get(chart);
            Button subChartNewTabBtn = subChartNewTabButtons.get(chart);
            if (!empty) {
                curKeyComboBox.setValue(selectedSeries.get(0));
            }
            if (canOpenSubChart) {
                subChartBtn.setDisable(false);
                subChartNewTabBtn.setDisable(false);
                curKeyComboBox.setDisable(false);
            } else {
                subChartBtn.setDisable(true);
                subChartNewTabBtn.setDisable(true);
                curKeyComboBox.setDisable(true);
            }
        }
    }

    public void updateChartData(Map<Long, GroupingNode> rootMapGroupings) {
        for (LifetimeStackedAreaJFreeChart curChart : curCharts) {
            //we create a shallow copy of the map, because we synchronize on the map
            curChart.setGroupings(new HashMap<>(rootMapGroupings));
            curChart.update(appInfo);
        }
        for (LifetimeVisualizationTab tab : subTabs) {
            tab.updateChartData(rootMapGroupings);
        }
    }

    public void removeFromChildTabUpdates(LifetimeVisualizationTab tabToRemove) {
        subTabs.remove(tabToRemove);
    }

    private Label createClassificationLabel(LifetimeStackedAreaJFreeChart chart) {
        Label classificationLbl = new Label(getClassificationText(chart.getDiagramInfo()));
        if (chartContainerBox.getChildren().size() != 0) {
            classificationLbl.setPadding(new Insets(20, 0, 5, 0));
        } else {
            classificationLbl.setPadding(new Insets(0, 0, 5, 0));
        }
        classificationLbl.setFont(new Font(classificationLbl.getFont().getName(), 20));
        return classificationLbl;
    }

    private VBox createChartContainerBox(LifetimeStackedAreaJFreeChart chart) {
        VBox curChartContainer = new VBox(chart);
        HBox.setHgrow(curChartContainer, Priority.ALWAYS);
        VBox.setVgrow(chart, Priority.ALWAYS);
        return curChartContainer;
    }

    private VBox createSettingsVBox(LifetimeStackedAreaJFreeChart chart) {
        VBox settingsVBox = new VBox(createChartTypeHBox(chart), createSeriesCountHBox(chart), createSeriesSortHBox(chart), createMemoryConsumptionHBox(chart),
                                     createOtherHBox(chart));
        settingsVBox.setSpacing(10);
        settingsVBox.setPrefWidth(250);
        return settingsVBox;
    }

    private HBox createChartTypeHBox(LifetimeStackedAreaJFreeChart chart) {
        Label chartTypeLbl = new Label("Chart Type:");
        chartTypeLbl.setPadding(new Insets(5, 0, 0, 0));
        chartTypeLbl.setPrefWidth(90);
        ComboBox<String> chartTypeComboBox = new ComboBox<>();
        chartTypeComboBox.setPrefWidth(130);
        chartTypeComboBox.getItems().addAll(lineChartType, stackedChartType);
        chartTypeComboBox.setValue(chart.getDiagramInfo().isUseLineChart() ? lineChartType : stackedChartType);
        chartTypeComboBox.valueProperty().addListener(new ChangeListener<String>() {
            @Override
            public void changed(ObservableValue<? extends String> observable, String oldValue, String newValue) {
                chart.setUseLineChart(newValue.equals(lineChartType));
                chart.update(appInfo);
                ChartSynchronizer chartSynchronizer = chart.getChartSynchronizer();
                ChartSelection lastChartSelection = chartSynchronizer.getLastChartSelection();
                // refresh chart selection with new chart
                if (lastChartSelection != null) {
                    chartSynchronizer.sync(new ChartSelection(0, 0, 0, 0, ""));
                    chartSynchronizer.sync(lastChartSelection);
                }
            }
        });
        HBox seriesSortHBox = new HBox(chartTypeLbl, chartTypeComboBox);
        seriesSortHBox.setSpacing(10);
        return seriesSortHBox;
    }

    private HBox createOtherHBox(LifetimeStackedAreaJFreeChart chart) {
        CheckBox displayOtherSeriesCheckBox = new CheckBox("Show 'Other' Series");
        displayOtherSeriesCheckBox.setSelected(true);
        displayOtherSeriesCheckBox.selectedProperty().addListener(new ChangeListener<Boolean>() {
            @Override
            public void changed(ObservableValue<? extends Boolean> observable, Boolean oldValue, Boolean newValue) {
                if (oldValue != newValue) {
                    chart.getDiagramInfo().setShowOtherSeries(newValue);
                    chart.update(appInfo);
                }
            }
        });
        HBox seriesSortHBox = new HBox(displayOtherSeriesCheckBox);
        seriesSortHBox.setSpacing(10);
        return seriesSortHBox;
    }

    private HBox createSeriesSortHBox(LifetimeStackedAreaJFreeChart chart) {
        Label seriesSortLbl = new Label("Sort Series By:");
        seriesSortLbl.setPadding(new Insets(5, 0, 0, 0));
        seriesSortLbl.setPrefWidth(90);
        ComboBox<SeriesSort> seriesSortComboBox = new ComboBox<>();
        seriesSortComboBox.setPrefWidth(130);
        seriesSortComboBox.getItems().addAll(SeriesSort.values());
        seriesSortComboBox.setValue(chart.getDiagramInfo().getSeriesSort());
        seriesSortComboBox.valueProperty().addListener(new ChangeListener<SeriesSort>() {
            @Override
            public void changed(ObservableValue<? extends SeriesSort> observable, SeriesSort oldValue, SeriesSort newValue) {
                chart.getDiagramInfo().setSeriesSort(newValue);
                chart.update(appInfo);
            }
        });
        HBox seriesSortHBox = new HBox(seriesSortLbl, seriesSortComboBox);
        seriesSortHBox.setSpacing(10);
        return seriesSortHBox;
    }

    private HBox createSeriesCountHBox(LifetimeStackedAreaJFreeChart chart) {
        Label seriesCountLbl = new Label("Series Count:");
        seriesCountLbl.setPadding(new Insets(5, 0, 0, 0));
        seriesCountLbl.setPrefWidth(90);
        ComboBox<Integer> curSeriesCountComboBox = new ComboBox<>();
        curSeriesCountComboBox.setPrefWidth(130);
        for (int i = 1; i <= 10; i++) {
            curSeriesCountComboBox.getItems().add(i);
        }
        curSeriesCountComboBox.setValue(chart.getDiagramInfo().getSeriesCount());
        curSeriesCountComboBox.valueProperty().addListener(new ChangeListener<Integer>() {
            @Override
            public void changed(ObservableValue<? extends Integer> observable, Integer oldValue, Integer newValue) {
                chart.getDiagramInfo().setSeriesCount(newValue);
                chart.update(appInfo);
            }
        });
        HBox seriesCountHBox = new HBox(seriesCountLbl, curSeriesCountComboBox);
        seriesCountHBox.setSpacing(10);
        return seriesCountHBox;
    }

    private HBox createMemoryConsumptionHBox(LifetimeStackedAreaJFreeChart chart) {
        Label memoryConsumptionLbl = new Label("Memory Unit:");
        memoryConsumptionLbl.setPadding(new Insets(5, 0, 0, 0));
        memoryConsumptionLbl.setPrefWidth(90);
        ComboBox<DiagramMetric> memoryConsumptionComboBox = new ComboBox<>();
        memoryConsumptionComboBox.setPrefWidth(130);
        memoryConsumptionComboBox.getItems().addAll(DiagramMetric.values());
        memoryConsumptionComboBox.setValue(chart.getDiagramInfo().getDiagramMetric());
        memoryConsumptionComboBox.valueProperty().addListener(new ChangeListener<DiagramMetric>() {
            @Override
            public void changed(ObservableValue<? extends DiagramMetric> observable,
                                DiagramMetric oldValue,
                                DiagramMetric newValue) {
                chart.getDiagramInfo().setDiagramMetric(newValue);
                chart.update(appInfo);
            }
        });
        HBox memoryConsumptionHBox = new HBox(memoryConsumptionLbl, memoryConsumptionComboBox);
        memoryConsumptionHBox.setSpacing(10);
        return memoryConsumptionHBox;
    }

    public LifetimeVisualizationConfigurationPane getConfigurationPane() {
        return configurationPane;
    }

    public void enableBtn() {
        generateBtn.setDisable(false);
    }

    public void disableBtn() {
        generateBtn.setDisable(true);
    }

    public void showScrollPane() {
        scrollPane.setVisible(true);
    }

    public LifetimeVisualizationInfo getLifetimeVisualizationInfo() {
        return lifetimeVisualizationInfo;
    }

    @Override
    protected void appInfoChangeAction(AppInfo.ChangeType type) {
    }

}
