/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization.configurationpane;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierFactory;
import at.jku.mevss.anttracks.classification.Filter;
import at.jku.mevss.anttracks.classification.FilterFactory;
import at.jku.mevss.trace.gui.classification.component.configurationpane.ConfigurationPane;
import at.jku.mevss.trace.gui.classification.component.selectionpane.ClassificationSelectionPane;
import at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization.LifetimeVisualizationTab;
import at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization.visualization.LifetimeVisualizationDiagramTask;

public class LifetimeVisualizationConfigurationPane extends ConfigurationPane {
    private LifetimeVisualizationTab tab;

    public void setTab(LifetimeVisualizationTab tab) {
        this.tab = tab;
    }

    private void checkClassifiersAndSetAvailableActions() {
        if (tab != null) {
            int size = this.getClassifierSelectionPanel().getSelected().size();
            if (size > 0) {
                tab.enableBtn();
                getDataStructureSwitch().setDisable(!LifetimeVisualizationDiagramTask.USE_INDEX_BASED_HEAP);
            } else {
                tab.disableBtn();
                getDataStructureSwitch().setDisable(true);
            }
        }
    }

    @Override
    protected ClassificationSelectionPane.ClassificationSelectionListener<Classifier<?>, ClassifierFactory> createClassifierListener() {
        return new ClassificationSelectionPane.ClassificationSelectionListener<Classifier<?>, ClassifierFactory>() {
            @Override
            public void selected(ClassificationSelectionPane<Classifier<?>, ClassifierFactory> sender, Classifier<?> x) {
                checkClassifiersAndSetAvailableActions();
            }

            @Override
            public void deselected(ClassificationSelectionPane<Classifier<?>, ClassifierFactory> sender, Classifier<?> x) {
                checkClassifiersAndSetAvailableActions();
            }

            @Override
            public void propertiesChanged(ClassificationSelectionPane<Classifier<?>, ClassifierFactory> sender, Classifier<?> x) {
                //Do Nothing
            }
        };
    }

    @Override
    protected ClassificationSelectionPane.ClassificationSelectionListener<Filter, FilterFactory> createFilterListener() {
        return new ClassificationSelectionPane.ClassificationSelectionListener<Filter, FilterFactory>() {
            @Override
            public void selected(ClassificationSelectionPane<Filter, FilterFactory> sender, Filter x) {
                checkClassifiersAndSetAvailableActions();
            }

            @Override
            public void deselected(ClassificationSelectionPane<Filter, FilterFactory> sender, Filter x) {
                checkClassifiersAndSetAvailableActions();
            }

            @Override
            public void propertiesChanged(ClassificationSelectionPane<Filter, FilterFactory> sender, Filter x) {
                //Do Nothing
            }
        };
    }

    @Override
    public void setLoadingMode(boolean isLoading) {
        //Do Nothing
    }

    @Override
    public void setAppropriateConfigView() {
        //DO Nothing
    }

    @Override
    public void switchMode() {
        tab.enableSettings();
        super.switchMode();
    }
}
