package at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization.table;

import at.jku.mevss.trace.gui.utils.FXMLUtil;
import javafx.beans.binding.DoubleBinding;
import javafx.beans.property.SimpleDoubleProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;

public class GrowthTable extends TableView<GrowthTableRow> {

    private TableColumn<GrowthTableRow, Number> absoluteGrowthColumn;

    public GrowthTable() {
        FXMLUtil.load(this, GrowthTable.class);
    }

    public void init() {
        // Define columns
        TableColumn<GrowthTableRow, String> labelColumn = new TableColumn<>("Name");
        labelColumn.setCellValueFactory(param -> param.getValue().labelProperty);

        TableColumn<GrowthTableRow, String> relativeGrowthColumn = new TableColumn<>("Relative Growth");
        relativeGrowthColumn.setCellValueFactory(param -> new SimpleStringProperty(param.getValue().relativeGrowth.get()));
        relativeGrowthColumn.setComparator((str1, str2) -> {
            Double val1 = tryParseDouble(str1);
            Double val2 = tryParseDouble(str2);
            if (val1 != null) {
                if (val2 != null) {
                    return Double.compare(val1, val2);
                } else {
                    return 1;
                }
            } else if (val2 != null) {
                return -1;
            } else {
                return 1; //could also return 0 here, but it makes no difference and probably saves another comparison
            }
        });

        absoluteGrowthColumn = new TableColumn<>("Absolute Growth");
        absoluteGrowthColumn.setCellValueFactory(param -> new SimpleDoubleProperty(param.getValue().absoluteGrowth.get()));

        getColumns().add(labelColumn);
        getColumns().add(relativeGrowthColumn);
        getColumns().add(absoluteGrowthColumn);

        setMinHeight(200);

        // Define column widths
        DoubleBinding halfSize = widthProperty().divide(2);
        DoubleBinding sixth = halfSize.divide(3);
        labelColumn.prefWidthProperty().bind(halfSize);
        relativeGrowthColumn.prefWidthProperty().bind(sixth);
        absoluteGrowthColumn.prefWidthProperty().bind(sixth.multiply(2)
                                                           .subtract(20)); // no horizontal scrollbar please

    }

    //returns null if not parsable
    private Double tryParseDouble(String input) {
        try {
            return Double.parseDouble(input);
        } catch (NumberFormatException e) {
            return null;
        }
    }

    public void setSorting() {
        absoluteGrowthColumn.setSortType(TableColumn.SortType.DESCENDING);
        getSortOrder().add(absoluteGrowthColumn);
    }
}
