/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.model;

import at.jku.mevss.anttracks.classification.ClassifierFactoryList;
import at.jku.mevss.anttracks.classification.FilterFactoryList;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierSourceCollection;
import at.jku.mevss.anttracks.diff.Interval;
import at.jku.mevss.anttracks.heap.DetailedHeap;
import at.jku.mevss.anttracks.heap.IndexBasedHeap;
import at.jku.mevss.anttracks.heap.symbols.Symbols;
import at.jku.mevss.trace.gui.utils.Consts;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.function.Supplier;

public class DiffInfo extends AvailableClassifierInfo {

    private final List<Long> times;
    private Interval.IntervalInfo heapDiff;

    private DetailedHeap latestHeap;
    private final AppInfo appInfo;

    public DiffInfo(AppInfo appInfo, List<ChartSelection> times) {
        this.appInfo = appInfo;
        this.times = new ArrayList<>();
        times.forEach(time -> this.times.add((long) time.x));
        Collections.sort(this.times);

        availableClassifier = new ClassifierFactoryList(getSymbolsSupplier(),
                                                        getFastHeapSupplier(),
                                                        Consts.CLASSIFIERS_DIRECTORY,
                                                        ClassifierSourceCollection.DETAILEDHEAP);
        availableFilter = new FilterFactoryList(getSymbolsSupplier(), getFastHeapSupplier(), Consts.FILTERS_DIRECTORY, ClassifierSourceCollection.DETAILEDHEAP);
    }

    public long getTime(int i) {
        if (i >= times.size()) {
            throw new IndexOutOfBoundsException("Requested time entry not existing. Requested index: " + i + ", times: " + times.size());
        }
        return times.get(i);
    }

    public void setHeapDiff(Interval.IntervalInfo heapDiff) {
        this.heapDiff = heapDiff;
    }

    public Interval.IntervalInfo getHeapDiff() {
        return heapDiff;
    }

    public void setHeap(DetailedHeap lastHeap) {
        latestHeap = lastHeap;
    }

    public AppInfo getAppInfo() {
        return appInfo;
    }

    public Supplier<DetailedHeap> getHeapSupplier() {
        return () -> latestHeap;
    }

    @Override
    public Supplier<IndexBasedHeap> getFastHeapSupplier() {
        return () -> null;
    }

    @Override
    public Supplier<Symbols> getSymbolsSupplier() {
        return () -> latestHeap.getSymbols();
    }
}
