/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.model.tablecellrenderer;

import at.jku.mevss.trace.gui.component.treetable.TreeTable;

import javax.swing.*;
import javax.swing.table.DefaultTableCellRenderer;
import java.awt.*;

public abstract class BarCellRenderer extends DefaultTableCellRenderer {

    private static final long serialVersionUID = -6648271521788186225L;
    private static final Color[] percentageColors = {new Color(160, 192, 56), new Color(222, 129, 144), new Color(255, 204, 0)};

    int colorId = 0;
    String textToShow = "";
    double fillRatio = 0.0;

    @Override
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
        if (value == null) {
            return super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
        }

        colorId = ((TreeTable<?>) table).getValueAt(row, column).node.getSubTreeLevel() % percentageColors.length;
        fillRatio = defineFillRatio(value);
        textToShow = defineTextToShow(value);
        // setHorizontalAlignment(SwingConstants.RIGHT);
        // setText(String.valueOf(percentage));
        return this;
    }

    abstract String defineTextToShow(Object value);

    abstract double defineFillRatio(Object value);

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2d = (Graphics2D) g;
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g2d.setColor(percentageColors[colorId]);

        // Bar
        double barWidth = fillRatio * getWidth();
        g2d.fillRect(0, 0, (int) barWidth, getHeight());

        // Empty space
        g2d.setColor(getBackground());
        g2d.fillRect((int) barWidth + 1, 0, (int) (getWidth() - barWidth), getHeight());

        // Right-aligned text
        g2d.setColor(Color.BLACK);
        float textX = getWidth() - g2d.getFontMetrics().stringWidth(textToShow) - 2.0f;
        // http://stackoverflow.com/questions/27706197/how-can-i-center-graphics-drawstring-in-java
        float textY = ((getHeight() - g2d.getFontMetrics().getHeight()) / 2) + g2d.getFontMetrics().getAscent();
        g2d.drawString(textToShow, textX, textY);
    }
}
