/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.utils;

import at.jku.mevss.anttracks.util.ApplicationStatistics;
import at.jku.mevss.anttracks.util.ApplicationStatistics.Measurement;
import io.micrometer.core.instrument.Timer;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.concurrent.Task;

import java.util.Arrays;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;

import static at.jku.mevss.trace.gui.frame.main.GUI.CLIENT_INFO;

public abstract class AntTask<T> extends Task<T> {
    protected final Logger LOGGER;
    private final Timer meter;
    protected BooleanProperty cancelProperty;

    private Operation operation;

    public AntTask() {
        LOGGER = Logger.getLogger(this.getClass().getSimpleName());
        cancelProperty = new SimpleBooleanProperty(false);

        this.meter = CLIENT_INFO.meterRegistry.timer(getClass().getSimpleName() + " - backgroundWork");

        updateProgress(0, 0);
        PlatformUtil.runAndWait(() -> {
            operation = new Operation(this);
            setOnSucceeded(wse -> {
                Measurement m = ApplicationStatistics.getInstance().createMeasurement(getClass().getSimpleName() + " - suceeded");
                suceeded();
                m.end();
            });

            setOnCancelled(wse -> {
                LOGGER.log(Level.SEVERE, "Cancelled AntTask.\n" + wse);
                cancelProperty.setValue(true);
            });

            setOnFailed(wse -> {
                LOGGER.log(Level.SEVERE,
                           "Failed AntTask.\n" + getException().getMessage() + "\n" + Arrays.stream(getException().getStackTrace())
                                                                                            .map(x -> x.toString())
                                                                                            .reduce((a, b) -> a + "\n" + b));
                getException().printStackTrace();
            });
        });
    }

    public boolean showOnUI() {
        return true;
    }

    @Override
    protected T call() throws Exception {
        if (showOnUI()) {
            CLIENT_INFO.operationManager.addOperation(operation);
        }
        Measurement m = ApplicationStatistics.getInstance().createMeasurement(getClass().getSimpleName() + " - backgroundWork");

        T ret = backgroundWork();
        m.end();
        meter.record(m.getDuration(), TimeUnit.NANOSECONDS);
        operation.close();
        return ret;
    }

    protected abstract T backgroundWork() throws Exception;

    protected abstract void suceeded();

}
