/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.utils;

import at.jku.mevss.trace.gui.frame.main.MainFrame;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.control.DialogPane;

import java.io.IOException;
import java.net.URL;
import java.util.Optional;
import java.util.logging.Logger;

public class FXMLUtil {

    static Logger LOGGER = Logger.getLogger(FXMLUtil.class.getName());
    static Class<?> mainCSS = MainFrame.class;

    public static void load(Object caller, Class<?> klass) {
        if (DialogPane.class.isAssignableFrom(klass)) {
            loadCSS(mainCSS, caller);
        }
        loadCSS(klass, caller);

        // if (caller.getClass() == klass || (caller.getClass().isAnonymousClass() && caller.getClass().getSuperclass() == klass)) {
        URL fxmlFile = klass.getResource(klass.getSimpleName() + ".fxml");
        FXMLLoader fxmlLoader = new FXMLLoader(fxmlFile);
        if (caller != null) {
            fxmlLoader.setController(caller);
        }
        if (caller != null) {
            fxmlLoader.setRoot(caller);
        }

        try {
            fxmlLoader.load();
        } catch (IOException exception) {
            throw new RuntimeException(exception);
        }
        // }
    }

    public static boolean shouldInitialize(Object caller, Class<?> klass) {
        return caller.getClass() == klass;
    }

    private static void loadCSS(Class<?> klass, Object caller) {
        try {
            Optional.of(klass.getResource(klass.getSimpleName() + ".css")).ifPresent(value -> {
                if (caller instanceof Parent) {
                    ((Parent) caller).getStylesheets().add(value.toExternalForm());
                }
            });
        } catch (NullPointerException e) {
            if (FXMLUtil.class.desiredAssertionStatus()) {
                LOGGER.warning("No CSS defined for " + klass.getName());
            }
        }
    }
}
