/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.utils;

import at.jku.mevss.anttracks.util.ImagePack;
import javafx.embed.swing.SwingFXUtils;
import javafx.scene.image.ImageView;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.FileInputStream;
import java.util.logging.Logger;

public class ImageUtil {
    private static final Logger LOGGER = Logger.getLogger(ImageUtil.class.getSimpleName());

    public static BufferedImage loadResourceImage(String name, String fileName) {
        return loadImage(name, "/images/" + fileName);
    }

    public static BufferedImage loadImage(String name, String path) {
        try {
            return ImageIO.read(ImageUtil.class.getResourceAsStream(path));
        } catch (Exception e) {
            try {
                return ImageIO.read(new FileInputStream("./resources/" + path));
            } catch (Throwable t) {
                LOGGER.warning("Could not load \"" + name + "\" image from " + path + "\n" + e);
                return null;
            }
        }
    }

    public static ImagePack getResourceImagePack(String name, String fileName) {
        return new ImagePack() {
            BufferedImage image = ImageUtil.loadResourceImage(name, fileName);
            ImageIcon icon = ImageUtil.loadResourceIcon(name, fileName);

            @Override
            public ImageView getAsNewNode() {
                ImageView ret = ImageUtil.getIconNode(icon);
                ret.setAccessibleText(name);
                return ret;
            }

            @Override
            public BufferedImage getImage() {
                return image;
            }

            @Override
            public ImageIcon getIcon() {
                return icon;
            }
        };
    }

    public static ImageIcon loadResourceIcon(String name, String fileName) {
        return asIcon(loadResourceImage(name, fileName));
    }

    public static ImageIcon loadIcon(String name, String path) {
        return asIcon(loadImage(name, path));
    }

    public static ImageIcon asIcon(BufferedImage image) {
        if (image == null) {
            return null;
        }
        return asIcon(image, Consts.ICON_SIZE);
    }

    public static ImageIcon asIcon(BufferedImage image, int iconSize) {
        return new ImageIcon(image.getScaledInstance(iconSize, iconSize, Image.SCALE_SMOOTH));
    }

    public static ImageView getIconNode(BufferedImage image) {
        ImageView iconNode = new ImageView(SwingFXUtils.toFXImage(image, null));
        iconNode.setFitWidth(Consts.ICON_SIZE);
        iconNode.setFitHeight(Consts.ICON_SIZE);
        return iconNode;
    }

    public static ImageView getIconNode(Icon imageIcon) {
        BufferedImage image = new BufferedImage(imageIcon.getIconWidth(), imageIcon.getIconHeight(), BufferedImage.TYPE_INT_ARGB);
        Graphics g = image.createGraphics();
        imageIcon.paintIcon(null, g, 0, 0);
        g.dispose();
        return getIconNode(image);
    }
}
