package at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment

import at.jku.mevss.anttracks.classification.trees.ClassificationTree
import at.jku.mevss.anttracks.util.ThreadUtil
import at.jku.mevss.trace.gui.classification.dialog.properties.ClassificationPropertiesDialog
import at.jku.mevss.trace.gui.frame.main.component.applicationbase.ApplicationBaseTab
import at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment.chart.DataStructureDevelopmentChartTask
import at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment.configurationpane.DataStructureDevelopmentConfigurationPane
import at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment.parser.DataStructureDevelopmentParserTaskResult
import at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment.tableview.DataStructureDevelopmentClassificationTreeTableView
import at.jku.mevss.trace.gui.model.AppInfo
import at.jku.mevss.trace.gui.model.DataStructureDevelopmentInfo
import at.jku.mevss.trace.gui.utils.FXMLUtil
import at.jku.mevss.trace.gui.utils.NodeUtil
import javafx.collections.ObservableList
import javafx.fxml.FXML
import javafx.scene.chart.BarChart
import javafx.scene.chart.CategoryAxis
import javafx.scene.chart.NumberAxis
import javafx.scene.chart.XYChart
import javafx.scene.control.Label
import javafx.scene.control.ScrollPane
import javafx.scene.control.TextArea
import javafx.scene.control.Tooltip

class DataStructureDevelopmentOverviewTab : ApplicationBaseTab() {
    @FXML
    private lateinit var containerScrollPane: ScrollPane
    @FXML
    private lateinit var growthBarChart: BarChart<String, Number>
    @FXML
    private lateinit var xAxis: CategoryAxis
    @FXML
    private lateinit var yAxis: NumberAxis
    @FXML
    private lateinit var configurationPane: DataStructureDevelopmentConfigurationPane
    @FXML
    private lateinit var treeTableView: DataStructureDevelopmentClassificationTreeTableView

    private var dataStructureDevelopmentInfo: DataStructureDevelopmentInfo? = null

    init {
        FXMLUtil.load(this, DataStructureDevelopmentOverviewTab::class.java)
    }

    fun init(appInfo: AppInfo, startTime: Long, endTime: Long) {
        super.init(appInfo)
        dataStructureDevelopmentInfo = DataStructureDevelopmentInfo(appInfo, startTime, endTime)
        text = "Overview"

        configurationPane.init(this, dataStructureDevelopmentInfo!!)

        treeTableView.init(info = dataStructureDevelopmentInfo!!,
                           selectLocalClassifiers = { ClassificationPropertiesDialog.showDialogForClassifier(it, dataStructureDevelopmentInfo, this.content) },
                           addTab = { if (!tabPane.tabs.contains(it)) tabPane.tabs.add(it) },
                           switchToTab = { tabPane.selectionModel.select(it) },
                           showTTVOnScreen = { tabPane.selectionModel.select(this) },
                           setSelectedClassifiers = { configurationPane.classifierSelectionPanel?.resetSelected(it.list) })

        NodeUtil.ignoreScrollingUnlessFocused(treeTableView, containerScrollPane)
    }

    override fun cleanupOnClose() {

    }

    override fun appInfoChangeAction(type: AppInfo.ChangeType) {

    }

    fun updateChart(series: ObservableList<XYChart.Series<String, Number>>) {
        fun initLegend() {
            growthBarChart.lookupAll(".chart-legend-item").forEach { legendItem ->
                val legendItemLabel = (legendItem as Label).text
                Tooltip.install(legendItem, Tooltip("Click to sort by $legendItemLabel"))
                legendItem.setOnMouseEntered { legendItem.setStyle("-fx-underline: true") }
                legendItem.setOnMouseExited { legendItem.setStyle("-fx-underline: false") }
                legendItem.setOnMouseClicked {
                    val sorting = findByText(legendItemLabel)

                    val barChartInitTask = DataStructureDevelopmentChartTask(dataStructureDevelopmentInfo!!, this, sorting!!)
                    tasks.add(barChartInitTask)
                    ThreadUtil.startTask<ObservableList<XYChart.Series<String, Number>>>(barChartInitTask)
                }
            }
        }

        fun initSeriesNodes() {
            // clicking a bar in the chart highlights the corresponding entry in the TreeTableView
            growthBarChart.data.forEach { s ->
                s.data.forEach { data ->
                    Tooltip.install(data.node, Tooltip("Click to highlight object in tree table"))
                    data.node.style = "-fx-opacity: 0.6"
                    data.node.setOnMouseEntered { data.node.style = "-fx-opacity: 1" }
                    data.node.setOnMouseExited { data.node.style = "-fx-opacity: 0.6" }
                    data.node.setOnMouseClicked {
                        // the address as used in the address classifier
                        treeTableView.showAndSelect(data.xValue.split("@".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()[1].replace(" ", ""))
                    }
                }
            }
        }

        xAxis.isAutoRanging = true
        yAxis.isAutoRanging = true
        growthBarChart.data = series
        initLegend()
        initSeriesNodes()
    }

    fun updateParserResult(data: DataStructureDevelopmentParserTaskResult) {
        dataStructureDevelopmentInfo!!.setDataStructureDevelopmentParserTaskResult(data)
        configurationPane.classify()
    }

    fun updateClassificationResult(tree: ClassificationTree) {
        treeTableView.setRoot(tree, false, false)
        initBarChart()
    }

    private fun initBarChart() {
        if (dataStructureDevelopmentInfo!!.developments.isEmpty()) {
            return
        }

        val barChartInitTask = DataStructureDevelopmentChartTask(dataStructureDevelopmentInfo!!, this, GrowthType.TRANSITIVE)
        tasks.add(barChartInitTask)
        ThreadUtil.startTask<ObservableList<XYChart.Series<String, Number>>>(barChartInitTask)
    }
}
