package at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment.chart

import at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment.DataStructureDevelopmentOverviewTab
import at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment.GrowthType
import at.jku.mevss.trace.gui.model.DataStructureDevelopmentInfo
import at.jku.mevss.trace.gui.utils.AntTask
import javafx.collections.FXCollections
import javafx.collections.ObservableList
import javafx.scene.chart.XYChart
import java.util.concurrent.ExecutionException

class DataStructureDevelopmentChartTask(private val info: DataStructureDevelopmentInfo,
                                        private val tab: DataStructureDevelopmentOverviewTab?,
                                        private val sorting: GrowthType) : AntTask<ObservableList<XYChart.Series<String, Number>>>() {
    @Throws(Exception::class)
    override fun backgroundWork(): ObservableList<XYChart.Series<String, Number>> {
        updateTitle("Initializing data structure growth chart")
        // find the 10 data structures with the strongest growth and put them into the chart
        updateMessage("Finding data structures with strongest growth...")

        // sort growths
        val sortedGrowths = info.developments.sortedBy {
            when (sorting) {
                GrowthType.TRANSITIVE -> -it.transitiveHeapGrowthPortion()
                GrowthType.RETAINED -> -it.retainedHeapGrowthPortion()
                GrowthType.DATA_STRUCTURE -> -it.dataStructureHeapGrowthPortion()
                GrowthType.DEEP_DATA_STRUCTURE -> -it.deepDataStructureHeapGrowthPortion()
            }
        }

        // put them into series
        val transitiveClosureGrowthSeries = XYChart.Series<String, Number>()
        val retainedClosureGrowthSeries = XYChart.Series<String, Number>()
        val dataStructureClosureGrowthSeries = XYChart.Series<String, Number>()
        val deepDataStructureClosureGrowthSeries = XYChart.Series<String, Number>()
        transitiveClosureGrowthSeries.name = GrowthType.TRANSITIVE.text
        retainedClosureGrowthSeries.name = GrowthType.RETAINED.text
        dataStructureClosureGrowthSeries.name = GrowthType.DATA_STRUCTURE.text
        deepDataStructureClosureGrowthSeries.name = GrowthType.DEEP_DATA_STRUCTURE.text

        sortedGrowths.stream()
                .limit(N_BARS.toLong())
                .forEach { dsd ->
                    transitiveClosureGrowthSeries.data.add(
                            XYChart.Data(dsd.label(info.fastHeapSupplier.get()!!), dsd.transitiveHeapGrowthPortion()))
                    retainedClosureGrowthSeries.data.add(
                            XYChart.Data(dsd.label(info.fastHeapSupplier.get()!!), dsd.retainedHeapGrowthPortion()))
                    dataStructureClosureGrowthSeries.data.add(
                            XYChart.Data(dsd.label(info.fastHeapSupplier.get()!!), dsd.dataStructureHeapGrowthPortion()))
                    deepDataStructureClosureGrowthSeries.data.add(
                            XYChart.Data(dsd.label(info.fastHeapSupplier.get()!!), dsd.deepDataStructureHeapGrowthPortion()))
                }

        return FXCollections.observableArrayList(transitiveClosureGrowthSeries,
                                                 retainedClosureGrowthSeries,
                                                 dataStructureClosureGrowthSeries,
                                                 deepDataStructureClosureGrowthSeries)
    }

    override fun suceeded() {
        try {
            val series = get()
            tab?.updateChart(series)
        } catch (e: InterruptedException) {
            e.printStackTrace()
        } catch (e: ExecutionException) {
            e.printStackTrace()
        }
    }

    companion object {
        var N_BARS = 10
    }
}
